@@ -1,5 +1,5 @@
-/* dclnt.c */
-/* Last edited by Mike Spreitzer July 28, 1995 4:50 pm PDT */
+/* $Id: dclnt.c,v 1.5 1996/04/23 21:49:16 spreitze Exp $ */
+/* Last edited by Mike Spreitzer April 23, 1996 2:45 pm PDT */
 
 #include <stdio.h>
 
@@ -18,7 +18,6 @@ int
 main(int argc, char *argv[])
 {
   Dialog_T        t;
-  char            sbh[250];
   ILU_C_ENVIRONMENT env;
   ilu_refany      timeout_alarm;
   ilu_FineTime    now, dt, timeout;
