@@ -1,5 +1,5 @@
-/* dsrv.c */
-/* Last edited by Mike Spreitzer July 17, 1995 1:50 pm PDT */
+/* $Id: dsrv.c,v 1.4 1996/05/04 05:17:43 janssen Exp $ */
+/* Last edited by Mike Spreitzer April 23, 1996 2:43 pm PDT */
 
 #include <stdio.h>
 
@@ -30,11 +30,31 @@ server_Dialog_T_M(Dialog_T t, Dialog_String s1, ILU_C_ENVIRONMENT * Env)
 int
 main(int argc, char **argv)
 {
+  char           *pinfo = NULL;
+  ilu_string      tinfo_space[10] = {"sunrpcrm", "tcp_0_0", ILU_NIL};
+  ilu_string     *tinfo = NULL;
   ilu_Server      s;
   Dialog_T        t;
   ilu_string      sbh, mstid;
+  int             i = 1;
+  while (i < argc) {
+    if (strcmp(argv[i], "-p") == 0) {
+      if (i++ < argc)
+	pinfo = argv[i++];
+      else
+	goto usage;
+    } else if (strcmp(argv[i], "-t") == 0) {
+      int             j = 0;
+      tinfo = tinfo_space;
+      ++i;
+      while ((i < argc) && (j < 9) && (argv[i][0] != '-'))
+	tinfo[j++] = argv[i++];
+    } else
+      goto usage;
+  }
   Dialog__InitializeServer();
-  s = ILU_C_InitializeServer("DialogTest-Server", NULL, NULL, NULL, ilu_TRUE);
+  s = ILU_C_InitializeServer("DialogTest-Server", NULL, pinfo, tinfo,
+			     NULL, ilu_TRUE);
   if (s == NULL) {
     fprintf(stderr, "Unable to create ILU server!\n");
     exit(1);
@@ -47,4 +67,8 @@ main(int argc, char **argv)
   sbh = ILU_C_SBHOfObject(t);
   printf("SBH = '%s'\n", sbh);
   ILU_C_Run();
+usage:
+  fprintf(stderr, "Usage: %s [-t tinfo [tinfo...]] [-p pinfo]\n",
+	  argv[0]);
+  return 2;
 }
