@@ -22,13 +22,8 @@
 #define _undefined_addport
 #endif /* AddPort */
 
-#ifdef WIN16
-#include "t1hdr.hh"
-#include "t3hdr.hh" 
-#else
 #include "Test1.hh"
 #include "Test3.hh" 
-#endif /* WIN16 */
 
 #ifdef _undefined_addport
 /* If AddPort was undeffed, redefine it as is done in winspool.h */
