@@ -14,8 +14,8 @@
 ** CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 ** XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: ilu.c,v 1.97 1996/03/13 19:57:35 janssen Exp $ */
-/* Last tweaked by Mike Spreitzer March 8, 1996 11:46 am PST */
+/* $Id: ilu.c,v 1.126 1996/07/12 18:01:57 larner Exp $ */
+/* Last tweaked by Mike Spreitzer July 10, 1996 11:42 pm PDT */
 
 #include <stdio.h>
 
@@ -26,12 +26,12 @@
 #endif /* ((defined(WIN32) || defined(WIN16)) && defined(_WINIO)) */
 
 #include <string.h>
-#if (defined(WIN32) || defined(WIN16))
-// dll - added to get declaration for exit(int)
-#include <stdlib.h>
-#endif
+#include <stdlib.h>		/* for exit */
 
 #include "iluchdrs.h"
+
+#include <oscalls.h>		/* for OS_SLEEP */
+#include <iludebug.h>		/* for _ilu_DebugLevel and LSR_DEBUG */
  
 #define  TRUE  1
 #define  FALSE 0
@@ -51,12 +51,12 @@
 
 /* L1, L2, Main unconstrained */
 
-static ilu_boolean ILU_C_Logging = ilu_FALSE;
+#define ILU_C_Logging (_ilu_DebugLevel & LSR_DEBUG)
 
 static ilu_boolean c_registered = FALSE;
 static ilu_cardinal c_lang = 47;
 
-static ilu_Server defaultServer = NULL;
+static ilu_Server defaultServer = ILU_NIL;
 
 static ilu_Server GetDefaultServer(void);
 
@@ -200,19 +200,19 @@ ilu_CString ILU_C_SBHOfObject (ILU_C_Object *obj)
 /* Main Invariant holds */
 ilu_CString ILU_C_IOROfObject (ILU_C_Object *obj)
 {
-  ilu_Error err;
-  ilu_CString sbh = ILU_NIL;
+  ilu_Error       err;
+  ilu_CString     sbh = ILU_NIL;
 
-  ilu_Object kobj = _ILU_C_KernelObjOfObj (obj);
-  sbh = ilu_IOROfObject (kobj, &err);
-  if (ILU_ERRNOK(err))
-    {
-      ilu_DebugPrintf ("ILU_C_IOROfObject:  Error <%s> in creating IOR.\n", ILU_ERR_NAME(err));
-      ILU_HANDLED(err);
-      sbh = ILU_NIL;
-    }
+  ilu_Object      kobj = _ILU_C_KernelObjOfObj(obj);
+  sbh = ilu_IOROfObject(kobj, &err);
+  if (ILU_ERRNOK(err)) {
+    ilu_DebugPrintf("ILU_C_IOROfObject:  Error <%s> in creating IOR.\n",
+		    ILU_ERR_NAME(err));
+    ILU_HANDLED(err);
+    sbh = ILU_NIL;
+  }
   if (kobj != ILU_NIL)
-    ilu_ExitServer (obj->server, MOST_SPECIFIC_ILU_CLASS(obj));
+    ilu_ExitServer(obj->server, MOST_SPECIFIC_ILU_CLASS(obj));
   return (sbh);
 }
 
@@ -269,7 +269,7 @@ ILU_C_PingObject(ILU_C_Object * obj, ILU_C_ENVIRONMENT * env)
     }
   } ILU_ERR_ENDSWITCH;
   ok = ilu_PingObject(kobj, &newconn);
-  if (newconn != NULL)
+  if (newconn != ILU_NIL)
     (*Fork) (MonitorOutgoingConnection, newconn);
   ilu_EnterServer(obj->server, mst);
   err = ilu_DeltaHolds(kobj, -1);
@@ -492,6 +492,83 @@ ILU_C_ReLookupObject(RETAIN(char *) sid, RETAIN(char *) ih,
   return (h);
 }
 
+/*L1, L2, Main unconstrained*/
+
+PASS(ilu_Passport)
+ILU_C_CreatePassport (OPTIONAL(PASS(ilu_IdentityInfo)) info,
+		      ILU_ERRS((no_memory)) *err)
+{
+  return ilu_CreatePassport(info, err);
+}
+
+PASS(ilu_IdentityInfo)
+ILU_C_CopyIdentity (RETAIN(const struct _ilu_IdentityInfo_s *) info,
+		    ILU_ERRS((no_memory)) *err)
+{
+  return ilu_CopyIdentity(info, err);
+}
+
+ilu_boolean
+ILU_C_AddIdentity (RETAIN(ilu_Passport) pp,
+		   PASS(const struct _ilu_IdentityInfo_s *) info,
+		   ilu_Error *err)
+{
+  return ilu_AddIdentity (pp, info, err);
+}
+
+OPTIONAL(RETAIN(ilu_IdentityInfo))
+ILU_C_FindIdentity (RETAIN(ilu_Passport) pp,
+		    ilu_IdentityType type)
+{
+  return ilu_FindIdentity (pp, type);
+}
+
+ilu_boolean
+ILU_C_DestroyPassport (PASS(ilu_Passport) pp,
+		       ilu_Error *err)
+{
+  return ilu_DestroyPassport(pp, err);
+}
+
+#ifdef SECURE_TRANSPORT
+
+ilu_IdentityInfo
+ILU_C_AcquireGSSIdentity (gss_cred_id_t cred, ilu_Error *err)
+{
+  return ilu_AcquireGSSIdentity (cred, err);
+}
+
+ilu_boolean
+ILU_C_DecodeGSSIdentity (ilu_IdentityInfo info,	/* input; retain; info to decode */
+			 gss_name_t * name,	/* output; name in identity */
+			 ilu_FineTime *till,	/* output; good-till; seconds past Unix epoch */
+			 gss_OID mech,		/* input; actual mechanism desired; optional */
+			 ilu_boolean *local,	/* if TRUE, local; otherwise remote */
+			 ilu_cardinal *flags,	/* connection flags, as in gss_inquire_context */
+			 ilu_Error *err)
+{
+  return ilu_DecodeGSSIdentity (info, name, till, mech, local, flags, err);
+}
+
+gss_cred_id_t
+ILU_C_AcquireGSSCredForName (char *name,		/* name */
+			     ilu_cardinal lifetime,	/* lifetime */
+			     gss_OID mech,		/* secmech */
+			     ilu_boolean accept_only,	/* accept_only */
+			     ilu_Error *err		/* err */)
+{
+  return ilu_AcquireGSSCredForName (name, lifetime, mech, accept_only, err);
+}
+
+PASS(ilu_string)
+ILU_C_GSSNameToString (gss_name_t name,
+		       ilu_Error *err)
+{
+  return ilu_GSSNameToString (name, err);
+}
+
+#endif /* def SECURE_TRANSPORT */
+
 /**before: Main Invariant, Call-Hi(call);
     after: Call-Invariant(call, err) && call->ca_ms == ilu_cmsHi*/
 ILU_C_Object   *
@@ -545,7 +622,8 @@ static ilu_Object
 
 static void _ILU_C_OT_FreeSelf (ilu_ObjectTable self)
 {
-  (*((ILU_C_ObjectTable)(self->ot_rock))->ot_free_ot)(((ILU_C_ObjectTable)(self->ot_rock))->ot_user_data);
+  (*((ILU_C_ObjectTable)(self->ot_rock))->ot_free_ot)
+  (((ILU_C_ObjectTable)(self->ot_rock))->ot_user_data);
   ilu_free(self->ot_rock);
 }
 
@@ -557,8 +635,8 @@ ILU_C_ObjectTable
 			   void (*free_ot) (ilu_private /* user_data */),
 			   ilu_private user_data)
 {
-  ILU_C_ObjectTable cot = ilu_malloc(sizeof(struct _ILU_C_ObjectTable_struct));
-  ilu_ObjectTable ot = ilu_malloc(sizeof(struct ilu_ObjectTable_struct));
+  ILU_C_ObjectTable cot = ilu_malloc(sizeof(*cot));
+  ilu_ObjectTable ot = ilu_malloc(sizeof(*ot));
   cot->ot_object_of_ih = object_of_ih;
   cot->ot_free_ot = free_ot;
   cot->ot_user_data = user_data;
@@ -572,7 +650,8 @@ ILU_C_ObjectTable
 /* locking unconstrained */
 void *_ILU_C_MallocFailure (ilu_cardinal nbytes)
 {
-  ilu_DebugPrintf ("*** ilu_malloc(%u) failed.  Returning NIL.\n", nbytes);
+  ilu_DebugPrintf("*** ilu_malloc(%u) failed.  Returning NIL.\n",
+		  nbytes);
   return ILU_NIL;
 }
 
@@ -581,12 +660,11 @@ ilu_CString ILU_C_Strdup (ilu_CString s)
 {
   if (s == ILU_NIL)
     return (ILU_NIL);
-  else
-    {
-      ilu_CString s2 = ilu_must_malloc(strlen(s) + 1);
-      strcpy (s2, s);
-      return(s2);
-    }
+  else {
+    ilu_CString     s2 = ilu_must_malloc(strlen(s) + 1);
+    strcpy(s2, s);
+    return (s2);
+  }
 }
 
 /* L1, L2, Main unconstrained */
@@ -606,6 +684,7 @@ ILU_C_Object *
 {
   ILU_C_Object   *h = ILU_NIL;
   ilu_Object      obj;
+  ilu_Server s;
   ILU_ERRS((bad_locks, broken_locks, inv_objref,
 	    no_memory, internal)) lerr;
   env->_major = ILU_C_NO_EXCEPTION;
@@ -787,12 +866,54 @@ ilu_boolean     ILU_C_SetFork(void (*fork) (void (*proc) (void *arg),
   threaded = ilu_TRUE;
   (void) MyLangIdx();
   (void) GetDefaultServer();
-  (*fork)(PassNewConnections, NULL);
+  (*fork)(PassNewConnections, ILU_NIL);
   (void) ilu_NewConnectionGetterForked(&lerr);
   ILU_MUST_BE_SUCCESS(lerr);
   return (ilu_TRUE);
 }
 
+/*
+ * We don't want to mention ilu_OSForkNewThread() or
+ * ilu_InitializeOSThreading explicitly, in case the application
+ * isn't using them, to avoid linking that object module
+ * unnecessarily.  So we indirect through KernelForkProc. If the
+ * application code uses ILU_C_USE_OS_THREADS, the right things are
+ * automatically invoked.
+ */
+
+static ILU_C_ErrableForkProc errableFork = 0;
+
+static void ErrlessFork(ILU_C_WorkProc threadproc, void *threadarg)
+{
+  ILU_ERRS((no_memory, no_resources, internal)) err;
+  (void) ((*errableFork) (threadproc, threadarg, &err));
+  ILU_MUST_BE_SUCCESS(err);
+}
+
+/*Main invariant holds*/
+ilu_boolean
+ILU_C_EnableThreads(ILU_C_ThreadSetupProc s,
+		    ILU_C_ErrableForkProc f)
+{
+  ILU_ERRS((bad_param, no_memory, no_resources,
+	    internal)) err;
+
+  if (s == 0 || f == 0) {
+    ilu_DebugPrintf("ilu.c:ILU_C_EnableThreads given silly arguments!\n");
+    return ilu_FALSE;
+  }
+  if (!(*s) (&err)) {
+    ilu_DebugPrintf("ILU_C_EnableThreads:  error %s (raised at line %d of %s) attempting to set ILU kernel multi-threaded!\n",
+		    ILU_ERR_NAME(err), ilu_ErrorLine(&err),
+		    ilu_ErrorFile(&err));
+    ILU_HANDLED(err);
+    return ilu_FALSE;
+  }
+  errableFork = f;
+  ILU_C_SetFork(ErrlessFork);
+  return ilu_TRUE;
+}
+
 /*Main invariant holds */
 static void MonitorOutgoingConnection(void *rock)
 {
@@ -809,7 +930,7 @@ static void PassNewConnections(void *rock)
   ILU_ERRS((internal)) lerr;
   while (1) {
     nu = ilu_OtherNewConnection(&lerr);
-    if (nu != NULL)
+    if (nu != ILU_NIL)
       (*Fork) (MonitorOutgoingConnection, nu);
     ILU_MUST_BE_SUCCESS(lerr);
   }
@@ -824,12 +945,12 @@ static void PassNewConnections(void *rock)
 ilu_Server
 ILU_C_InitializeServer(char *serverID, ILU_C_ObjectTable obj_tab,
 		       char *protocol, ilu_TransportInfo transport,
-		       ilu_boolean setdefaultport)
+		       ilu_Passport pp, ilu_boolean setdefaultport)
 {
   ilu_Server      server;
   ilu_Port        port;
   ilu_boolean     closed = 0;
-  int             fd;
+  ilu_Error	 err;
   char           *sid;
   char           *pinfo = ((protocol == ILU_NIL) ? DefaultProtocol
 			   : protocol);
@@ -840,13 +961,15 @@ ILU_C_InitializeServer(char *serverID, ILU_C_ObjectTable obj_tab,
     sid = ilu_InventID();
   else
     sid = ILU_C_Strdup(serverID);
-  if (sid == NULL) {
+  if (sid == ILU_NIL) {
     ilu_DebugPrintf (
        "ilu.c:  Unable to allocate for copy of server ID \"%s\"\n",
 	    serverID ? serverID : "(unknown gensymed ID)");
     exit(1);
   }
-  server = ilu_CreateTrueServer(sid, (obj_tab == ILU_NIL) ? ILU_NIL : obj_tab->ot_kernel_ot, MyLangIdx());
+  server = ilu_CreateTrueServer(sid,
+	    (obj_tab == ILU_NIL) ? ILU_NIL : obj_tab->ot_kernel_ot,
+				MyLangIdx());
   if (server == ILU_NIL) {
     ilu_DebugPrintf (
 	"ilu.c:  Couldn't create true server from (\"%s\", %p).\n",
@@ -854,12 +977,13 @@ ILU_C_InitializeServer(char *serverID, ILU_C_ObjectTable obj_tab,
     exit(1);
   }
   if (protocol != ILU_NIL OR transport != ILU_NIL OR setdefaultport) {
-    port = ilu_CreatePort(server, pinfo, tinfo);
+    port = ilu_CreatePort(server, pinfo, tinfo, pp, &err);
     if (port == ILU_NIL) {
       char **ti;
       ilu_DebugPrintf (
-	      "ilu.c:  Couldn't open Port to export object with protocol = \"%s\" and transport =",
-	      pinfo);
+	      "ilu.c:  Error <%s> opening port with protocol = \"%s\" and transport =",
+	      ILU_ERR_NAME(err), pinfo);
+      ILU_HANDLED(err);
       for (ti = tinfo;  *ti != ILU_NIL;  ti++)
 	ilu_DebugPrintf (" \"%s\"", *ti);
       ilu_DebugPrintf (".\n");
@@ -887,11 +1011,12 @@ ILU_C_InitializeServer(char *serverID, ILU_C_ObjectTable obj_tab,
       }
     }
   }
-  port = ilu_CreatePort(server, DefaultProtocol, inmemTinfo);
-  if (port == NULL) {
+  port = ilu_CreatePort(server, DefaultProtocol, inmemTinfo, pp, &err);
+  if (port == ILU_NIL) {
     ilu_DebugPrintf (
-	    "ilu.c: Couldn't create local port (protocol=\"%s\", tinfo=\"%s\")\n",
-	    DefaultProtocol, inmemTinfo[0]);
+	    "ilu.c:  Error <%s> creating local port (protocol=\"%s\", tinfo=\"%s\")\n",
+	    ILU_ERR_NAME(err), DefaultProtocol, inmemTinfo[0]);
+    ILU_HANDLED(err);
   } else {
     ILU_ERRS((no_memory, imp_limit,
 	      no_resources, bad_param,
@@ -917,11 +1042,12 @@ ILU_C_InitializeServer(char *serverID, ILU_C_ObjectTable obj_tab,
 /* Main invariant holds */
 static ilu_Server GetDefaultServer(void)
 {
-  if (defaultServer != NULL)
+  if (defaultServer != ILU_NIL)
     return defaultServer;
-  defaultServer = ILU_C_InitializeServer(NULL, NULL, NULL, NULL, TRUE);
-  if (defaultServer == NULL) {
-    ilu_DebugPrintf ("ilu.c:  Error:  Couldn't create default server.\n");
+  defaultServer = ILU_C_InitializeServer(ILU_NIL, ILU_NIL, ILU_NIL,
+					 ILU_NIL, ILU_NIL, TRUE);
+  if (defaultServer == ILU_NIL) {
+    ilu_DebugPrintf("ilu.c:  Error:  Couldn't create default server.\n");
     exit(1);
   }
   return defaultServer;
@@ -938,6 +1064,10 @@ static void _ILU_C_RunConnection(void *arg)
       break;
     closed = _ILU_C_FinalServiceRequest(conn, ilu_FALSE);
   }
+  if (ILU_C_Logging)
+    ilu_DebugPrintf(
+		 "Connection %p no longer waiting for requests.\n",
+		    conn);
   ilu_CloseConnection(conn);
 }
 
@@ -947,25 +1077,28 @@ extern void _ilu_WaitForInputOnFD (int, ilu_boolean *, void *);
 static void _ILU_C_ReadConnectionRequests(void *arg)
 {
   ilu_Port        p = (ilu_Port) arg;
-  ilu_boolean     sure, closed = 0;
+  ilu_boolean     closed = 0;
   ilu_Connection  conn;
 
-  while (NOT closed) {
+  while (1) {
     if (ILU_C_Logging)
-      printf("Mooring %p :  ", p);
+      ilu_DebugPrintf("Port %p waiting for connection request.\n", p);
     ilu_WaitForPortConnectionRequest(p);
-    if ((conn = ilu_HandleNewConnection(p, &closed)) == ILU_NIL) {
+    conn = ilu_HandleNewConnection(p, &closed);
+    if (closed)
+      break;
+    if (conn == ILU_NIL) {
       if (ILU_C_Logging)
-	printf("Unable to accept connection.\n");
-      else
-	ilu_DebugPrintf (
-		"ilu.c unable to accept connection.\n");
-      return;
+	ilu_DebugPrintf("Port %p connection request declined.\n", p);
     } else {
       /* fork thread to run new connection */
       Fork(_ILU_C_RunConnection, conn);
     }
   }
+  if (ILU_C_Logging)
+    ilu_DebugPrintf(
+	    "Port %p no longer waiting for connection requests.\n",
+		    p);
 }
 
 /*L1, L2, Main unconstrained*/
@@ -980,15 +1113,15 @@ static ilu_boolean EnableRequests(ilu_Connection conn, ilu_Call call)
 				      conn, &lerr);
   if (!ans) {
     if (ILU_C_Logging)
-      printf("ilu.c:  Can't register input handler of conn %p server %s!\nError = %s\n",
+      printf("ilu.c:  Can't register request handler for conn %p server %s!\nError = %s\n",
 	     conn, sid, ILU_ERR_NAME(lerr));
     else
       ilu_DebugPrintf(
-	    "ilu.c:  Can't register input handler of conn %p server %s!\nError = %s\n",
+	    "ilu.c:  Can't register request handler for conn %p server %s!\nError = %s\n",
 	    conn, sid, ILU_ERR_NAME(lerr));
     ILU_HANDLED(lerr);
   } else if (ILU_C_Logging)
-    printf("Connection %p server %s handling input.\n",
+    printf("Connection %p server %s awaiting requests.\n",
 	   conn, sid);
   return ans;
 }
@@ -1001,17 +1134,17 @@ static ilu_boolean DisableRequests(ilu_Connection conn, ilu_Call call)
   ilu_boolean     ans;
   ILU_ERRS((no_memory, no_resources)) lerr;
   call->ca_reqs_enabled = ilu_FALSE;
-  ans = ilu_SetConnectionInputHandler(conn, NULL, NULL, &lerr);
+  ans = ilu_SetConnectionInputHandler(conn, NULL, ILU_NIL, &lerr);
   if (!ans) {
     if (ILU_C_Logging)
-      printf("ilu.c:  failure (%s) detected when unregistering input source for conn %p server %s!\n",
+      printf("ilu.c:  failure (%s) detected when unregistering request handler for conn %p server %s!\n",
 	     ILU_ERR_NAME(lerr), conn, sid);
     else
-      ilu_DebugPrintf("ilu.c:  failure (%s) detected when unregistering input source for conn %p server %s!\n",
+      ilu_DebugPrintf("ilu.c:  failure (%s) detected when unregistering request handler for conn %p server %s!\n",
 		      ILU_ERR_NAME(lerr), conn, sid);
     ILU_HANDLED(lerr);
   } else if (ILU_C_Logging)
-    printf("Connection %p server %s ignoring input.\n",
+    printf("Connection %p server %s not awaiting requests.\n",
 	   conn, sid);
   return ans;
 }
@@ -1020,27 +1153,40 @@ static ilu_boolean DisableRequests(ilu_Connection conn, ilu_Call call)
 static void
 _ILU_C_ReadConnectionRequest(ilu_private arg)
 {
-  ilu_Port        p;
-  int             fd2;
+  ilu_Port        p = (ilu_Port) arg;
   ilu_boolean     closed = 0;
   ilu_Connection  conn;
   ilu_Call_s      nocall;
-
-  p = (ilu_Port) arg;
   if (ILU_C_Logging)
-    printf("Mooring %p:  ", p);
-  if ((conn = ilu_HandleNewConnection(p, &closed)) == ILU_NIL) {
+    printf("Port %p handling connection request.\n", p);
+  conn = ilu_HandleNewConnection(p, &closed);
+  if (closed) {
+    ilu_DebugPrintf("Port %p closed.\n", p);
+  } else if (conn == ILU_NIL) {
     if (ILU_C_Logging)
-      printf("Unable to accept connection.\n");
-    else
-      ilu_DebugPrintf("ilu.c was unable to accept connection.\n");
-    return;
+      ilu_DebugPrintf("Port %p connection request declined.\n", p);
   } else
     EnableRequests(conn, &nocall);
 }
 
 typedef void (*cstubproc) (ilu_Call, ilu_Error *);
 
+typedef struct {
+  ilu_Call        call;
+  ilu_Method      method;
+}               C_Invokn;
+
+/** Main;
+    before: L2    >=    {call's conn's callmu, iomu};
+    after:  L2 disjoint {call's conn's callmu, iomu} */
+static void DoInvocation(void *arg)
+{
+  C_Invokn       *ci = (C_Invokn *) arg;
+  ILU_ERRS((bad_locks, IoErrs)) lerr;
+  (*((cstubproc) ci->method->me_stubproc)) (ci->call, &lerr);
+  ILU_MUST_BE_SUCCESS(lerr);	/* should hand off to app */
+}
+
 /* Returns TRUE iff connection closed. */
 /*Main Invariant holds
   before: L2 disjoint {arg's callmu, iomu}
@@ -1051,25 +1197,36 @@ static          ilu_boolean
 _ILU_C_FinalServiceRequest(ilu_private arg,
 			   ilu_boolean single_threaded)
 {
-  ilu_Call_s      call;
+  ilu_Call_s      call_s, *call;
   ilu_boolean     initted;
   ilu_Class       class;
   ilu_Connection  conn = (ilu_Connection) arg;
   ilu_Method      method;
   ilu_cardinal    SN;
   ilu_RcvReqStat  stat;
-  ILU_ERRS((bad_locks, IoErrs)) lerr;
+  ILU_ERRS((bad_locks, IoErrs)) lerr = ILU_INIT_NO_ERR;
+  C_Invokn       *ci = ILU_NIL;
 
   if (ILU_C_Logging)
     printf("Activity on connection %p:  ", conn);
-  if (single_threaded)
-    DisableRequests(conn, &call);
-  stat = ilu_ReceiveRequest(&call, &initted, conn, &class, &method,
+  if (single_threaded) {
+    call = &call_s;
+    DisableRequests(conn, call);
+  } else if (ilu_ThreadPerRequest(conn)) {
+    call = ilu_malloc(sizeof(*call));
+    ci = ilu_malloc(sizeof(*ci));
+    if (call == ILU_NIL || ci == ILU_NIL) {	/* out of memory */
+      ilu_CloseConnection(conn);
+      return ilu_TRUE;
+    }
+  } else
+    call = &call_s;
+  stat = ilu_ReceiveRequest(call, &initted, conn, &class, &method,
 			    &SN, &lerr);
   if (ILU_C_Logging) {
     if (stat == ilu_RcvReqStat_request)
       printf("Received request SN %lu, method \"%s\" of class \"%s\"\n",
-	     SN, method->me_name, class->cl_name);
+	     (unsigned long) SN, method->me_name, class->cl_name);
     else
       printf("ReceiveRequest => %s, *initted = %s, *err = %s\n",
 	     ((stat == ilu_RcvReqStat_noop) ? "noop" :
@@ -1080,18 +1237,36 @@ _ILU_C_FinalServiceRequest(ilu_private arg,
   }
   if (stat == ilu_RcvReqStat_request) {
     /* L2 >= {conn's callmu, iomu} */
-    (*((cstubproc) method->me_stubproc)) (&call, &lerr);
-    /* L2 disjoint {conn's callmu, iomu} */
-    /* single_threaded == threaded */
-    /* !threaded <=> _ILU_C_ReadServiceRequest registered */
+    if (ci) {
+      ci->call = call;
+      ci->method = method;
+      (*Fork) (DoInvocation, ci);
+      /* L2 disjoint {conn's callmu, iomu} (handed off to new thd) */
+    } else {
+      (*((cstubproc) method->me_stubproc)) (call, &lerr);
+      /* L2 disjoint {conn's callmu, iomu} */
+      /* single_threaded != threaded */
+      /* !threaded <=> _ILU_C_ReadServiceRequest registered */
+    }
     return (ilu_FALSE);
   } else if (initted) {
-    ilu_FinishCall(&call, &lerr);
+    ilu_FinishCall(call, &lerr);
   }
+
+  ILU_ERR_SWITCH(lerr) {
+
+    ILU_SUCCESS_CASE ;
+
+    ILU_ERR_ELSE
+      ILU_ERRPRINTF("_ILU_C_FinalServiceRequest Warning: error %s from %s line %d\n",
+		ILU_ERR_NAME(lerr), ilu_ErrorFile(&lerr), ilu_ErrorLine(&lerr));
+
+  } ILU_ERR_ENDSWITCH;	/* should hand error off to app */
+
   if (stat == ilu_RcvReqStat_quit)
     return ilu_TRUE;
   if (single_threaded)
-    EnableRequests(conn, &call);
+    EnableRequests(conn, call);
   return ilu_FALSE;
 }
 
@@ -1176,8 +1351,6 @@ _ILU_C_FinishReply(ilu_Call call, ILU_ERRS((bad_locks, IoErrs)) * err)
   ilu_Connection  conn = ilu_ConnectionOfCall(call);
   ilu_boolean     ans;
   ans = ilu_FinishReply(call, err);
-  if (!threaded)
-    EnableRequests(conn, call);
   return ans;
 }
 
@@ -1211,8 +1384,6 @@ _ILU_C_FinishException(ilu_Call call,
   ilu_Connection  conn = ilu_ConnectionOfCall(call);
   ilu_boolean     ans;
   ans = ilu_FinishException(call, err);
-  if (!threaded)
-    EnableRequests(conn, call);
   return ans;
 }
 
@@ -1230,8 +1401,6 @@ _ILU_C_NoReply(ilu_Call call,
   ilu_boolean     ans;
 
   ans = ilu_NoReply(call, err);
-  if ((!threaded) && (!ilu_ThreadPerRequest(conn)))
-    EnableRequests(conn, call);
   return ans;
 }
 
@@ -1247,7 +1416,10 @@ void ILU_C_Run( )
     int		stop = 0;
 
     threadedSet = ilu_TRUE;
-    ilu_RunMainLoop( &stop );
+    if (threaded)
+      while (1) OS_SLEEP(30);
+    else
+      ilu_RunMainLoop( &stop );
 }
 
 /* unrestricted */
@@ -1377,7 +1549,7 @@ _ILU_C_CatchException(ilu_Call call, ilu_Method method,
   } else {
     unsigned int    valsize;
     ilu_Class       valclass;
-    void            (*valfn) (ilu_Call, void *, ilu_Error *);
+    ILU_C_InputFn   valfn;
 
     status->_major = ILU_C_USER_EXCEPTION;
     status->returnCode = ((ILU_C_ExceptionCode)
@@ -1387,19 +1559,17 @@ _ILU_C_CatchException(ilu_Call call, ilu_Method method,
       status->ptr = (void *) ilu_MallocE(valsize, err);
       if (ILU_ERRNOK(*err))
 	return ilu_FALSE;
-      if (evec[exceptionIndex - 1].class != NULL) {
+      if (evec[exceptionIndex - 1].class != ILU_NIL) {
 	valclass = evec[exceptionIndex - 1].class;
 	(status->ptr) = (void *) _ILU_C_InputObject(call, valclass,
 						    ilu_FALSE, err);
       } else {
-	valfn = ((void (*) (ilu_Call, void *, ilu_Error *))
-		 evec[exceptionIndex - 1].fn1);
-	(void) (*valfn) (call, (void *) status->ptr, err);
+	valfn = evec[exceptionIndex - 1].inFn;
+	(void) (*valfn) (call, status->ptr, err);
       }
       if (ILU_ERRNOK(*err))
 	return ilu_FALSE;
-      status->freeRoutine = (void (*) (void *)) evec[exceptionIndex
-						     - 1].fn2;
+      status->freeRoutine = evec[exceptionIndex - 1].freeFn;
     }
   }
   return ilu_TRUE;
@@ -1471,9 +1641,10 @@ static ilu_cardinal _ILU_C_WStrlen(ilu_wstring s)
   p & P:  object
   q & Q:  C string
   r & R:  Unicode string
+  y    :  Out variable-length type
   z & Z:  other
 
-  s-y & S-Y:  reserved for future use
+  s-x & S-Y:  reserved for future use
 
   Values of primitive ISL types are presented on the varargs
   list as appropriate for a value of this type, given the CORBA ANSI C calling
@@ -1494,7 +1665,8 @@ static ilu_cardinal _ILU_C_WStrlen(ilu_wstring s)
   characters representing Out arguments.  For the purposes of this routine,
   the return value of the method is passed and coded as another Out argument.
   The return value argument appears before the actual Out and InOut arguments
-  in the arglist.
+  in the arglist.  The code 'y' is added to represent Out parameters of
+  variable-length types.
 
   Example:
 
@@ -1512,7 +1684,7 @@ static ilu_cardinal _ILU_C_WStrlen(ilu_wstring s)
 
 void _FOO_Output_R (ilu_Call _call, FOO_R * _val, ilu_Error *err)
 {
-  if (_val == NULL)
+  if (_val == ILU_NIL)
     return;
 
   ilu_OutputRecord (_call, err);
@@ -1553,7 +1725,7 @@ FOO_R * _FOO_Input_R (ilu_Call _call, FOO_R * _ref, ilu_Error *err)
   ilu_InputRecord (_call, err);
   if (ILU_ERRNOK(*err)) return _val;
 
-  if (_ref != NULL)
+  if (_ref != ILU_NIL)
     _val = _ref;
   else
     _val = (FOO_R *) ilu_must_malloc (sizeof (FOO_R));
@@ -1599,10 +1771,6 @@ ilu_real _FOO_O4_Real_R_to_Real_R
 
 */
 
-typedef ilu_cardinal (*sizeFn) (ilu_Call, void *, ilu_Error *);
-typedef void (*outputFn) (ilu_Call, void *, ilu_Error *);
-typedef void (*inputFn) (ilu_Call, void *, ilu_Error *);
-
 void
   _ILU_C_GenericCall (ilu_Class class, ilu_Method method,
 		      _ILU_C_ExceptionDescription evec,
@@ -1615,6 +1783,7 @@ void
   ilu_Error       err = ILU_INIT_NO_ERR;
   ilu_cardinal    size = 0, s;
   ilu_boolean     vaing = ilu_FALSE, initted = FALSE;
+  ilu_boolean	needs_sizing;
   ilu_Object      kobj;
   va_list         ap;
   char           *p;
@@ -1623,13 +1792,14 @@ void
   ilu_ProtocolException protocolErr;
   ILU_C_InterruptHandle intH = ILU_NIL;
   ilu_Connection  newconn;
+  enum foo {a, b, c};
 
   ILU_CLER(err);
 
   (void) ilu_StartCall(call, discriminant->server, class, method,
 		     MyLangIdx(), status->callerPassport,
 		     &newconn, &err);
-  if (newconn != NULL)
+  if (newconn != ILU_NIL)
     (*Fork) (MonitorOutgoingConnection, newconn);
   if (ILU_ERRNOK(err))
     goto kerrOhneCall;
@@ -1640,8 +1810,10 @@ void
       goto cerr;
     }
 
+  needs_sizing = ilu_CallNeedsSizing(call);
+
   /* figure size of discriminant */
-  if (_ILU_C_IsSingleton(class))
+  if ((!needs_sizing) || _ILU_C_IsSingleton(class))
     size = 0;
   else {
     if ((kobj = _ILU_C_KernelObjOfObj(discriminant)) == ILU_NIL) {
@@ -1661,30 +1833,30 @@ void
 
 #define GENERIC_SIZEI(type,fn) { \
                                  type *v = va_arg(ap, type *); \
-				 s = fn (call, *v, &err); \
-				 if (s == 0) goto kerr; \
-				 else size += s; \
+				 if (needs_sizing) {s = fn (call, *v, &err); \
+				 if (ILU_ERRNOK(err)) goto kerr; \
+				 else size += s; }\
 			       } \
 			       break
 #define GENERIC_SIZES(type,fn) { \
                                  type v = (type) va_arg(ap, int); \
-				 s = fn (call, v, &err); \
-				 if (s == 0) goto kerr; \
-				 else size += s; \
+				 if (needs_sizing) { s = fn (call, v, &err); \
+				 if (ILU_ERRNOK(err)) goto kerr; \
+				 else size += s; }\
 			       } \
 			       break
 #define GENERIC_SIZER(type,fn) { \
                                  type v = (type) va_arg(ap, double); \
-				 s = fn (call, v, &err); \
-				 if (s == 0) goto kerr; \
-				 else size += s; \
+				 if (needs_sizing) { s = fn (call, v, &err); \
+				 if (ILU_ERRNOK(err)) goto kerr; \
+				 else size += s; }\
 			       } \
 			       break
 #define GENERIC_SIZE(type,fn) { \
                                  type v = va_arg(ap, type); \
-				 s = fn (call, v, &err); \
-				 if (s == 0) goto kerr; \
-				 else size += s; \
+				 if (needs_sizing) { s = fn (call, v, &err); \
+				 if (ILU_ERRNOK(err)) goto kerr; \
+				 else size += s; }\
 			       } \
 			       break
 
@@ -1772,10 +1944,23 @@ void
 
 	  /* Enumeration */
 	case 'o':
-	  GENERIC_SIZES(ilu_shortcardinal, ilu_SizeOfEnum);
+	  {
+	    enum foo v;
+	    v = va_arg(ap, enum foo);
+	    if (needs_sizing) {
+	      size += ilu_SizeOfEnum (call, (ilu_shortcardinal) v, &err);
+	      if (ILU_ERRNOK(err)) goto kerr; };
+	    break;
+	  }
 	case 'O':
-	  GENERIC_SIZEI(ilu_shortcardinal, ilu_SizeOfEnum);
-
+	  {
+	    enum foo *v;
+	    v = va_arg(ap, enum foo *);
+	    if (needs_sizing) {
+	      size += ilu_SizeOfEnum (call, (ilu_shortcardinal) (*v), &err);
+	      if (ILU_ERRNOK(err)) goto kerr; };
+	    break;
+	  }
 	  /* Objects */
 
 	case 'p':
@@ -1785,9 +1970,10 @@ void
 
 	    v = va_arg(ap, ILU_C_Object *);
 	    pclass = va_arg(ap, ilu_Class);
-	    s = _ILU_C_SizeOfObject (call, v, pclass, ilu_FALSE, &err);
-	    if (ILU_ERRNOK(err)) goto kerr;
-	    size += s;
+	    if (needs_sizing) {
+	      s = _ILU_C_SizeOfObject (call, v, pclass, ilu_FALSE, &err);
+	      if (ILU_ERRNOK(err)) goto kerr;
+	      size += s; };
 	    break;
 	  }
 	case 'P':
@@ -1797,9 +1983,11 @@ void
 
 	    v = va_arg(ap, ILU_C_Object **);
 	    pclass = va_arg(ap, ilu_Class);
-	    s = _ILU_C_SizeOfObject (call, *v, pclass, ilu_FALSE, &err);
-	    if (ILU_ERRNOK(err)) goto kerr;
-	    size += s;
+	    if (needs_sizing) {
+	      s = _ILU_C_SizeOfObject (call, *v, pclass, ilu_FALSE, &err);
+	      if (ILU_ERRNOK(err)) goto kerr;
+	      size += s;
+	    };
 	    break;
 	  }
 
@@ -1812,9 +2000,11 @@ void
 
 	    str = va_arg(ap, ilu_string);
 	    limit = va_arg(ap, ilu_cardinal);
-	    s = ilu_SizeOfString (call, str, strlen(str), limit, &err);
-	    if (ILU_ERRNOK(err)) goto kerr;
-	    size += s;
+	    if (needs_sizing) {
+	      s = ilu_SizeOfString (call, str, strlen(str), limit, &err);
+	      if (ILU_ERRNOK(err)) goto kerr;
+	      size += s;
+	    };
 	    break;
 	  }
 	case 'Q':
@@ -1824,9 +2014,11 @@ void
 
 	    str = va_arg(ap, ilu_string *);
 	    limit = va_arg(ap, ilu_cardinal);
-	    s = ilu_SizeOfString (call, *str, strlen(*str), limit, &err);
-	    if (ILU_ERRNOK(err)) goto kerr;
-	    size += s;
+	    if (needs_sizing) {
+	      s = ilu_SizeOfString (call, *str, strlen(*str), limit, &err);
+	      if (ILU_ERRNOK(err)) goto kerr;
+	      size += s;
+	    };
 	    break;
 	  }
 
@@ -1839,9 +2031,11 @@ void
 
 	    str = va_arg(ap, ilu_wstring);
 	    limit = va_arg(ap, ilu_cardinal);
-	    s = ilu_SizeOfWString (call, str, _ILU_C_WStrlen(str), limit, ILU_NIL, 0, &err);
-	    if (ILU_ERRNOK(err)) goto kerr;
-	    size += s;
+	    if (needs_sizing) {
+	      s = ilu_SizeOfWString (call, str, _ILU_C_WStrlen(str), limit, &err);
+	      if (ILU_ERRNOK(err)) goto kerr;
+	      size += s;
+	    };
 	    break;
 	  }
 	case 'R':
@@ -1851,23 +2045,26 @@ void
 
 	    str = va_arg(ap, ilu_wstring *);
 	    limit = va_arg(ap, ilu_cardinal);
-	    s = ilu_SizeOfWString (call, *str, _ILU_C_WStrlen(*str), limit, ILU_NIL, 0, &err);
-	    if (ILU_ERRNOK(err)) goto kerr;
-	    size += s;
+	    if (needs_sizing) {
+	      s = ilu_SizeOfWString (call, *str, _ILU_C_WStrlen(*str), limit, &err);
+	      if (ILU_ERRNOK(err)) goto kerr;
+	      size += s;
+	    }
 	    break;
 	  }
 
 	case 'z':
 	case 'Z':
 	  {
-	    void *v;
-	    sizeFn fp;
+	    void           *v;
+	    ILU_C_SizeFn    fp;
 
 	    v = va_arg(ap, void *);
-	    fp = va_arg(ap, sizeFn);
-
-	    s = (*fp) (call, v, &err);
-	    if (s == 0) goto kerr; else size += s;
+	    fp = va_arg(ap, ILU_C_SizeFn);
+	    if (needs_sizing) {
+	      s = (*fp) (call, v, &err);
+	      if (ILU_ERRNOK(err)) goto kerr; else size += s;
+	    };
 	    break;
 	  }
 
@@ -2009,9 +2206,21 @@ void
 	  /* Enumeration */
 
 	case 'o':
-	  GENERIC_OUTPUTS(ilu_shortcardinal, ilu_OutputEnum);
+	  {
+	    enum foo v;
+	    v = va_arg(ap, enum foo);
+	    ilu_OutputEnum (call, (ilu_shortcardinal) v, &err);
+	    if (ILU_ERRNOK(err)) goto kerr;
+	    break;
+	  }
 	case 'O':
-	  GENERIC_OUTPUTI(ilu_shortcardinal, ilu_OutputEnum);
+	  {
+	    enum foo *v;
+	    v = va_arg(ap, enum foo *);
+	    ilu_OutputEnum (call, (ilu_shortcardinal) (*v), &err);
+	    if (ILU_ERRNOK(err)) goto kerr;
+	    break;
+	  }
 
 	case 'p':
 	  {
@@ -2070,7 +2279,7 @@ void
 
 	    str = va_arg(ap, ilu_wstring);
 	    limit = va_arg(ap, ilu_cardinal);
-	    ilu_OutputWString (call, str, _ILU_C_WStrlen(str), limit, ILU_NIL, 0, &err);
+	    ilu_OutputWString (call, str, _ILU_C_WStrlen(str), limit, &err);
 	    if (ILU_ERRNOK(err)) goto kerr;
 	    break;
 	  }
@@ -2081,7 +2290,7 @@ void
 
 	    str = va_arg(ap, ilu_wstring *);
 	    limit = va_arg(ap, ilu_cardinal);
-	    ilu_OutputWString (call, *str, _ILU_C_WStrlen(*str), limit, ILU_NIL, 0, &err);
+	    ilu_OutputWString (call, *str, _ILU_C_WStrlen(*str), limit, &err);
 	    if (ILU_ERRNOK(err)) goto kerr;
 	    break;
 	  }
@@ -2089,11 +2298,11 @@ void
 	case 'z':
 	case 'Z':
 	  {
-	    void *v;
-	    outputFn fp;
+	    void           *v;
+	    ILU_C_OutputFn  fp;
 
 	    v = va_arg(ap, void *);
-	    fp = va_arg(ap, outputFn);
+	    fp = va_arg(ap, ILU_C_OutputFn);
 
 	    (*fp) (call, v, &err);
 	    if (ILU_ERRNOK(err)) goto kerr;
@@ -2211,7 +2420,15 @@ void
 
 		    case 'o':
 		    case 'O':
-		      GENERIC_INPUT(ilu_shortcardinal, ilu_InputEnum);
+		      {
+			ilu_shortcardinal val;
+			enum foo *v;
+			v = (enum foo *) va_arg(ap, enum foo *);
+			ilu_InputEnum (call, &val, &err);
+			if (ILU_ERRNOK(err)) goto kerr;
+			*v = (enum foo) val;
+			break;
+		      }
 
 		      /* Object */
 
@@ -2292,16 +2509,29 @@ void
 
 		      /* other constructed type */
 
+		    case 'y':
+		      {
+			void ** v;
+			ILU_C_InputFn fp;
+
+			v = va_arg(ap, void **);
+			fp = va_arg(ap, ILU_C_InputFn);
+
+			*v = (*fp) (call, ILU_NIL, &err);
+			if (ILU_ERRNOK(err)) goto kerr;
+			break;
+		      }
+
 		    case 'z':
 		    case 'Z':
 		      {
-			void ** v;
-			inputFn fp;
+			void * v;
+			ILU_C_InputFn fp;
 
-			v = va_arg(ap, void **);
-			fp = va_arg(ap, inputFn);
+			v = va_arg(ap, void *);
+			fp = va_arg(ap, ILU_C_InputFn);
 
-			(*fp) (call, v, &err);
+			(void) (*fp) (call, v, &err);
 			if (ILU_ERRNOK(err)) goto kerr;
 			break;
 		      }
@@ -2378,7 +2608,7 @@ _ILU_C_SendException(ilu_Call call, _ILU_C_ExceptionDescription evec,
 
   ILU_CLER(*err);
 
-  if (status->returnCode == NULL)
+  if (status->returnCode == ILU_NIL)
     return;
 
   limit = method->me_exceptionCount;
@@ -2394,83 +2624,85 @@ found:
 
 #define GENERIC_ESIZE(type,fn) argSize = fn (call, *((type *) status->ptr), err); break
 
-  argSize = ilu_BeginSizingException(call, eindex+1, err);
-  if (ILU_ERRNOK(*err))
-    return;
+  if (ilu_CallNeedsSizing(call)) {
+    argSize = ilu_BeginSizingException(call, eindex+1, err);
+    if (ILU_ERRNOK(*err))
+      return;
   
-  switch (evec[eindex].size)
-    {
-      /* Integers */
+    switch (evec[eindex].size)
+      {
+	/* Integers */
 
-    case 'a':
-      GENERIC_ESIZE(ilu_shortinteger, ilu_SizeOfShortInteger);
-    case 'b':
-      GENERIC_ESIZE(ilu_integer, ilu_SizeOfInteger);
-    case 'c':
-      GENERIC_ESIZE(ilu_longinteger, ilu_SizeOfLongInteger);
+      case 'a':
+	GENERIC_ESIZE(ilu_shortinteger, ilu_SizeOfShortInteger);
+      case 'b':
+	GENERIC_ESIZE(ilu_integer, ilu_SizeOfInteger);
+      case 'c':
+	GENERIC_ESIZE(ilu_longinteger, ilu_SizeOfLongInteger);
 
-      /* Cardinals */
+	/* Cardinals */
 
-    case 'd':
-      GENERIC_ESIZE(ilu_shortcardinal, ilu_SizeOfShortCardinal);
-    case 'e':
-      GENERIC_ESIZE(ilu_cardinal, ilu_SizeOfCardinal);
-    case 'f':
-      GENERIC_ESIZE(ilu_longcardinal, ilu_SizeOfLongCardinal);
+      case 'd':
+	GENERIC_ESIZE(ilu_shortcardinal, ilu_SizeOfShortCardinal);
+      case 'e':
+	GENERIC_ESIZE(ilu_cardinal, ilu_SizeOfCardinal);
+      case 'f':
+	GENERIC_ESIZE(ilu_longcardinal, ilu_SizeOfLongCardinal);
 
-      /* Reals */
+	/* Reals */
 
-    case 'g':
-      GENERIC_ESIZE(ilu_shortreal, ilu_SizeOfShortReal);
-    case 'h':
-      GENERIC_ESIZE(ilu_real, ilu_SizeOfReal);
-    case 'i':
-      GENERIC_ESIZE(ilu_longreal, ilu_SizeOfLongReal);
+      case 'g':
+	GENERIC_ESIZE(ilu_shortreal, ilu_SizeOfShortReal);
+      case 'h':
+	GENERIC_ESIZE(ilu_real, ilu_SizeOfReal);
+      case 'i':
+	GENERIC_ESIZE(ilu_longreal, ilu_SizeOfLongReal);
 
-      /* Characters */
+	/* Characters */
 
-    case 'j':
-      GENERIC_ESIZE(ilu_shortcharacter, ilu_SizeOfShortCharacter);
-    case 'k':
-      GENERIC_ESIZE(ilu_character, ilu_SizeOfCharacter);
-      /*
-	case 'l':
-	GENERIC_ESIZE(ilu_longcharacter, ilu_SizeOfLongCharacter);
-	*/
-      /* Byte */
+      case 'j':
+	GENERIC_ESIZE(ilu_shortcharacter, ilu_SizeOfShortCharacter);
+      case 'k':
+	GENERIC_ESIZE(ilu_character, ilu_SizeOfCharacter);
+	/*
+	  case 'l':
+	  GENERIC_ESIZE(ilu_longcharacter, ilu_SizeOfLongCharacter);
+	  */
+	/* Byte */
 
-    case 'm':
-      GENERIC_ESIZE(ilu_byte, ilu_SizeOfByte);
+      case 'm':
+	GENERIC_ESIZE(ilu_byte, ilu_SizeOfByte);
 
-      /* Boolean */
+	/* Boolean */
 
-    case 'n':
-      GENERIC_ESIZE(CORBA_boolean, ILU_C_SizeOfBoolean);
+      case 'n':
+	GENERIC_ESIZE(CORBA_boolean, ILU_C_SizeOfBoolean);
 
-      /* Enumeration */
-    case 'o':
-      GENERIC_ESIZE(ilu_shortcardinal, ilu_SizeOfEnum);
+	/* Enumeration */
+      case 'o':
+	GENERIC_ESIZE(ilu_shortcardinal, ilu_SizeOfEnum);
 
-      /* Objects */
+	/* Objects */
 
-    case 'p':
-      argSize = _ILU_C_SizeOfObject (call, (ILU_C_Object *) status->ptr, evec[eindex].class, ilu_FALSE, err);
-      break;
+      case 'p':
+	argSize = _ILU_C_SizeOfObject (call, (ILU_C_Object *) status->ptr, evec[eindex].class, ilu_FALSE, err);
+	break;
 
-      /* C String */
-    case 'q':
-      argSize = ilu_SizeOfString (call, *((ilu_string *)(status->ptr)),
-				  strlen(*((ilu_string *)(status->ptr))),
-				  0xFFFF, err);
+	/* C String */
+      case 'q':
+	argSize = ilu_SizeOfString (call, *((ilu_string *)(status->ptr)),
+				    strlen(*((ilu_string *)(status->ptr))),
+				    0xFFFF, err);
 
-    case 'z':
-      argSize = (*((ilu_cardinal (*) (ilu_Call, void *, ilu_Error *))(evec[eindex].fn1)))(call, status->ptr, err);
-      break;
+      case 'z':
+	argSize = (*evec[eindex].sizeFn)(call, status->ptr, err);
+	break;
 
-    default:
-      argSize = 0;
-    }
-  if (ILU_ERRNOK(*err)) return;
+      default:
+	argSize = 0;
+      }
+    if (ILU_ERRNOK(*err)) return;
+  }
 
   _ILU_C_BeginException (call, eindex+1, argSize, err);
   if (ILU_ERRNOK(*err)) return;
@@ -2543,7 +2775,7 @@ found:
 			0xFFFF, err);
 
     case 'z':
-      (*((void (*) (ilu_Call, void *, ilu_Error *))(evec[eindex].fn2)))(call, status->ptr, err);
+      (*evec[eindex].outFn)(call, status->ptr, err);
       break;
 
     default:
@@ -2570,11 +2802,14 @@ void _ilu_CString__Output (ilu_Call call, ilu_string str, ilu_Error *err)
 ilu_CString _ilu_CString__Input (ilu_Call call, ilu_CString *strp, ilu_Error *err)
 {
   ilu_cardinal len = 0;
-  ilu_InputString(call, strp, &len, 0, err);
+  ilu_string p = ILU_NIL;
+
+  ilu_InputString(call, &p, &len, 0, err);
   if (ILU_ERROK(*err))
     {
-      (*strp)[len] = 0;
-      return (*strp);
+      if (strp != ILU_NIL)
+	*strp = p;
+      return (p);
     }
   else
     return (ILU_NIL);
@@ -2686,7 +2921,7 @@ void _ILU_C_EnsureGcClient(void)
   if (CallbackInited)
     return;
   GcCallbackMethods.c = ilu_GetGcCallbackClass();
-  cbs = ILU_C_InitializeServer(ILU_NIL, ILU_NIL, ILU_NIL, ILU_NIL, TRUE);
+  cbs = ILU_C_InitializeServer(ILU_NIL, ILU_NIL, ILU_NIL, ILU_NIL, ILU_NIL, TRUE);
   cbo = _ILU_C_CreateTrueObject(GcCallback__TrueTypeVector,
 				"the-gc-callback", cbs, ILU_NIL, ilu_FALSE);
   if (cbo != ILU_NIL) {
