@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: connect.c,v 1.61 1996/03/08 19:53:13 spreitze Exp $ */
-/* Last edited by Mike Spreitzer March 8, 1996 9:26 am PST */
+/* $Id: connect.c,v 1.67 1996/07/02 17:26:17 larner Exp $ */
+/* Last edited by Mike Spreitzer June 21, 1996 8:55 am PDT */
 
 #define _POSIX_SOURCE
 
@@ -45,8 +45,15 @@ _ilu_BlockingWaitForInputOnConnection(ilu_Connection conn,
 				      ilu_FineTime * limit,
 				      ILU_ERRS((interrupt)) * err)
 {
-  ilu_Transport   x = connection_transport(conn);
-  return (transport_wait_for_input(x, limit, err));
+  ilu_Transport   p_transport;
+  if (connection_closed(conn)) 
+    /* if we were to proceed with this call on a closed connection, we're
+       likely to crash since the transport info has been closed - freed 
+       we can end up here with a closed connection (at least) in the case 
+       where the garbage collector fired up and closed the connection .*/
+    return ilu_FALSE;
+  p_transport = connection_transport(conn);
+  return (transport_wait_for_input(p_transport, limit, err));
 }
 
 ilu_boolean
@@ -121,6 +128,7 @@ _ilu_CloseIoingConnection(ilu_Connection conn,
   ilu_Transport   trans = conn->co_transport;
   ilu_integer     dfd;
 
+  _ilu_Assert(conn->co_ioing, "bad call on _ilu_CloseIoingConnection");
   if (set_cfails)
     conn->co_server->sr_cfails = ilu_TRUE;
   if (!connection_closed(conn)) {
@@ -179,11 +187,10 @@ static struct _ilu_Call_s fauxcall = {0};
 
 /* Usable on either incoming or outgoing connections. */
 /*L1 = {conn's server, cmu}; L2 disjoint {conn's callmu, iomu}*/
-static ILU_ERRS((bad_locks, broken_locks))
+ILU_ERRS((bad_locks, broken_locks))
 _ilu_CloseConnection(ilu_Connection connection)
 {
   ilu_Error       err;
-  ilu_Server      s = connection->co_server;
   if (!_ilu_EnterConnIO(connection, FALSE, &err))
     return err;
   _ilu_CloseIoingConnection(connection, ilu_FALSE);
@@ -352,7 +359,6 @@ _ilu_CreateConnection(ilu_Transport bs,
 {
   ilu_Error		lerr = ILU_INIT_NO_ERR;
   ilu_Connection	ans;
-  ilu_IdentityInfo	ident;
 
   ans = (ilu_Connection) ilu_MallocE(sizeof(*ans), err);
   if (ans == NIL)
@@ -378,7 +384,7 @@ _ilu_CreateConnection(ilu_Transport bs,
   ans->co_conn_identity.ii_owned_by_passport = ilu_FALSE;
   if (port != NIL) {
     ans->co_conn_identity.ii_type = ilu_ConnectionIdentity;
-    ans->co_conn_identity.ii_info.ii_ConnectionInfo = conn_peerinfo(ans);
+    ans->co_conn_identity.ii_info = (ilu_refany) conn_peerinfo(ans);
     ans->co_auth_info = pp;
     if (!ilu_AddIdentity (pp, &ans->co_conn_identity, err))
       goto faild;
