@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: courier.c,v 1.104 1996/03/13 04:57:59 janssen Exp $ */
-/* Last edited by Mike Spreitzer February 9, 1996 7:54 am PST */
+/* $Id: courier.c,v 1.117 1996/07/09 23:47:04 janssen Exp $ */
+/* Last edited by Mike Spreitzer June 27, 1996 4:30 pm PDT */
 
 #ifdef MACOS
 #pragma segment ilu
@@ -44,7 +44,7 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #endif /* (defined(WIN32) && defined(_WINIO)) */
 
 
-#define ILU_COURIER_PROGRAM_NUMBER	0x00100000
+#define ILU_COURIER_PROGRAM_NUMBER	0x00110000
 
 
 #define MEMCPY(to,from,len)	memcpy((void *)(to),(void *)(from),(len))
@@ -162,16 +162,20 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
 		   class->cl_name);
 	  exit(1);
 	}
-      if (NOT (strncmp(class->cl_singleton, "courier_", 9) == 0 AND
-	       (s->cri_pnumber = _ilu_atoi(class->cl_singleton + 9, &next),
-		(s->cri_pnumber > 0 && (next != NIL) && (*next == '_'))) AND
-	       (s->cri_version = _ilu_atoi(next + 1, NIL),
-		(s->cri_version > 0))))
+      if (strncmp(class->cl_singleton, "courier_", 9) == 0)
 	{
-	  ILU_ERRPRINTF("(ILU:courier)  Couldn't determine Courier program number for ILU class \"%s\", given native Courier info \"%s\".\n",
-		   class->cl_name, class->cl_singleton);
-	  return (NIL);
+	  if (NOT ((s->cri_pnumber = _ilu_atoi(class->cl_singleton + 9, &next),
+		    (s->cri_pnumber > 0 && (next != NIL) && (*next == '_'))) AND
+		   (s->cri_version = _ilu_atoi(next + 1, NIL),
+		    (s->cri_version > 0))))
+	    {
+	      ILU_ERRPRINTF("(ILU:courier)  Couldn't determine Courier program number for ILU class \"%s\", given native Courier info \"%s\".\n",
+			    class->cl_name, class->cl_singleton);
+	      return (NIL);
+	    }
 	}
+      else
+	return NIL;
     }
   else
     {
@@ -180,10 +184,10 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
       s->cri_pnumber = ILU_COURIER_PROGRAM_NUMBER | ((hash >> 16) & 0xFFFF);
       s->cri_version = hash & 0xFFFF;
     }
-  _ilu_Assert(_ilu_hash_AddToTable(RegistryHashTable,
+  _ilu_Assert((int) _ilu_hash_AddToTable(RegistryHashTable,
 				   s->cri_type_id, s),
 	      "Courier AddToTable RegistryHashTable");
-  _ilu_Assert(_ilu_hash_AddToTable(ProgramNumberHashTable, s, s),
+  _ilu_Assert((int) _ilu_hash_AddToTable(ProgramNumberHashTable, s, s),
 	      "Courier AddToTable ProgramNumberHashTable");
   return (s);
 }
@@ -203,8 +207,8 @@ static courierinfo *_courier_CourierInformationForClass (ilu_Class class)
 	(stderr, "%s \"%s:%s\", pnumber is 0x%lx, version is %lu.\n",
 	 "_courier_CourierInformationForClass:  Class",
 	 class->cl_name, class->cl_unique_id,
-	 (s == NIL) ? 0 : s->cri_pnumber,
-	 (s == NIL) ? 0 : s->cri_version));
+	 (s == NIL) ? 0L : s->cri_pnumber,
+	 (s == NIL) ? 0L : s->cri_version));
 
   return (s);
 }
@@ -290,7 +294,7 @@ _courier_OutputShortCardinal(ilu_Call call, ilu_shortcardinal i,
 #ifdef WORDS_BIGENDIAN
     *((ilu_shortcardinal *) &buf[0]) = i;
 #else
-    *((ilu_shortcardinal *) &buf[0]) = ((i >> 8) & ((i & 0xFF) << 8));
+    *((ilu_shortcardinal *) &buf[0]) = ((i >> 8) | ((i & 0xFF) << 8));
 #endif
   }
   return;
@@ -314,7 +318,7 @@ _courier_InputShortCardinal(ilu_Call call, ilu_shortcardinal * i,
 #ifdef WORDS_BIGENDIAN
     *i = *((ilu_shortcardinal *) &buf[0]);
 #else
-    *i = (buf[1] << 8) & buf[0];
+    *i = (buf[0] << 8) | buf[1];
 #endif
   }
   return;
@@ -561,7 +565,7 @@ static void
 _courier_InputLongReal(ilu_Call call, ilu_longreal * d,
 		       ILU_ERRS((IoErrs)) * err)
 {
-  transport_read_bytes(callport(call), (ilu_bytes) d, 16, err);
+  (void) transport_read_bytes(callport(call), (ilu_bytes) d, 16, err);
 }
 
 /*ARGSUSED*/
@@ -701,7 +705,7 @@ _courier_InputByte(ilu_Call call, ilu_byte * b,
 {
   ilu_shortcardinal l = 0;
   _courier_InputShortCardinal(call, &l, err);
-  *b = l & 0xFF;
+  *b = (ilu_byte)(l & 0xFF);
   return;
 }
 
@@ -714,6 +718,30 @@ _courier_SizeOfByte(ilu_Call call, ilu_byte i,
   return (2);
 }
 
+/* ==================== short char ==================== */
+
+static void
+_courier_OutputShortChar(ilu_Call call, ilu_shortcharacter b,
+			 ILU_ERRS((IoErrs)) * err)
+{
+  _courier_OutputByte(call, (ilu_byte) b, err);
+}
+
+static void
+_courier_InputShortChar(ilu_Call call, ilu_shortcharacter * b,
+			ILU_ERRS((IoErrs)) * err)
+{
+  _courier_InputByte(call, (ilu_byte*) b, err);
+}
+
+/*ARGSUSED*/
+static          ilu_cardinal
+_courier_SizeOfShortChar(ilu_Call call, ilu_shortcharacter i,
+			 ILU_ERRS((IoErrs)) * err)
+{
+  return _courier_SizeOfByte(call, (ilu_byte) i, err);
+}
+
 /* ==================== opaque ==================== */
 
 static void
@@ -1018,6 +1046,7 @@ _courier_DiscardMessage(ilu_Call call, ILU_ERRS((internal)) * err)
       return FALSE;
     }
   } ILU_ERR_ENDSWITCH;
+  return ILU_CLER(*err);
 }
 
 static          ilu_boolean
@@ -1370,6 +1399,7 @@ static ilu_Protocol _courier_NewCourier (void)
 		     ilu_malloc(sizeof(struct _ilu_Protocol_s));
 
   new->pr_concurrent_requests = ilu_FALSE;
+  new->pr_sizing_required = ilu_FALSE;
 
   new->pr_init_call = _courier_InitCall;
   new->pr_start_request = _courier_StartRequest;
@@ -1381,6 +1411,7 @@ static ilu_Protocol _courier_NewCourier (void)
   new->pr_begin_exception = _courier_BeginException;
   new->pr_finish_exception = _courier_FinishException;
   new->pr_finish_call = _courier_FinishCall;
+  new->pr_prefinish_call = NULLFN;
 
   new->pr_output_integer = _courier_OutputInteger;
   new->pr_input_integer = _courier_InputInteger;
@@ -1450,14 +1481,18 @@ static ilu_Protocol _courier_NewCourier (void)
 	(ilu_cardinal (*)(ilu_Call,ilu_boolean,ilu_Error*))
 	_courier_SizeOfCardinal;
 
-  new->pr_output_shortchar = _courier_OutputByte;
-  new->pr_input_shortchar = _courier_InputByte;
-  new->pr_size_of_shortchar = _courier_SizeOfByte;
+  new->pr_output_shortchar = _courier_OutputShortChar;
+  new->pr_input_shortchar = _courier_InputShortChar;
+  new->pr_size_of_shortchar = _courier_SizeOfShortChar;
 
   new->pr_output_string = _courier_OutputString;
   new->pr_input_string = _courier_InputString;
   new->pr_size_of_string = _courier_SizeOfString;
 
+  new->pr_output_wstring = _ilu_OutputWString;
+  new->pr_input_wstring = _ilu_InputWString;
+  new->pr_size_of_wstring = _ilu_SizeOfWString;
+
   new->pr_output_bytes = _courier_OutputBytes;
   new->pr_input_bytes = _courier_InputBytes;
   new->pr_size_of_bytes = _courier_SizeOfBytes;
@@ -1466,6 +1501,10 @@ static ilu_Protocol _courier_NewCourier (void)
   new->pr_input_opaque = _courier_InputOpaque;
   new->pr_size_of_opaque = _courier_SizeOfOpaque;
 
+  new->pr_output_wstringvec = _ilu_OutputWStringVec;
+  new->pr_input_wstringvec = _ilu_InputWStringVec;
+  new->pr_size_of_wstringvec = _ilu_SizeOfWStringVec;
+
   new->pr_output_object_id = _ilu_OutputObjectID;
   new->pr_input_object_id = _ilu_InputObjectID;
   new->pr_size_of_object_id = _ilu_SizeOfObjectID;
