@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: gc.c,v 1.59 1996/03/08 19:53:13 spreitze Exp $ */
+/* $Id: gc.c,v 1.63 1996/07/01 23:08:41 jacobi Exp $ */
 /* Last tweaked by Mike Spreitzer March 8, 1996 9:04 am PST */
 
 #define _POSIX_SOURCE
@@ -55,15 +55,15 @@ const ilu_Class _ilu_GcCallbackClass = &GCCallbackClassRecord;
   obj true && collectible => L1 >= {gcmu}*/
 ilu_boolean ilu_VeryInterested(ilu_Object obj)
 {
-  if (   obj->ob_gclist != NIL
-      && _ilu_vector_size(VECTOR(obj->ob_gclist)) != 0)
+  if (object_gclist(obj) != NIL &&
+      _ilu_vector_size(VECTOR(object_gclist(obj))) != 0)
     return TRUE;
   if (   class_collectible(obj->ob_class)
-      && ilu_CoarseTime_Now() < obj->ob_lastRemote + obj->ob_timeout)
+      && ilu_CoarseTime_Now() < object_lastRemote(obj) + obj->ob_timeout)
     return TRUE;
-  if (obj->ob_notifying && !obj->ob_known)
+  if (object_notifying(obj) && !object_known(obj))
     return TRUE;
-  if (obj->ob_holds != 0)
+  if (object_holds(obj) != 0)
     return TRUE;
   return FALSE;
 }
@@ -111,21 +111,21 @@ static ilu_Vector Objects = NIL;
 /* The set {obj | obj is a GCed true object} */
 
 /* gcmu invariant:
-   cc->nObjs =  |{obj | cc in obj->ob_gclist}|.
+   cc->nObjs =  |{obj | cc in object_gclist(obj)}|.
    Objects = {obj | obj is true && collectible && not released}.
    Clients = {<cc->client, cc> | cc->nObjs != 0 || cc->sweeping}.
    (cc->nObjs != 0 || cc->sweeping) contributes 1 to cc->client->ob_holds.
    cc->gcc is set iff (cc->nObjs != 0 || cc->sweeping).
    forall collectible true obj:
-	obj->ob_gclist is empty => lastRemote is when it became empty;
+	object_gclist(obj) is empty => lastRemote is when it became empty;
  */
 
 /*L1 >= {gcmu, cmu, obj's server}*/
 ILU_ERRS((BadDataStructure, KernelBroken))
 _ilu_TouchedObj(ilu_Object obj)
 {
-  int             should = (obj->ob_gclist == NIL)
-  || (_ilu_vector_size(VECTOR(obj->ob_gclist)) == 0);
+  int             should = (object_gclist(obj) == NIL)
+  || (_ilu_vector_size(VECTOR(object_gclist(obj))) == 0);
   ilu_Error       ans;
   ASSERT(object_collectible(obj), buf,
 	 (buf, "gc.c:TouchedObj: !collectible(%s/%s)",
@@ -135,10 +135,10 @@ _ilu_TouchedObj(ilu_Object obj)
 	  obj->ob_server->sr_id, obj->ob_ih));
   if (should) {
     ilu_FineTime    t = {0, 0};
-    t.ft_s = obj->ob_timeout + obj->ob_lastRemote;
-    ilu_MXASet(&gcor, &obj->ob_gco, t);
+    t.ft_s = obj->ob_timeout + object_lastRemote(obj);
+    ilu_MXASet(&gcor, &object_gco(obj), t);
   } else
-    ilu_MXAClear(&gcor, &obj->ob_gco);
+    ilu_MXAClear(&gcor, &object_gco(obj));
   ans = _ilu_VIUpdate(obj);
   ILU_ERR_SWITCH(ans) {
     ILU_SUCCESS_CASE
@@ -203,7 +203,7 @@ gcoInvoke(ilu_Alarmette a)
   ilu_Error       err;
   static struct _ilu_Object_s dumby;
   ilu_Object      obj = (ilu_Object) (((char *) a) -
-		   (((char *) &dumby.ob_gco) - ((char *) &dumby)));
+		   (((char *) &dumby.ob_collectibleInfo.ob_true.ob_gco) - ((char *) &dumby)));
   ilu_Server      s = object_server(obj);
   _ilu_AcquireMutex(ilu_cmu);
   _ilu_AcquireMutex(server_lock(s));
@@ -254,10 +254,10 @@ void _ilu_StartGCingTrueObj (ilu_Object obj)
 void _ilu_StopGCingTrueObj (ilu_Object obj)
 {
   _ilu_HoldMutex(ilu_gcmu);
-  if (obj->ob_gclist != NIL)
-      _ilu_vector_destroy(VECTOR(obj->ob_gclist),
+  if (object_gclist(obj) != NIL)
+      _ilu_vector_destroy(VECTOR(object_gclist(obj)),
                           NULLFN/*vector is empty*/);
-  obj->ob_gclist = NIL;
+  object_gclist(obj) = NIL;
   _ilu_vector_remove (Objects, obj);
   return;
 }
@@ -285,14 +285,14 @@ AddGCInterest(ilu_Object obj, ilu_Object interest)
   _ilu_AcquireMutex(ilu_gcmu);
   _ilu_AcquireMutex(ilu_cmu);
   _ilu_AcquireMutex(server_lock(s));
-  if (obj->ob_gclist == NIL)
-    obj->ob_gclist = (void *) _ilu_vector_new(INITIAL_GCLIST_SIZE);
+  if (object_gclist(obj) == NIL)
+    object_gclist(obj) = (void *) _ilu_vector_new(INITIAL_GCLIST_SIZE);
   if (Clients == NIL)
     Clients = _ilu_hash_MakeNewTable(INITIAL_GCCLIENTS_SIZE,
 		 _ilu_hash_HashPointer, _ilu_hash_PointerCompare);
-  for (i = 0, j = _ilu_vector_size(VECTOR(obj->ob_gclist)),
+  for (i = 0, j = _ilu_vector_size(VECTOR(object_gclist(obj))),
        p = (counted_client **)
-       _ilu_vector_elements(VECTOR(obj->ob_gclist));
+       _ilu_vector_elements(VECTOR(object_gclist(obj)));
        i < j; i++)
     if (p[i]->client == interest) {
       _ilu_ReleaseMutex(server_lock(s));
@@ -308,7 +308,7 @@ AddGCInterest(ilu_Object obj, ilu_Object interest)
     cc->client = interest;
     cc->sweeping = FALSE;
     cc->gcc = gcc;
-    _ilu_Assert(_ilu_hash_AddToTable(Clients, interest, cc),
+    _ilu_Assert((int)_ilu_hash_AddToTable(Clients, interest, cc),
 		"AddGCInterest AddToTable Clients");
     ilu_MXASet(&gccr, &cc->gcc,
 	       ilu_FineTime_Add(ilu_FineTime_Now(), pingPeriod));
@@ -324,8 +324,8 @@ AddGCInterest(ilu_Object obj, ilu_Object interest)
         _ilu_Assert(0, "DeltaHolds(interest) => GcRegFailed");
     } ILU_ERR_ENDSWITCH;
   }
-  _ilu_vector_add(VECTOR(obj->ob_gclist), cc);
-  if (_ilu_vector_size(VECTOR(obj->ob_gclist)) == 1) {
+  _ilu_vector_add(VECTOR(object_gclist(obj)), cc);
+  if (_ilu_vector_size(VECTOR(object_gclist(obj))) == 1) {
     err = _ilu_TouchedObj(obj);
     ILU_ERR_SWITCH(err) {
       ILU_SUCCESS_CASE;
@@ -342,27 +342,33 @@ fini:
 /*L1 = {gcmu, cmu, obj's server} before, {gcmu, cmu} after;
   forall conn: (L2 >= {conn.iomu}) => (L2 >= {conn.callmu})*/
 static
-ILU_ERRS((BadDataStructure, KernelBroken,
+ILU_ERRS((BadDataStructure, KernelBroken, internal,
 	  bad_locks, broken_locks))
-DropGCInterest(ilu_Object obj, ilu_Object interest,
+DropGCInterest(ilu_Object obj,		/* non-NIL true obj of collectible type */
+	       ilu_Object interest,	/* non-NIL surrogate obj of type "GCCallba*/
 	       ilu_boolean fullFlush)
 {
   register ilu_integer i, j;
   register counted_client **p;
   ilu_Error       err = ILU_NO_ERR;
 
-  if (obj == NIL OR obj->ob_gclist == NIL OR interest == NIL)
-    return err;
+  if (obj == NIL OR interest == NIL)
+    return ILU_ERR_CONS1(internal, &err, minor, ilu_im_broken, err);
+  else if (object_gclist(obj) == NIL)
+    {
+      _ilu_ReleaseMutex(server_lock(object_server(obj)));
+      return err;
+    }
 
-  for (i = 0, j = _ilu_vector_size(VECTOR(obj->ob_gclist)),
+  for (i = 0, j = _ilu_vector_size(VECTOR(object_gclist(obj))),
        p = (counted_client **)
-       _ilu_vector_elements(VECTOR(obj->ob_gclist));
+       _ilu_vector_elements(VECTOR(object_gclist(obj)));
        i < j; i++) {
     if (p[i]->client == interest) {
       counted_client *p2 = p[i];
-      _ilu_vector_remove(VECTOR(obj->ob_gclist), p2);
-      if (_ilu_vector_size(VECTOR(obj->ob_gclist)) == 0) {
-	obj->ob_lastRemote = ilu_CoarseTime_Now();
+      _ilu_vector_remove(VECTOR(object_gclist(obj)), p2);
+      if (_ilu_vector_size(VECTOR(object_gclist(obj))) == 0) {
+	object_lastRemote(obj) = ilu_CoarseTime_Now();
 	err = _ilu_TouchedObj(obj);
 	ILU_ERR_SWITCH(err) {
 	  ILU_SUCCESS_CASE;
