@@ -14,9 +14,13 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: iluerror.h,v 1.20 1996/02/27 02:41:25 mdavidso Exp $ */
+/* $Id: iluerror.h,v 1.22 1996/07/09 21:54:50 mdavidso Exp $ */
 /* Last edited by Mike Spreitzer February 1, 1996 11:54 am PST */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #ifndef _ILUERROR_H_
 #define _ILUERROR_H_
 
@@ -450,15 +454,15 @@ do  {	ilu_Error *__ilu_CurrentError = &(x); \
 	} else if ( 1 ) {
 
 #define ILU_ERR_ENDSWITCH \
-	else {char buf[1000]; \
+	else {char __errbuf[1000]; \
 	if (ILU_ERROK(*__ilu_CurrentError)) \
-	  sprintf(buf, "unhandled success"); \
+	  sprintf(__errbuf, "unhandled success"); \
 	else \
-	  sprintf(buf, "unhandled error %s from line %d in file %s", \
+	  sprintf(__errbuf, "unhandled error %s from line %d in file %s", \
 	    ILU_ERR_NAME(*__ilu_CurrentError), \
 	    ilu_ErrorLine(__ilu_CurrentError), \
 	    ilu_ErrorFile(__ilu_CurrentError)); \
-	_ilu_Assert(0, buf);} } while (0)
+	_ilu_Assert(0, __errbuf);} } while (0)
 
 #define ILU_ERR_CASE2(t1, t2) \
 	} else if (\
@@ -509,3 +513,6 @@ ILU_ERRMEM_T(x) *ilu_ErrpQua_##x (ilu_Error *e)
 
 
 #endif /* ndef _ILUERROR_H_ */
+#ifdef __cplusplus
+}
+#endif
