@@ -14,8 +14,12 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: iluerrs.h,v 1.54 1996/03/07 21:31:07 janssen Exp $ */
-/* Last edited by Mike Spreitzer February 11, 1996 10:35 pm PST */
+/* $Id: iluerrs.h,v 1.61 1996/07/12 20:04:20 janssen Exp $ */
+/* Last edited by Mike Spreitzer April 2, 1996 10:37 pm PST */
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #ifndef _ILUERRS_H_
 #define _ILUERRS_H_
@@ -59,11 +63,20 @@ typedef enum {
   ilu_bpm_unionDiscSize,	/* invalid discriminant size */
   ilu_bpm_nil,			/* NIL passed where not allowed */
   ilu_bpm_closed,		/* op invoked on closed something */
+  ilu_bpm_small_buffer,		/* callee needs larger buffer */
+  ilu_bpm_string_null_char,	/* octet 0 in `string' parm */
   ilu_bpm_threading,
   /*
    * A proc appropriate only for single-threaded runtimes was called
    * in a mult-threaded runtime --- or the other way around.
    */
+  ilu_bpm_protocol_registered,
+  /* attempt to register already-registered protocol */
+  ilu_bpm_transport_registered,
+  /* attempt to register already-registered transport */
+  ilu_bpm_identity_type_registered,
+  /* attempting to register already-registered identity type */
+
   ilu_bpm_bogus_raise,		/* Method tried to raise an
 				 * exception not in its RAISES list. */
   ilu_bpm_some_raise		/* Method with empty exn list tried
@@ -87,12 +100,16 @@ typedef enum {
    * ILU will marshal only strings & byte-sequences less than a
    * certain length.
    */
-  ilu_ilm_nomst
+  ilu_ilm_nomst,
   /*
    * When importing a surrogate, ILU requires that the importing
    * program know some of the object's types, and that one of those
    * known types is a subtype of all the other known types.
    */
+  ilu_ilm_max_protocols,	/* too many protocols registered */
+  ilu_ilm_max_transports,	/* too many transports registered */
+  ilu_ilm_max_identity_types	/* too many identity types registered */
+
 }               ilu_imp_limit_Minor;
 
 ILU_DECL_ERR(imp_limit)
@@ -115,6 +132,7 @@ typedef enum {
   /* Second, failures of established connections: */
 
   ilu_cfm_eof,			/* unexpected eof on connection */
+  ilu_cfm_protocol_sync_lost,	/* unexpected bytes with no way to recover */
   ilu_cfm_conn_lost		/* other or unknown reason */
 }               ilu_comm_failure_Minor;
 
@@ -183,6 +201,7 @@ typedef enum {
   ilu_im_typeMismatch,		/* two stubs with different ideas */
   ilu_im_typeIncomplete,	/* type not completely constructed */
   ilu_im_threading,		/* confusion on whether threaded */
+  ilu_im_threadFork,		/* fork failed */
   ilu_im_tportRole,		/* incoming vs. outgoing transport err */
   ilu_im_check			/* internal consistency check failed */
 }               ilu_internal_Minor;
@@ -202,10 +221,16 @@ typedef enum {
 				 * sequence longer than its limit */
   ilu_mm_badMagicNumber,	/* bad message header magic number */
   ilu_mm_versionMismatch,	/* wrong version of message protocol */
-  ilu_mm_badInteger,		/* signed or unsigned integer that doesn't fit position */
-  ilu_mm_badFloat,		/* floating point value that doesn't fit position */
-  ilu_mm_dgramLimit		/* datagram (eg, UDP) size limit
+  ilu_mm_badInteger,		/* signed or unsigned integer that
+				 * doesn't fit position */
+  ilu_mm_badFloat,		/* floating point value that doesn't
+				 * fit position */
+  ilu_mm_dgramLimit,		/* datagram (eg, UDP) size limit
 				 * exceeded */
+  ilu_mm_msgTypeUnknown,	/* invalid message type received */
+  ilu_mm_utf2Len		/* UTF2 encoded string's length
+				 * doesn't correspond to plain
+				 * string length */
 }               ilu_marshal_Minor;
 
 ILU_DECL_ERR(marshal)
@@ -568,4 +593,7 @@ ILU_PUBLIC unsigned long
  */
 
 #endif /* ndef _ILUERRS_H_ */
+#ifdef __cplusplus
+}
+#endif
 
