@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: iluntrnl.h,v 1.142 1996/03/12 17:45:40 spreitze Exp $ */
-/* Last edited by Mike Spreitzer March 12, 1996 9:33 am PST */
+/* $Id: iluntrnl.h,v 1.161 1996/07/15 06:41:32 janssen Exp $ */
+/* Last edited by Mike Spreitzer June 27, 1996 4:08 pm PDT */
 
 #ifndef _ILU_INTERNALS_
 #define _ILU_INTERNALS_
@@ -60,6 +60,55 @@ extern void
 /* Adds the identities in conn->co_auth_info
    to the call->ca_caller passport.  Creates the passport if that field is NIL */
 
+/* ----- utility functions for dealing with WString IO ----- */
+
+/* L1, L2 unconstrained */
+ilu_cardinal
+  _ilu_SizeOfWString (ilu_Call call,
+		      ilu_wstring s,
+		      ilu_cardinal l1	/* size of wstring */,
+		      ilu_cardinal limit,
+		      ILU_ERRS((IoErrs)) *err);
+
+/* Main holds, L2 >= {call's connection's callmu, iomu} */
+void
+  _ilu_OutputWString (ilu_Call call,
+		      ilu_wstring s,
+		      ilu_cardinal l1,
+		      ilu_cardinal limit,
+		      ILU_ERRS((IoErrs)) * err);
+
+/* Main holds, L2 >= {call's connection's callmu, iomu} */
+void
+  _ilu_InputWString (ilu_Call call,
+		     ilu_wstring * s,
+		     ilu_cardinal * l,
+		     ilu_cardinal limit,
+		     ILU_ERRS((IoErrs)) * err);
+
+/* L1, L2 unconstrained */
+ilu_cardinal
+  _ilu_SizeOfWStringVec (ilu_Call call,
+			 ilu_wstring s,
+			 ilu_cardinal l1	/* size of wstringvec */,
+			 ILU_ERRS((IoErrs)) *err);
+
+/* Main holds, L2 >= {call's connection's callmu, iomu} */
+void
+  _ilu_OutputWStringVec (ilu_Call call,
+			 ilu_wstring s,
+			 ilu_cardinal l1,
+			 ILU_ERRS((IoErrs)) * err);
+
+/* Main holds, L2 >= {call's connection's callmu, iomu} */
+void
+  _ilu_InputWStringVec (ilu_Call call,
+			ilu_wstring * s,
+			ilu_cardinal l,
+			ILU_ERRS((IoErrs)) * err);
+
+/* ----- utility functions for dealing with object IO ----- */
+
 /*L2 >= {call's connection's callmu, iomu}.
   kobj == NIL => Main Invariant holds.
   kobj != NIL => all the following:
@@ -99,7 +148,8 @@ typedef enum {
   ilu_rhrc_ok,			/* Header successfully started */
   ilu_rhrc_eof,			/* Found EOF instead of header */
   ilu_rhrc_nothing,		/* No input available w/o blocking */
-  ilu_rhrc_error		/* An error is being raised */
+  ilu_rhrc_error,		/* An error is being raised */
+  ilu_rhrc_handled		/* A protocol-specific request was handled */
 }               ilu_ReadHeaderResultCode;
 
 struct _ilu_Protocol_s {
@@ -107,12 +157,19 @@ struct _ilu_Protocol_s {
    * are readonly.  The locking comments refer to invoc'ns of the methods.
    */
 
-  ilu_boolean     pr_concurrent_requests;
+  ilu_boolean	pr_concurrent_requests;
   /*
    * Does this protocol support concurrent requests on one
    * connection?
    */
 
+  ilu_boolean	pr_sizing_required;
+  /*
+   * TRUE if this protocol requires accurate argument sizes to be passed to
+   * pr_start_request, pr_begin_reply, and pr_begin_exception.  When FALSE,
+   * implies that the pr_size_of_* calls may be invalid or NIL.
+   */
+
   /*L2, Main unconstrained*/
   /*L1_sup < prmu (prmu protects class<->prog&ver registry)*/
   ilu_string (*pr_form_handle)(ilu_Object obj);
@@ -131,8 +188,7 @@ struct _ilu_Protocol_s {
    * Called early in the life of incoming and outgoing calls, so
    * that ca_prdata1 and ca_prdata2 may be initialized; ca_server
    * and ca_connection are the only other fields that are certainly
-   * initialized yet.  Next: sizing of arguments on client side,
-   * pr_read_header on server side.
+   * initialized yet.  Next: sizing of arguments, or pr_read_header.
    */
   
   /*Main Invariant holds; L2 >= {call's conn's callmu, iomu}*/
@@ -153,7 +209,8 @@ struct _ilu_Protocol_s {
    * pr_finish_request, between pr_begin_reply and pr_finish_reply,
    * or between pr_begin_exception and pr_finish_exception, to abort
    * composing, and not send, the message being composed.  Caller
-   * next calls pr_finish_call.
+   * next calls pr_finish_call, pr_begin_sizing_exn, or
+   * pr_begin_exception.
    */
   
   ilu_boolean(*pr_finish_request) (ilu_Call call,
@@ -179,12 +236,16 @@ struct _ilu_Protocol_s {
    * then call pr_delay_interp, pr_interpret_reply,
    * pr_interpret_request, or pr_discard_input.  If EOF is found,
    * return ilu_rhrc_eof; caller will close the ilu_Connection and
-   * proceed to pr_finish_call.  If it is impossible to tell whether
+   * proceed to pr_finish_call.  Note that finding EOF should not
+   * be considered an error.  If it is impossible to tell whether
    * a message or EOF is next without blocking (i.e., caller was
    * wrong about something waiting), return ilu_rhrc_nothing; client
    * stub will go back to waiting for input, server stub will
-   * proceed to pr_finish_call.  Return ilu_rhrc_error iff raising
-   * an error; caller will proceed to pr_finish_call.
+   * proceed to pr_finish_call.  If a control message internal to
+   * the protocol was received and handled, return ilu_rhrc_handled;
+   * client stub will go back to waiting for an input, server stub
+   * will proceed to pr_finish_call.  Return ilu_rhrc_error iff
+   * raising an error; caller will proceed to pr_finish_call.
    */
 
   ilu_refany(*pr_delay_interp) (ilu_Call call,
@@ -329,7 +390,17 @@ struct _ilu_Protocol_s {
    * pr_finish_call.
    */
 
-  /*L1, L2 unconstrained*/
+  /**L1 >= {cmu, calls' server}, L1.sup < trmu;
+     Main Remnant holds; L2 >= {call's conn's callmu, iomu}*/
+  ilu_boolean(*pr_prefinish_call) (ilu_Call call,
+				   ILU_ERRS((IoErrs)) * err);
+  /*
+   * Called whenever pr_finish_call is, unless kernel is too broken.
+   * May be null, meaning NO-OP.  Called just before pr_finish_call.
+   * Note that call's connection may already be closed.
+   */
+
+  /*Main Invariant holds; L2 disjoint {call's conn's callmu, iomu}*/
   ilu_boolean(*pr_finish_call) (ilu_Call call,
 				ILU_ERRS((IoErrs)) * err);
   /*
@@ -678,6 +749,7 @@ struct _ilu_TransportCreator_s {
 				   ilu_TransportInfo * /* tinfo_out */ ,
 				   ilu_boolean /* buffer */ ,
 				   ilu_integer * /* dfd */ ,
+				   ilu_Passport /* passport */,
 				   ILU_ERRS((IoErrs)) * /* err */ );
   /*
    * The Mooring instance creation procedure.  Caller promises
@@ -821,7 +893,12 @@ struct _ilu_TransportClass_s {
    * of any stage in the input pipeline --- regardless of whether
    * any bytes dribble out this end.  Used in single-threaded and
    * multi-threaded runtimes.  In S-T R/T, blocks by running main
-   * loop; in M-T, blocks only the calling thread.
+   * loop; in M-T, blocks only the calling thread.  Note there's no
+   * guarantee that /self/ won't be closed at return time.  In fact,
+   * there's a design bug here: because no mutex is held at call
+   * time, there's not really any guarantee /self/ will be open at
+   * call time (but callee must assume so --- otherwise he has no
+   * guarantee that /self/ isn't a dangling pointer!).
    */
 
   ilu_boolean(*tc_interrupt) (ilu_Transport /* self */ ,
@@ -1151,10 +1228,12 @@ struct _ilu_Server_s {
    * is usually only seen in ILU URLs.
    */
 
-  ilu_string      sr_id;	/* part of an oid; plain form */
+  ilu_string	sr_id;		/* part of an oid; plain form */
+  ilu_cardinal	sr_crc32;	/* CRC32 of the plain server ID */
 
   ilu_LanguageIndex sr_true_language; /* index in language table */
 
+
   /* L1 >= {self} */
 
   /*
@@ -1222,29 +1301,36 @@ struct _ilu_Object_s {
    * Vector of slots in which to store LS pointers, indexed by
    * language-index
    */
-  ilu_integer     ob_holds;	/* Count of uses in root method
-				 * stubs and gc.c. */
-  int             ob_intNoted;	/* LSR thinks this about whether
+  unsigned int	ob_intNoted : 1;/* LSR thinks this about whether
 				 * kernel is very interested in me */
+  unsigned int	ob_holds : 31;	/* Count of uses in root method
+				 * stubs and gc.c. */
+  union {
 
-  /* The following fields are meaningful only for coll. surrogate
-   * objs. */
+    struct {
+      /* The following fields are meaningful only for coll. surrogate
+       * objs. */
 
-  int             ob_notifying;	/* we're calling true server now */
-  int             ob_known;	/* when last !notifying, did true
-				 * server know about this surrogate? */
+      ilu_boolean	ob_notifying;	/* we're calling true server now */
+      ilu_boolean	ob_known;	/* when last !notifying, did true
+					 * server know about this surrogate? */
+    } ob_surrogate;
 
-  /* The following fields are meaningful only for collectible true
-   * objs. */
+    struct {
+      /* The following fields are meaningful only for collectible true
+       * objs. */
 
-  /* L1 >= {gcmu} */
+      /* L1 >= {gcmu} */
 
-  ilu_Alarmette_s ob_gco;	/* mxamu==gcmu */
-  ilu_integer     ob_lastRemote;/* when we last knew the object to
-				 * be loose in the network; not
-				 * meaningful while gclist non-empty */
-  void           *ob_gclist;	/* set of gc.c:counted_client*; NIL
-				 * may represent the empty set */
+      ilu_Alarmette_s ob_gco;	/* mxamu==gcmu */
+      ilu_integer     ob_lastRemote;/* when we last knew the object to
+				     * be loose in the network; not
+				     * meaningful while gclist non-empty */
+      void           *ob_gclist;	/* set of gc.c:counted_client*; NIL
+					 * may represent the empty set */
+    } ob_true;
+
+  } ob_collectibleInfo;
 };
 /*
 The string fields are owned by the object.
@@ -1478,7 +1564,12 @@ ilu_boolean
 _ilu_BlockingWaitForInputOnConnection(ilu_Connection conn,
 				      ilu_FineTime * limit,
 				      ILU_ERRS((interrupted)) * err);
-/* Calls conn's transport's wait_for_input method. */
+/*
+ * Calls conn's transport's wait_for_input method.  Note lack of
+ * guarantee that /conn/ isn't closed at return time. (In fact
+ * there's a bug here: no mutex is held to guarantee that conn isn't
+ * closed at call time.)
+ */
 
 /*L1 >= {port's server}*/
 
@@ -1505,6 +1596,13 @@ _ilu_CloseIoingConnection(ilu_Connection conn,
  * server's connect info is bad.
  */
 
+
+/*L1 = {conn's server, cmu}; L2 disjoint {conn's callmu, iomu}*/
+extern ILU_ERRS((bad_locks, broken_locks))
+_ilu_CloseConnection(ilu_Connection connection);
+/* Usable on either incoming or outgoing connections. */
+
+
 /*Main Invariant, L2 >= {conn's iomu}*/
 extern ilu_boolean 
 _ilu_CloseConnWithIo(ilu_Connection conn, ilu_boolean set_cfails,
@@ -1672,7 +1770,6 @@ void _ilu_HandlePing (ilu_Call call);
 ILU_ERRS((BadDataStructure, KernelBroken, bad_locks, broken_locks))
 _ilu_ServerRemoveObject(ilu_Server s, ilu_Object obj);
 
-
 /*Inside(server, result's type)*/
 ilu_Object _ilu_FindObjectInServer(ilu_string ih, ilu_Server s);
 
@@ -1750,6 +1847,12 @@ extern ilu_refany _ilu_ioTimeoutAlarm;
 extern ilu_refany _ilu_grAlarm;		/* for use in ilu_GetReply */
 extern ilu_refany _ilu_udpAlarm;	/* for use in udp.c */
 
+/*L1, L2, Main unconstrained*/
+
+#ifdef UDPSOCKET_TRANSPORT
+void _ilu_udp_SetTimeouts (double to1, double toN, double tto);
+#endif
+
 /*L1_sup < trmu*/
 
 ilu_TransportCreator 
@@ -1800,6 +1903,8 @@ _ilu_WaitForOutputOnFD(int fd, ilu_boolean * sure,
  * *sure was set to true.
  * 
  * Use of _ilu_InterruptFD does not cause interrupted to be raised.
+ * 
+ * Note lack of guarantees about what's not closed at return time.
  */
 
 ilu_boolean
@@ -1861,20 +1966,6 @@ _ilu_VIUpdate(ilu_Object obj);
  */
 
 
-/****************************** from crc32.c ********************/
-
-/*L1, L2, Main unconstrained */
-
-extern ilu_cardinal _ilu_CRC32
-  (ilu_bytes,		/* buffer to take the CRC-32 of, retain */
-   ilu_cardinal);	/* length of buffer */
-
-extern ilu_cardinal _ilu_CRC32WithAccum
-  (ilu_bytes,		/* buffer to take the CRC-32 of, retain */
-   ilu_cardinal,	/* length of buffer */
-   ilu_cardinal);	/* accumulated CRC returned from previous call to
-			   _ilu_CRC32 or _ilu_CRC32WithAccum */
-
 /****************************** from type.c ********************/
 
 /*L1, L2, Main unconstrained */
@@ -1904,10 +1995,40 @@ void _ilu_EnumerateClasses (void (*proc) (ilu_Class, ilu_refany rock), ilu_refan
 ilu_Class _ilu_FindClassViaRPC (ilu_Object o);
 /* o->ob_class is temporarily set to some known type of o */
 
+#ifdef IIOP_PROTOCOL
+
+/*Main Invariant holds; L2 otherwise unconstrained*/
+
+ilu_Class _ilu_IIOP_FindClassViaRPC(ilu_Object o);
+/* o->ob_class is temporarily set to some known type of o */
+
+/*L1, L2 unconstrained*/
+
+extern ilu_boolean _ilu_IIOP_ParseIIOP (ilu_string, ilu_string *, ilu_string *, ilu_string *, ilu_string *, ilu_cardinal *, ilu_Error *);
+extern ilu_boolean _ilu_IIOP_ParseIOR (ilu_string, ilu_string *, ilu_string *, ilu_string *, ilu_string *, ilu_cardinal *, ilu_Error *);
+
+/*L1, L2, Main unconstrained (this is only for calling from debugger)*/
+ilu_cardinal _ilu_IIOP_SetMaxStringSize (ilu_cardinal size);
+
+#endif
+
+
+#ifdef HTTP_PROTOCOL
+extern ilu_boolean _ilu_Parse_HTTP_URL(ilu_string, ilu_string *, ilu_string *, ilu_string *, ilu_string *, ilu_cardinal *, ilu_Error *);
+#endif
+
+
 /*L2    >=    {conn's callmu, iomu} before,
   L2 disjoint {conn's callmu, iomu} after*/
 void _ilu_HandleGetTypes (ilu_Call call);
 
+/* L1 > otmu */
+/* To be used inside an _ilu_EnumerateClasses enumeration */
+ilu_boolean 
+_ilu_IsSubObjectType(ilu_Class a, ilu_Class b);
+/* Returns TRUE iff a is a subtype of b
+   (including the degenerate case of a=b). */
+
 /****************************** Hash Table ops ********************/
 
 #include "iluhash.h"
