@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: inmem.c,v 1.19 1996/03/12 17:45:40 spreitze Exp $ */
+/* $Id: inmem.c,v 1.27 1996/05/29 03:34:30 janssen Exp $ */
 /* Last edited by Mike Spreitzer March 12, 1996 9:38 am PST */
 
 /* In-memory Transport
@@ -86,7 +86,10 @@ typedef struct {
   ilu_boolean     td_input; /* vs. output.  Meaningful if working. */
 
   ilu_boolean     td_outgoing; /* vs. incoming. */
-/*  MooringParms    td_mooringData;  /* Address of mooring parms */
+
+#if 0
+  MooringParms    td_mooringData;  /* Address of mooring parms */
+#endif
 
   SharedData      td_sharedData; /* shared with "other end" transport */
   BufferList      td_current; /* NIL if !working */
@@ -361,7 +364,6 @@ _inmem_WriteBytes(ilu_Transport self, ilu_bytes buf,
 {
 
   TransportParms  tp = (TransportParms) self->tr_data;
-  SharedData      sd = tp->td_sharedData;
   ilu_bytes       b;
   ilu_cardinal    more = bufferSize + 16;
 
@@ -382,7 +384,7 @@ _inmem_WriteBytes(ilu_Transport self, ilu_bytes buf,
   if (buf != NIL) {
     /* Copy it into the exposed buffer */
     memcpy((void *) (self->tr_outBuff + self->tr_outNext),
-	   (void *) buf, bufferSize);
+	   (void *) buf, (SIZE_T) bufferSize);
     self->tr_outNext += bufferSize;
   }
   return ILU_CLER(*err);
@@ -398,7 +400,6 @@ _inmem_ReadBytes(ilu_Transport self,
 {
   
   TransportParms tp = (TransportParms) self->tr_data;
-  SharedData sd = tp->td_sharedData;
 
   rpt->tr_eom = rpt->tr_eof = FALSE;
   
@@ -477,6 +478,7 @@ _inmem_CloseTransport(ilu_Transport self, ilu_integer * dfd,
   }
   ilu_free(tp);
   self->tr_data = NIL;
+  ilu_free(self);
   return ILU_CLER(*err);
 }
 
@@ -562,7 +564,7 @@ _inmem_AcceptClient(ilu_Mooring self, ilu_string * tinfo_out,
   char buf[32];
 
   *dfd = 0;
-  _ilu_Assert(mp->md_locked, "_inmem_AcceptClient");
+  _ilu_Assert((int) mp->md_locked, "_inmem_AcceptClient");
   /* signal ilu_im_brokenLocks? */
   
   if (tinfo_out != NIL) {
@@ -684,9 +686,9 @@ _inmem_FdUsage(ilu_TransportCreator self, ilu_boolean mooring)
 }
 
 /* Main Invariant holds */
-ilu_Transport
-_inmem_CreateTransport(ilu_TransportCreator self, ilu_boolean buffer,
-		     ilu_integer *dfd, ilu_Passport pp, ILU_ERRS((IoErrs)) * err)
+static ilu_Transport
+  _inmem_CreateTransport(ilu_TransportCreator self, ilu_boolean buffer,
+			 ilu_integer *dfd, ilu_Passport pp, ILU_ERRS((IoErrs)) * err)
 {
   /* Creates outgoing transport, makes call to create incoming transport
      and link the two.  (Will allocate buffers even if !buffer.) */
@@ -776,10 +778,11 @@ static struct _ilu_Mooring_s mooringProto = {
 /* L1_sup < trmu; L2 unconstrained */
 static ilu_Mooring 
 _inmem_CreateMooring(ilu_TransportCreator self,
-		   ilu_TransportInfo * tinfo_out,
-		   ilu_boolean buffer, /* ignored */
-		   ilu_integer *dfd,
-		   ILU_ERRS((IoErrs)) * err)
+		     ilu_TransportInfo * tinfo_out,
+		     ilu_boolean buffer, /* ignored */
+		     ilu_integer *dfd,
+		     ilu_Passport pp,	/* unused */
+		     ILU_ERRS((IoErrs)) * err)
 {
   CreatorParms    cp = (CreatorParms) self->tcr_data;
   MooringParms    mp;
@@ -886,8 +889,8 @@ _ilu_inmem_TransportCreator(ilu_TransportInfo tinfo,
 
 /* Main Invariant holds */
 ilu_Transport
-_ilu_BufferTransport(ilu_cardinal size, ilu_bytes buffer,
-		     ILU_ERRS((IoErrs)) * err)
+_ilu_BufferTransport_Create (ilu_cardinal size, ilu_bytes buffer,
+			     ILU_ERRS((IoErrs)) * err)
 {
   TransportParms  tp;
   ilu_Transport   t;
@@ -955,21 +958,36 @@ _ilu_BufferTransport(ilu_cardinal size, ilu_bytes buffer,
 
 /*L1, l2?*/
 void
-_ilu_BufferTransportParms(ilu_Transport self, ilu_cardinal * size,
-			  ilu_bytes * buffer, ILU_ERRS((IoErrs)) * err)
+_ilu_BufferTransport_Destroy (ilu_Transport self, ilu_cardinal * size,
+			      ilu_bytes * buffer, ILU_ERRS((IoErrs)) * err)
 {
   TransportParms  tp;
+  ilu_boolean	input;
+  BufferList	current;
+  ilu_integer	dfd;
+
   tp = (TransportParms) self->tr_data;
+  input = tp->td_input;
+  current = tp->td_current;
 
   _inmem_EndMessage(self, TRUE, NIL, err);
   if (ILU_ERRNOK(*err))
     return;
 
-  if (size != NIL)
-    *size = tp->td_current->offset;
-  if (buffer != NIL) {
-    *buffer = tp->td_current->buffer;
-    tp->td_current->buffer = NIL;
-  }
-  ILU_CLER(*err);
+  if (current != NIL)
+    {
+      if (size != NIL)
+	*size = current->offset;
+      if (buffer != NIL) {
+	*buffer = current->buffer;
+	current->buffer = NIL;
+      }
+    }
+
+  /* we need to close this, as this transport is always td_outgoing == FALSE */
+  CloseSharedData(tp->td_sharedData);
+  tp->td_sharedData = NIL;
+  tp->td_current = NIL;
+
+  transport_close(self, &dfd, err);
 }
