@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: newtcp.c,v 1.48 1996/03/13 04:57:28 janssen Exp $ */
-/* Last edited by Mike Spreitzer March 5, 1996 4:13 pm PST */
+/* $Id: newtcp.c,v 1.59 1996/07/11 07:23:28 spreitze Exp $ */
+/* Last edited by Mike Spreitzer July 10, 1996 11:18 pm PDT */
 
 #include "iluntrnl.h"
 
@@ -26,6 +26,11 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
 #include "oscalls.h"
 
+#ifdef XEROX_FIREWALL_HANDLING
+#include "xerox_firewall.c"
+#define connect(a,b,c)	xerox_firewall_connect((a),(b),(c))
+#endif /* XEROX_FIREWALL_HANDLING */
+
 #if (defined(WIN32) || defined(WIN16))
 
 static int set_doer_non_blocking (int fd)
@@ -48,7 +53,7 @@ static int add_fcntl_flag (int fd, int dflags)
 #define set_listener_non_blocking(fd)	add_fcntl_flag((fd), O_NONBLOCK)
 #define set_doer_non_blocking(fd)	add_fcntl_flag((fd), O_NONBLOCK)
 
-#elif defined(_IS_BSD)
+#elif (defined(_IS_BSD) && defined(FIONBIO))
 
 static int set_doer_non_blocking (int fd)
 {
@@ -57,12 +62,33 @@ static int set_doer_non_blocking (int fd)
 }
 #define set_listener_non_blocking	set_doer_non_blocking
 
+#elif (defined(_IS_BSD) && defined(HAS_SOLARIS1_NONBLOCKING_BUG))
+static int set_doer_non_blocking (int fd)
+{
+  int one = 1;
+  return (OS_SOCKIOCTL(fd, 0x8004667e, &one));
+  return 0;
+}
+#define set_listener_non_blocking	set_doer_non_blocking
+
 #else	/* not POSIX or BSD or Windows */
 
 #error "Don't know how to do non-blocking I/O for anything but POSIX, Windows, and BSD just now"
 
 #endif
 
+#ifdef ILU_NEEDS_NETINET_TCP_H_FOR_NODELAY
+#include <netinet/tcp.h>
+#endif
+
+#if 0
+#undef XEROX_FIREWALL_HANDLING
+#undef set_listener_non_blocking
+#undef set_doer_non_blocking
+#define set_listener_non_blocking(fd) 0
+#define set_doer_non_blocking(fd) 0
+#endif
+
 typedef unsigned short ilutcpport_t;
 /* WIN port needs arg of htons() to be an unsigned short */
 
@@ -274,12 +300,13 @@ Write(ilu_Transport self, ilu_bytes buf, ilu_cardinal size,
 {
   TCPParms        p = (TCPParms) self->tr_data;
   DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	(stderr, "%s %lu bytes from %p to fd %d.\n",
-	 "(tcp.c:Write):  writing", size, buf, p->fd));
+	(stderr, "tcp(%p):  writing %lu bytes from %p to FD %d.\n",
+	 self, size, buf, p->fd));
 
 #ifdef ENABLE_DEBUGGING
 
-  if ((_ilu_DebugLevel & PACKET_DEBUG) != 0)
+  if (((_ilu_DebugLevel & PACKET_DEBUG) != 0) &&
+      (size > 0))
     _ilu_debug_DumpPacket(buf, size, "outgoing TCP");
 
 #endif /* ENABLE_DEBUGGING */
@@ -337,7 +364,6 @@ _tcp_WriteBytes(ilu_Transport self, ilu_bytes b,
     }
     if (!Write(self, b, bufferSize, err))
       return FALSE;
-    return ILU_CLER(*err);
   } else {
     ilu_cardinal    l1 = MIN(rem, bufferSize);
     memcpy((void*)(self->tr_outBuff + self->tr_outNext), (void*)b, l1);
@@ -354,6 +380,7 @@ _tcp_WriteBytes(ilu_Transport self, ilu_bytes b,
       self->tr_outNext += bufferSize;
     }
   }
+  return ILU_CLER(*err);
 }
 
 static          ilu_cardinal
@@ -367,7 +394,8 @@ _tcp_ReadBytes(ilu_Transport self,
   ilu_cardinal	  toread;
   ilu_cardinal    lread;
   ilu_bytes	  actbuff;
-  ilu_boolean	  use_internal = (buffer == NIL) || (len < DIRECT_THRESHOLD);
+  ilu_boolean     use_internal = ((buffer == NIL)
+				  || (len < DIRECT_THRESHOLD));
   rpt->tr_eom = rpt->tr_eof = FALSE;
   if (self->tr_inBuff != NIL && self->tr_inNext != self->tr_inLimit)
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_tcInputSkipsBuff,
@@ -386,19 +414,20 @@ _tcp_ReadBytes(ilu_Transport self,
   if (ILU_ERRNOK(*err)) {
 
 #ifdef ENABLE_DEBUGGING
-    if (_ilu_DebugLevel & INCOMING_DEBUG)
-      ILU_ERRPRINTF("_tcp_ReadBytes: got error %s from FD %d!\n",
-	      ILU_ERR_NAME(*err), p->fd);
+    if (_ilu_DebugLevel & TCP_DEBUG)
+      ILU_ERRPRINTF("tcp(%p): ReadBytes got error %s from FD %d!\n",
+		    self, ILU_ERR_NAME(*err), p->fd);
 #endif /* ENABLE_DEBUGGING */
 
   } else {
-    DEBUG(INCOMING_DEBUG,
+    DEBUG(TCP_DEBUG,
 	  (stderr,
-	   "_tcp_ReadBytes:  read %u bytes from FD %d, eof=%s\n",
-	   lread, p->fd, rpt->tr_eof ? "T" : "F"));
+	   "tcp(%p): read %u bytes from FD %d, eof=%s\n",
+	   self, lread, p->fd, rpt->tr_eof ? "T" : "F"));
 
 #ifdef ENABLE_DEBUGGING
-    if ((_ilu_DebugLevel & PACKET_DEBUG) != 0)
+    if (((_ilu_DebugLevel & PACKET_DEBUG) != 0) &&
+	(lread > 0))
       _ilu_debug_DumpPacket(actbuff, lread, "incoming TCP");
 #endif /* ENABLE_DEBUGGING */
 
@@ -412,6 +441,24 @@ _tcp_ReadBytes(ilu_Transport self,
   return lread;
 }
 
+
+#if (defined (WIN32) || defined (WIN16))
+/* **************************************************** */
+/* returns 1 is the fd is ready for writing, else 0 */
+
+static int _tcp_writeable_fd(int fd) {
+
+	fd_set write_set;
+	struct timeval time = {0, 0};
+	
+	FD_ZERO(&write_set);
+	FD_SET(fd, &write_set);
+		
+	return select(fd + 1, NULL, &write_set, NULL, &time);
+}
+#endif
+
+
 /*L1, L2, Main unconstrained*/
 
 /*Main Invariant holds*/
@@ -422,7 +469,7 @@ _tcp_Connect(ilu_Transport self,
 {
   TCPParms        p = (TCPParms) self->tr_data;
   int             fd, err1;
-  struct sockaddr_in sin = cp->addr;
+  struct sockaddr_in sinaddr = cp->addr;
   ilu_boolean     status = TRUE;
   unsigned        port;
 
@@ -432,17 +479,17 @@ _tcp_Connect(ilu_Transport self,
   if (self == NIL)
     return ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_nil, FALSE);
 
-  port = ntohs(sin.sin_port);
+  port = ntohs(sinaddr.sin_port);
   
   DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-     (stderr, "tcp_Connect:  connecting to host %s, port %u...\n",
-      inet_ntoa(sin.sin_addr), port));
+	(stderr, "tcp(%p):  connecting to host %s, port %u...\n",
+	 self, inet_ntoa(sinaddr.sin_addr), port));
 
   if (OS_SOCKINV(fd = socket(AF_INET, SOCK_STREAM, 0))) {
     err1 = sockerrno;
     DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	  (stderr, "tcp_Connect:  socket call failed:  %s.\n",
-	   strerror(err1)));
+	  (stderr, "tcp(%p):  socket call failed:  %s.\n",
+	   self, strerror(err1)));
     switch (err1) {
     case SOCKERRID(ACCES):		/* permission error */
       ILU_ERR_CONS0(no_permission, err, 0);
@@ -512,23 +559,46 @@ _tcp_Connect(ilu_Transport self,
 	     self));
       ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_networkAddr, 0);
       status = FALSE;
+#ifndef XEROX_FIREWALL_HANDLING
+      /* firewall code doesn't work with non-blocking connect */
     } else if (OS_SOCKERR(set_doer_non_blocking(fd))) {
       err1 = sockerrno;
       DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
 	    (stderr,
-	     "tcp_Connect:  Failed to set socket (descriptor %d, host %s) non-blocking.  Error \"%s\".\n",
-	     fd, inet_ntoa(sin.sin_addr),
+	     "tcp(%p):  Failed to set socket (FD %d, host %s)"
+	     " non-blocking, error \"%s\".\n",
+	     self, fd, inet_ntoa(sinaddr.sin_addr),
 	     strerror(err1)));
       ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_nonblock, 0);
       status = FALSE;
+#endif /* XEROX_FIREWALL_HANDLING */
     } else {
-      if (connect(fd, (struct sockaddr *) & sin, sizeof(sin)) == 0) {
+      int             connres;
+      connres = connect(fd, (struct sockaddr *) & sinaddr,
+			sizeof(sinaddr));
+      if (!OS_SOCKERR(connres)) {
 	DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
 	      (stderr,
 	       "tcp(%p):  connected to %s:%u on FD %d\n",
-	       self, inet_ntoa(sin.sin_addr), port, fd));
+	       self, inet_ntoa(sinaddr.sin_addr), port, fd));
 	p->fd = fd;
 	status = TRUE;
+#ifdef XEROX_FIREWALL_HANDLING
+	/* do non-blocking after the connect */
+	if (OS_SOCKERR(set_doer_non_blocking(fd))) {
+	  err1 = sockerrno;
+	  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
+		(stderr,
+		 "tcp(%p):  Failed to set socket (FD %d, host %s)"
+		 " non-blocking.  Error \"%s\".\n",
+		 self, fd, inet_ntoa(sinaddr.sin_addr),
+		 strerror(err1)));
+	  close(fd);
+	  p->fd = -1;
+	  ILU_ERR_CONS1(comm_failure, err, minor, ilu_cfm_nonblock, 0);
+	  status = FALSE;
+	}
+#endif /* XEROX_FIREWALL_HANDLING */
       }
 #if (defined(WIN32) || defined(WIN16))
       else if (sockerrno == SOCKERRID(INPROGRESS) ||
@@ -541,20 +611,30 @@ _tcp_Connect(ilu_Transport self,
 	SOCKET_SIZE_TYPE pnlen = sizeof(peername);
 	ilu_boolean     sure;
 	_ilu_WaitForOutputOnFD(fd, &sure, NIL, err);
-	if (getpeername(fd, &peername, &pnlen) == 0) {
+	if (
+#if (defined (WIN32) || defined (WIN16))
+	/*
+	 * getpeername is not a good test for connectivity on Win32,
+	 * so check to make sure the _ilu_WaitForOutputOnFD returned
+	 * because of writability, not because of an exception.
+	 */
+	    _tcp_writeable_fd(fd) > 0
+#else
+	    getpeername(fd, &peername, &pnlen) == 0
+#endif
+	  ) {
 	  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
 		(stderr,
 		 "tcp(%p):  eventually connected to %s:%u on FD %d\n",
-		 self, inet_ntoa(sin.sin_addr), port, fd));
+		 self, inet_ntoa(sinaddr.sin_addr), port, fd));
 	  p->fd = fd;
 	  status = TRUE;
 	} else {
 	  DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
 		(stderr,
-		 "tcp_Connect: connect to %s:%u failed "
+		 "tcp(%p): connect FD %d to %s:%u failed "
 		 "(no meaningful errno available).\n",
-		 inet_ntoa(sin.sin_addr),
-		 port));
+		 self, fd, inet_ntoa(sinaddr.sin_addr), port));
 	  OS_SOCKLOSE(fd);
 	  ILU_ERR_CONS1(comm_failure, err, minor,
 			ilu_cfm_connect_failed, 0);
@@ -563,11 +643,10 @@ _tcp_Connect(ilu_Transport self,
       } else {
 	int erri = sockerrno;
 	DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	      (stderr, "tcp_Connect: connect to %s:%u failed,"
+	      (stderr, "tcp(%p): connect FD %d to %s:%u failed,"
 	       " error %d (%s)\n",
-	       inet_ntoa(sin.sin_addr),
-	       port, erri,
-	       strerror(erri)));
+	       self, fd, inet_ntoa(sinaddr.sin_addr),
+	       port, erri, strerror(erri)));
 	  OS_SOCKLOSE(fd);
 	switch (erri)
 	  {
@@ -668,10 +747,10 @@ NewT(ilu_boolean buffer, ILU_ERRS((IoErrs)) * err)
 }
 
 /*Main Invariant holds*/
-ilu_Transport
-_tcp_CreateTransport(ilu_TransportCreator self, ilu_boolean buffer,
-		     ilu_integer * dfd, ilu_Passport pp,
-		     ILU_ERRS((IoErrs)) * err)
+static ilu_Transport
+  _tcp_CreateTransport(ilu_TransportCreator self, ilu_boolean buffer,
+		       ilu_integer * dfd, ilu_Passport pp,
+		       ILU_ERRS((IoErrs)) * err)
 {
   CreatorParms    cp = (CreatorParms) self->tcr_data;
   ilu_Transport   ans;
@@ -703,9 +782,9 @@ _tcp_Close(ilu_Transport self, ilu_integer * dfd,
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_broken, FALSE);
   parms = (TCPParms) self->tr_data;
   ILU_CLER(*err);
-  ilu_UnregisterInputSource(parms->fd);
-  ilu_UnregisterOutputSource(parms->fd);
   if (parms->fd >= 0) {
+    ilu_UnregisterInputSource(parms->fd);
+    ilu_UnregisterOutputSource(parms->fd);
     OS_SOCKLOSE(parms->fd);
     *dfd = 1;
   }
@@ -733,7 +812,8 @@ _tcp_FormInfo(ilu_string hostname, ilutcpport_t port,
 {
   ilu_string      retval;
   char            buf[1000];
-  sprintf(buf, altfmt ? "TCP %s %p" : "tcp_%s_%lu", hostname, ((unsigned long) port) & 0xFFFF);
+  sprintf(buf, altfmt ? "TCP %s %u" : "tcp_%s_%u", hostname,
+	  ((unsigned) port) & 0xFFFF);
   if ((retval = _ilu_Strdup(buf)) == NIL)
     return ILU_ERR_CONS1(no_memory, err, nbytes, strlen(buf) + 1, NIL);
   ILU_CLER(*err);
@@ -748,8 +828,8 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
 {
   MooringParms    mParms;
   TCPParms        tParms;
-  ilu_integer     ns;
-  struct sockaddr_in sin;
+  int             ns;
+  struct sockaddr_in sinaddr;
   ilu_Transport   newT = NIL;
   SOCKET_SIZE_TYPE addrlen;
   int             mfd;
@@ -759,17 +839,18 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
     mParms = (MooringParms) self->mo_data;
   mfd = mParms->fd;
   DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-     (stderr, "_tcp_AcceptClient: mooring file descriptor is %d\n",
-      mfd));
-  addrlen = sizeof(sin);
-  if (OS_SOCKINV(ns = OS_ACCEPT(mfd, (struct sockaddr *) & sin,
+	(stderr,
+	 "tcp(%p): accepting connection request on FD %d.\n",
+	 self, mfd));
+  addrlen = sizeof(sinaddr);
+  if (OS_SOCKINV(ns = OS_ACCEPT(mfd, (struct sockaddr *) & sinaddr,
 				&addrlen))) {
     int             theerr = sockerrno;
     ilu_no_resources_Minor nrm;
     switch (theerr) {
     case SOCKERRID(WOULDBLOCK):
       DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	    (stderr, "tcp(%p): Spurrious call on accept(FD=%d).\n",
+	    (stderr, "tcp(%p): Spurrious call on accept(FD %d).\n",
 	     self, mfd));
       ILU_CLER(*err);
       return NIL;
@@ -781,20 +862,20 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
   raise_no_resources:
       DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
 	    (stderr,
-	     "tcp(%p): accept(FD=%d) => errno %d, %s.\n",
+	     "tcp(%p): accept(FD %d) => errno %d, %s.\n",
 	     self, mfd, theerr, strerror(theerr)));
       return ILU_ERR_CONS1(no_resources, err, minor, nrm, NIL);
 #if (defined(WIN32) || defined(WIN16))
     case WSANOTINITIALISED:
       DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
 	    (stderr,
-	     "tcp(%p): Can't accept(FD=%d) because WinSock not initialized.\n",
+	     "tcp(%p): Can't accept(FD %d) because WinSock not initialized.\n",
 	     self, mfd));
       goto bitch_errno;
 #endif
     default:
       DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
-	    (stderr, "tcp(%p): accept(FD=%d) => errno %d.\n",
+	    (stderr, "tcp(%p): accept(FD %d) => errno %d.\n",
 	     self, mfd, theerr));
   bitch_errno:
       return ILU_ERR_CONS1(internal, err, minor, ilu_im_errno, NIL);
@@ -805,9 +886,9 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
     int theerr = sockerrno;
     DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
 	  (stderr,
-	   "_tcp_AcceptClient:  Failed to set socket (descriptor"
-	   " %ld) non-blocking.  Error \"%s\".\n",
-	   ns, strerror(theerr)));
+	   "tcp(%p):  Failed to set socket"
+	   " (FD %d) non-blocking.  Error \"%s\".\n",
+	   self, ns, strerror(theerr)));
   }
   /* See above comment on similar code below */
 #ifdef IPPROTO_TCP
@@ -822,8 +903,8 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
 
   if (tinfo_out != NIL) {
     if (ilu_EnterMutex(ilu_trmu, err)) {
-      *tinfo_out = _tcp_FormInfo(inet_ntoa(sin.sin_addr),
-				 ntohs(sin.sin_port), TRUE, err);
+      *tinfo_out = _tcp_FormInfo(inet_ntoa(sinaddr.sin_addr),
+				 ntohs(sinaddr.sin_port), TRUE, err);
       (void) ilu_ExitMutex(ilu_trmu, TRUE, err);
     }
     if (ILU_ERRNOK(*err)) {
@@ -837,9 +918,10 @@ _tcp_AcceptClient(ilu_Mooring self, ilu_string *tinfo_out,
     return NIL;
   DEBUG((CONNECTION_DEBUG | TCP_DEBUG),
 	(stderr,
-	 "_tcp_AcceptClient: new transport %p, on fd %ld,"
+	 "tcp(%p): new transport %p, on FD %d,"
 	 " from IP host %s, port %u.\n",
-	 newT, ns, inet_ntoa(sin.sin_addr), ntohs(sin.sin_port)));
+	 self, newT, ns, inet_ntoa(sinaddr.sin_addr),
+	 ntohs(sinaddr.sin_port)));
   tParms = (TCPParms) newT->tr_data;
   tParms->fd = ns;
   return (newT);
@@ -936,14 +1018,16 @@ _tcp_CreateMooring(ilu_TransportCreator self,
 		   ilu_TransportInfo * tinfo_out,
 		   ilu_boolean buffer,
 		   ilu_integer *dfd,
+		   ilu_Passport pp,	/* unused here */
 		   ILU_ERRS((IoErrs)) * err)
 {
   CreatorParms    cp = (CreatorParms) self->tcr_data;
   MooringParms    mp = NIL;
-  struct sockaddr_in sin = cp->addr;
+  struct sockaddr_in sinaddr = cp->addr;
   struct linger   linger = {0};
   SOCKET_SIZE_TYPE namelen;
-  ilu_integer     skt = -1, on = 1;
+  int             skt = -1;
+  int             on = 1;
   ilu_Mooring     ans = NIL;
   int             err1;
   ilu_boolean     loopedback = FALSE;
@@ -960,8 +1044,8 @@ _tcp_CreateMooring(ilu_TransportCreator self,
   if (OS_SOCKINV(skt = socket(AF_INET, SOCK_STREAM, 0))) {
     err1 = sockerrno;
     DEBUG((EXPORT_DEBUG | TCP_DEBUG),
-	  (stderr, "_tcp_CreateMooring: socket failed:  %s.\n",
-	   strerror(err1)));
+	  (stderr, "tcp(%p): create Mooring socket failed:  %s.\n",
+	   self, strerror(err1)));
     switch (err1) {
 #if !(defined(WIN32) || defined(WIN16))
     case SOCKERRID(ACCES):		/* permission error */
@@ -997,16 +1081,17 @@ _tcp_CreateMooring(ilu_TransportCreator self,
 	     (char *) &linger, sizeof(struct linger));
 
   while (1) {
-    int             err1;
-    if (!OS_SOCKERR(bind(skt, (struct sockaddr *) & sin, sizeof(sin))))
+    int             err2;
+    if (!OS_SOCKERR(bind(skt, (struct sockaddr *) & sinaddr,
+			 sizeof(sinaddr))))
       break;
-    err1 = sockerrno;
+    err2 = sockerrno;
     DEBUG((EXPORT_DEBUG | TCP_DEBUG),
 	  (stderr,
 	   "tcp(%p): bind to %08lx:%u failed:  %s.\n",
-	   self, ntohl(sin.sin_addr.s_addr), ntohs(sin.sin_port),
-	   strerror(err1)));
-    switch (err1) {
+	   self, ntohl(sinaddr.sin_addr.s_addr), ntohs(sinaddr.sin_port),
+	   strerror(err2)));
+    switch (err2) {
     case SOCKERRID(ADDRNOTAVAIL):
       /*
        * We should get this error when name-to-addr(hostname) isn't
@@ -1021,7 +1106,7 @@ _tcp_CreateMooring(ilu_TransportCreator self,
 	 * bind to INADDR_ANY.  So we bind to that, and report
 	 * "127.0.0.1" (the loopback address) in the tinfo_out.
 	 */
-	sin.sin_addr.s_addr = INADDR_ANY;
+	sinaddr.sin_addr.s_addr = INADDR_ANY;
 	loopedback = TRUE;
 	goto tryagain;
       }
@@ -1049,17 +1134,17 @@ tryagain:
   }
   
   /* If tinfo gave port 0 then discover kernel allocated port */
-  if (sin.sin_port == 0) {
-    namelen = sizeof(sin);
-    if (OS_SOCKERR(getsockname(skt, (struct sockaddr *) & sin,
+  if (sinaddr.sin_port == 0) {
+    namelen = sizeof(sinaddr);
+    if (OS_SOCKERR(getsockname(skt, (struct sockaddr *) & sinaddr,
 			       &namelen))) {
-      int             err1 = sockerrno;
+      int             err2 = sockerrno;
       DEBUG((EXPORT_DEBUG | TCP_DEBUG),
-	 (stderr, "_tcp_CreateMooring: getsockname failed:  %s.\n",
-	  strerror(err1)));
+	 (stderr, "tcp(%p): getsockname failed:  %s.\n",
+	  self, strerror(err2)));
       OS_SOCKLOSE(skt);
       *dfd = 0;
-      switch (err1) {
+      switch (err2) {
       case SOCKERRID(FAULT):
 	ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_badPointer, 0);
 	break;
@@ -1081,14 +1166,14 @@ tryagain:
     }
   }
   if (OS_SOCKERR(listen(skt, 4))) {
-    int             err1 = sockerrno;
+    int             err2 = sockerrno;
     DEBUG((EXPORT_DEBUG | TCP_DEBUG),
 	  (stderr,
-	   "_tcp_CreateMooring:  listen on port %u failed:  %s.\n",
-	   ntohs(sin.sin_port), strerror(err1)));
+	   "tcp(%p):  listen on port %u, FD %d failed:  %s.\n",
+	   self, ntohs(sinaddr.sin_port), skt, strerror(err2)));
     OS_SOCKLOSE(skt);
     *dfd = 0;
-    switch (err1) {
+    switch (err2) {
     case SOCKERRID(BADF):
     case SOCKERRID(NOTSOCK):
     case SOCKERRID(OPNOTSUPP):
@@ -1102,11 +1187,11 @@ tryagain:
     goto failc;
   }
   if (OS_SOCKERR(set_listener_non_blocking(skt))) {
-    int             err1 = sockerrno;
+    int             err2 = sockerrno;
     DEBUG((EXPORT_DEBUG | TCP_DEBUG),
 	  (stderr,
-	   "_tcp_CreateMooring:  Unable to set listening socket (FD %d) non-blocking: %s\n",
-	   skt, strerror(err1)));
+	   "tcp(%p):  Unable to set listening socket (FD %d) non-blocking: %s\n",
+	   self, skt, strerror(err2)));
     OS_SOCKLOSE(skt);
     *dfd = 0;
     ILU_ERR_CONS1(internal, err, minor, ilu_im_errno, 0);
@@ -1124,8 +1209,8 @@ tryagain:
   mp->fd = skt;
   mp->buff_p = buffer;
   DEBUG(TCP_DEBUG,
-	(stderr, "tcp(%p): CreateMooring, FD=%ld, port=%u\n",
-	 ans, (long int) skt, ntohs(sin.sin_port)));
+	(stderr, "tcp(%p): created Mooring %p on FD %d, port %u.\n",
+	 self, ans, skt, ntohs(sinaddr.sin_port)));
   if (tinfo_out != NIL) {
     ilu_string      hostname, tinfo;
     if (!loopedback)
@@ -1134,7 +1219,7 @@ tryagain:
       hostname = "localhost";
     else
       hostname = "127.0.0.1";
-    tinfo = _tcp_FormInfo(hostname, ntohs(sin.sin_port), FALSE, err);
+    tinfo = _tcp_FormInfo(hostname, ntohs(sinaddr.sin_port), FALSE, err);
     if (ILU_ERRNOK(*err))
       goto faild;
     * tinfo_out = ilu_MallocE((2 * SIZEOF_VOID_P) + strlen(tinfo) + 1,
@@ -1200,3 +1285,4 @@ _ilu_tcp_TransportCreator(ilu_TransportInfo tinfo,
   return (ans);
 }
 
+
