@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: object.c,v 1.131 1996/03/08 19:53:13 spreitze Exp $ */
-/* Last edited by Mike Spreitzer March 7, 1996 1:11 pm PST */
+/* $Id: object.c,v 1.143 1996/07/15 06:41:44 janssen Exp $ */
+/* Last edited by Mike Spreitzer June 28, 1996 8:07 am PDT */
 
 #define _POSIX_SOURCE
 
@@ -25,6 +25,8 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
 #include "iluntrnl.h"
 
+#include <ctype.h>
+
 #include "object.h"
 #include "server.h"
 #include "type.h"
@@ -116,7 +118,7 @@ Encode(char *b, ilu_cardinal len,
        ILU_ERRS((internal, no_memory)) * err)
 {
   char           *p, *plim = b + len, *ans;
-  int             count = 0, i;
+  ilu_cardinal   count = 0, i;
   for (p = b; p < plim; p++) {
     if (!ISSAFE(*p))
       count++;
@@ -164,7 +166,7 @@ Decode(char *b, ilu_cardinal len,
        ilu_inv_objref_Minor minor)
 {
   char           *p, *plim = b + len, *ans;
-  int             count = 0, i;
+  ilu_cardinal   count = 0, i;
   for (p = b; p < plim;)
     if (*p == '%') {
       count++;
@@ -277,11 +279,6 @@ void ilu_RegisterSBHParser (ilu_string sbh_scheme, ilu_SBHParser parser)
   _ilu_hash_AddToTable (RegisteredParsers, _ilu_Strdup(sbh_scheme), (ilu_private) parser);
 }
 
-#ifdef IIOP_PROTOCOL
-extern ilu_boolean _ilu_IIOP_ParseIIOP (ilu_string, ilu_string *, ilu_string *, ilu_string *, ilu_string *, ilu_cardinal *, ilu_Error *);
-extern ilu_boolean _ilu_IIOP_ParseIOR (ilu_string, ilu_string *, ilu_string *, ilu_string *, ilu_string *, ilu_cardinal *, ilu_Error *);
-#endif
-
 ilu_boolean
   ilu_ParseSBH(ilu_string encodedSBH,
 	       ilu_string * plainInstanceHandle,
@@ -303,6 +300,9 @@ ilu_boolean
       ilu_RegisterSBHParser ("IOR", _ilu_IIOP_ParseIOR);
       ilu_RegisterSBHParser ("ior", _ilu_IIOP_ParseIOR);
       ilu_RegisterSBHParser ("iiop", _ilu_IIOP_ParseIIOP);
+#endif
+#ifdef HTTP_PROTOCOL
+      ilu_RegisterSBHParser ("http", _ilu_Parse_HTTP_URL);
 #endif
       initted = ilu_TRUE;
     }
@@ -381,7 +381,13 @@ _ilu_ParseConnectInfo(ilu_string cinfo,
 
 /*L1, L2, Main unconstrained*/
 
-/* This procedure is used internally for creating both true and surrogate objects.  For true objects, this procedure is called with non-NIL ih, server, cl, and lspo; for surrogate objects, this procedure is called with non-NIL ih, server, and sbh.  Storage for ih and sbh is owned by the object, for mstid is owned by the caller. */
+/*
+ * This procedure is used internally for creating both true and
+ * surrogate objects.  For true objects, this procedure is called
+ * with non-NIL ih, server, cl, and lspo; for surrogate objects,
+ * this procedure is called with non-NIL ih, server, and sbh.
+ * Storage for ih, sbh, and mstid is owned by the caller.
+ */
 static ilu_Object CreateObject (ilu_string ih, ilu_Server server,
 				ilu_Class class, ilu_refany lspo,
 				ilu_string sbh, ilu_string mstid)
@@ -401,7 +407,7 @@ static ilu_Object CreateObject (ilu_string ih, ilu_Server server,
   if (server_is_true(server))
     object_lspo(new, server_true_language(server)) = lspo;
   object_holds(new)	= 0;
-  object_intNoted(new)	= FALSE;
+  object_intNoted(new)	= (int) FALSE;
   object_notifying(new)	= FALSE;
   object_known(new)	= FALSE;
   object_gco(new)	= gco;
@@ -425,7 +431,7 @@ static void UncreateObject(ilu_Object o)
 
 ilu_boolean _ilu_Addable(ilu_Server s, ilu_Class t, ilu_Object *h)
 {
-  int i, l;
+  ilu_cardinal i, l;
   if (!class_singleton(t))
       return TRUE;
   if (  (*h = (ilu_Object) _ilu_hash_FindInTable(server_singles(s), t))
@@ -446,10 +452,10 @@ ilu_boolean _ilu_Addable(ilu_Server s, ilu_Class t, ilu_Object *h)
 void 
 _ilu_AddSingleton(ilu_Server s, ilu_Class t, ilu_Object o)
 {
-  int             i, l;
+  ilu_cardinal i, l;
   if (!class_singleton(t))
     return;
-  _ilu_Assert(_ilu_hash_AddToTable(server_singles(s), t, o),
+  _ilu_Assert((int) _ilu_hash_AddToTable(server_singles(s), t, o),
 	      "AddSingleton");
   l = class_superclass_count(t);
   for (i = 0; i < l; i++)
@@ -464,48 +470,49 @@ _ilu_AddSingleton(ilu_Server s, ilu_Class t, ilu_Object o)
   L1 >= {gcmu} if cl collectible*/
 ilu_Object ilu_FindOrCreateTrueObject (ilu_string ih, ilu_Server server,
 					ilu_Class cl, ilu_refany lspo)
-{ 
-  ilu_Object new;
-  if (server == NIL) {
+{
+  ilu_Object      new;
+  if (server == NIL)
     return (NIL);
-  }
   _ilu_Assert(ih != NIL, "CreateTrueObject: ih==NIL");
   _ilu_Assert(lspo != NIL, "CreateTrueObject: lspo==NIL");
   _ilu_Assert(cl != NIL, "CreateTrueObject: class==NIL");
   _ilu_HoldMutex(server_lock(server));
   new = (ilu_Object) _ilu_hash_FindInTable(server_objs(server), ih);
   if (new != NIL) {
-      if (new->ob_class != cl) {
-          DEBUG(OBJECT_DEBUG,
-		(stderr,
-		 "FindOrCreateTrueObject: type of %s/%s is %s, not %s",
-		 server->sr_id, ih, new->ob_class->cl_unique_id,
-		 cl->cl_unique_id));
-          return (NIL);
-        }
-      return (new);
+    if (new->ob_class != cl) {
+      DEBUG(OBJECT_DEBUG,
+	    (stderr,
+	     "FindOrCreateTrueObject: type of %s/%s is %s, not %s",
+	     server->sr_id, ih, new->ob_class->cl_unique_id,
+	     cl->cl_unique_id));
+      return (NIL);
     }
+    return (new);
+  }
   if (server->sr_closing == TRUE) {
-      DEBUG(OBJECT_DEBUG,
-	    (stderr,
-	     "ilu_CreateTrueObject: invoked on closing server %s.\n",
-	     server->sr_id));
-      return (NIL);
-    }
+    DEBUG(OBJECT_DEBUG,
+	  (stderr,
+	   "ilu_CreateTrueObject: invoked on closing server %s.\n",
+	   server->sr_id));
+    return (NIL);
+  }
   if (!_ilu_Addable(server, cl, &new)) {
-      DEBUG(OBJECT_DEBUG,
-	    (stderr,
-	     "%s %s %s of type %s in server %s because %s %s.\n",
-	     "ilu_CreateTrueObject: can't create another",
-	     "singleton object", ih, cl->cl_unique_id, server->sr_id, new->ob_ih,
-	     "already exists"));
-      return (NIL);
-    }
+    DEBUG(OBJECT_DEBUG,
+	  (stderr,
+	   "%s %s %s of type %s in server %s because %s %s.\n",
+	   "ilu_CreateTrueObject: can't create another",
+	   "singleton object", ih, cl->cl_unique_id, server->sr_id,
+	   new->ob_ih, "already exists"));
+    return (NIL);
+  }
   new = CreateObject(ih, server, cl, lspo, NIL, cl->cl_unique_id);
   DEBUG(OBJECT_DEBUG,
-	(stderr, "ilu_FindOrCreateTrueObject (ih=\"%s\", server=\"%s\", class=\"%s\", lspo=%p) => %p\n",
+	(stderr,
+	 "ilu_FindOrCreateTrueObject (ih=\"%s\", server=\"%s\", class=\"%s\", lspo=%p) => %p\n",
 	 ih, server->sr_id, cl->cl_name, lspo, new));
-  _ilu_Assert(_ilu_hash_AddToTable(server_objs(server), ih, new),
+  _ilu_Assert((int) _ilu_hash_AddToTable(server_objs(server),
+					 object_ih(new), new),
 	      "FindOrCreateTrueObject: AddToTable failed");
   _ilu_AddSingleton(server, cl, new);
   if (class_collectible(cl))
@@ -525,9 +532,6 @@ ilu_Object _ilu_FindOrCreateObject (ilu_string ih, ilu_Server s,
   ilu_Object      o = NIL;
   ilu_Class       cl = (found_class == NIL) ? static_type : found_class;
   ilu_boolean     tidfound = (found_class != NIL);
-#ifdef IIOP_PROTOCOL
-  extern ilu_Class _ilu_IIOP_FindClassViaRPC(ilu_Object);
-#endif
 
   ILU_CLER(*err);
 
@@ -561,10 +565,7 @@ ilu_Object _ilu_FindOrCreateObject (ilu_string ih, ilu_Server s,
       if (mstid == NIL || !tidfound) {
 	ilu_ExitServer(s, static_type);
 #ifdef IIOP_PROTOCOL
-	if (sbh != NIL &&
-	    (strncmp(sbh, "IOR:", 4) == 0 ||
-	     strncmp(sbh, "iiop:", 5) == 0 ||
-	     strncmp(sbh, "ior:", 4) == 0) &&
+	if ((strncmp (s->sr_id, "CORBA:", 6) == 0) &&
 	    ih[0] == '$' && ih[1] == 0)
 	  /* non-ILU object; use "-is-a" instead of "ILUGetTypes";
 	     less efficient but works with non-ILU orbs */
@@ -610,7 +611,7 @@ ilu_Object _ilu_FindOrCreateObject (ilu_string ih, ilu_Server s,
 	  ilu_ExitServer(s, static_type);
 	  o = NIL;
 	} else {
-	  _ilu_Assert(_ilu_hash_AddToTable(server_objs(s), object_ih(o), o),
+	  _ilu_Assert((int) _ilu_hash_AddToTable(server_objs(s), object_ih(o), o),
 	       "_ilu_FindOrCreateObject: add to cache (2) failed");
 	  _ilu_AddSingleton(s, object_class(o), o);
 	}
@@ -618,8 +619,8 @@ ilu_Object _ilu_FindOrCreateObject (ilu_string ih, ilu_Server s,
     }
   }
   DEBUG(OBJECT_DEBUG, (stderr,
-	       "_ilu_FindOrCreateObject:  Object of <%s> is %p.\n",
-		       sbh, o));
+	       "_ilu_FindOrCreateObject:  Object of <%s/%s> is %p.\n",
+		s->sr_id, ih, o));
   if (o == NIL) {
   } else if (o->ob_mstid == NIL && mstid != NIL)
     o->ob_mstid = _ilu_Strdup(mstid);
@@ -913,8 +914,8 @@ ilu_string ilu_IhOfObject (ilu_Object obj)
 }
 
 /*L1 >= {cmu, s}; L1_sup < trmu*/
-ILU_ERRS((BadDataStructure, KernelBroken, bad_locks, broken_locks))
-_ilu_DestroyObject(ilu_Server s, ilu_Object obj)
+static ILU_ERRS((BadDataStructure, KernelBroken, bad_locks, broken_locks))
+     DestroyObject(ilu_Server s, ilu_Object obj)
 {
   ilu_Class       c = object_class(obj);
   ilu_Error       err;
@@ -927,7 +928,7 @@ _ilu_DestroyObject(ilu_Server s, ilu_Object obj)
   _ilu_HoldMutex(ilu_cmu);
   _ilu_HoldMutex(server_lock(s));
   DEBUG(OBJECT_DEBUG,
-	(stderr, "_ilu_DestroyObject (server=\"%s\", obj=%p, \"%s\")\n", s->sr_id, obj, obj->ob_ih));
+	(stderr, "DestroyObject (server=\"%s\", obj=%p, \"%s\")\n", s->sr_id, obj, obj->ob_ih));
   if (object_collectible(obj) && server_is_true(s))
     _ilu_StopGCingTrueObj(obj);
   err = _ilu_ServerRemoveObject(s, obj);
@@ -966,14 +967,14 @@ IUpdate(ilu_Object obj, ilu_Class st)
   ilu_Error       ans = ILU_INIT_NO_ERR;
   unsigned int    i;
 
-  if (obj->ob_notifying)
+  if (object_notifying(obj))
     return ILU_NO_ERR;
   if (object_collectible(obj) && !object_is_true(obj)) {
-    register ilu_boolean should = (obj->ob_holds != 0);
+    register ilu_boolean should = (object_holds(obj) != 0);
     for (i = 0; (!should) && (i < _ilu_NLanguages); i++)
       should = (obj->ob_lspos[i] != NIL);
-    obj->ob_notifying = TRUE;
-    while (should != obj->ob_known && ILU_ERROK(ans)) {
+    object_notifying(obj) = TRUE;
+    while (should != object_known(obj) && ILU_ERROK(ans)) {
       ilu_ExitServer(s, st);
       if (should)
 	ans = _ilu_RegisterGCInterest(obj);
@@ -981,31 +982,31 @@ IUpdate(ilu_Object obj, ilu_Class st)
 	ans = _ilu_UnregisterGCInterest(obj);
       ilu_EnterServer(s, st);
       if (ILU_ERROK(ans))
-	obj->ob_known = should;
-      should = (obj->ob_holds != 0);
+	object_known(obj) = should;
+      should = (object_holds(obj) != 0);
       for (i = 0; (!should) && (i < _ilu_NLanguages); i++)
 	should = (obj->ob_lspos[i] != NIL);
     }
-    obj->ob_notifying = FALSE;
+    object_notifying(obj) = FALSE;
   }
   if (ILU_ERRNOK(ans))
     return ans;
-  if (obj->ob_holds != 0)
+  if (object_holds(obj) != 0)
     return ILU_NO_ERR;
   for (i = 0; i < _ilu_NLanguages; i++) {
     if (obj->ob_lspos[i] != NIL)
       return ILU_NO_ERR;
   }
   if (object_is_true(obj) && object_collectible(obj)) {
-    if (obj->ob_gclist != NIL && _ilu_vector_size(obj->ob_gclist) != 0)
+    if (object_gclist(obj) != NIL && _ilu_vector_size(object_gclist(obj)) != 0)
       return ILU_NO_ERR;
-    if (ilu_CoarseTime_Now() < obj->ob_lastRemote + obj->ob_timeout)
+    if (ilu_CoarseTime_Now() < object_lastRemote(obj) + obj->ob_timeout)
       return ILU_NO_ERR;
   }
   DEBUG(OBJECT_DEBUG,
      (stderr, "IUpdate:  deleting uninteresting object %s/%s (%p).\n",
       s->sr_id, obj->ob_ih, obj));
-  return _ilu_DestroyObject(object_server(obj), obj);
+  return DestroyObject(object_server(obj), obj);
 }
 
 /*Inside(obj's server, obj's type)*/
@@ -1017,16 +1018,16 @@ _ilu_VIUpdate(ilu_Object obj)
   int             vi;
   unsigned int    i;
 
-  vi = ((obj->ob_holds != 0)
+  vi = ((object_holds(obj) != 0)
 	|| (object_collectible(obj) &&
 	    (object_is_true(obj)
-	     ? ((obj->ob_gclist != NIL &&
-		 _ilu_vector_size(obj->ob_gclist) != 0)
+	     ? ((object_gclist(obj) != NIL &&
+		 _ilu_vector_size(object_gclist(obj)) != 0)
 		|| (ilu_CoarseTime_Now() <
-		    obj->ob_lastRemote + obj->ob_timeout))
-	     : (obj->ob_notifying && !obj->ob_known))));
-  if (vi != obj->ob_intNoted) {
-    obj->ob_intNoted = vi;
+		    object_lastRemote(obj) + obj->ob_timeout))
+	     : (object_notifying(obj) && !object_known(obj)))));
+  if (vi != object_intNoted(obj)) {
+    object_intNoted(obj) = vi;
     for (i = 0;  i < _ilu_NLanguages;  i++)
       {
 	if (theNoters[i] != NULLFN)
@@ -1065,7 +1066,9 @@ void ilu_SetLSO (ilu_Object obj, ilu_Class t, ilu_refany lso, ilu_cardinal lang)
 void 
 ilu_DHolds(ilu_Object obj, ilu_integer dholds)
 {
-  obj->ob_holds += dholds;
+  ilu_integer new_holds = object_holds(obj) + dholds;
+  _ilu_Assert(new_holds>=0,"holds on object goes negative");
+  object_holds(obj) = new_holds;
   return;
 }
 
@@ -1080,7 +1083,9 @@ ILU_ERRS((BadDataStructure, KernelBroken, GcRegFailed,
 	  bad_locks, broken_locks))
 _ilu_DeltaHolds(ilu_Object obj, ilu_integer dholds)
 {
-  obj->ob_holds += dholds;
+  ilu_integer new_holds = object_holds(obj) + dholds;
+  _ilu_Assert(new_holds>=0,"holds on object goes negative");
+  object_holds(obj) = new_holds;
   return _ilu_VIUpdate(obj);
 }
 
@@ -1226,34 +1231,3 @@ dun:
   }
   return;
 }
-
-/* L1, L2, Main unconstrained */
-
-static ilu_string ServerIdentity = NIL;
-static ilu_string ClientIdentity = NIL;
-
-void ilu_SetServerIdentity (ilu_string identity)
-{
-  ServerIdentity = _ilu_Strdup(identity);
-}
-
-void ilu_SetClientIdentity (ilu_string identity)
-{
-  ClientIdentity = _ilu_Strdup(identity);
-}
-
-ilu_string ilu_GetServerIdentity ()
-{
-  return (ServerIdentity);
-}
-
-ilu_string ilu_GetClientIdentity ()
-{
-  return (ClientIdentity);
-}
-
-void ilu_SetIdentity (ilu_string identity)
-{
-  ilu_SetClientIdentity (identity);
-  ilu_SetServerIdentity (identity);
-}
