@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: sunrpc.c,v 1.164 1996/03/13 02:51:30 janssen Exp $ */
-/* Last edited by Mike Spreitzer February 9, 1996 7:52 am PST */
+/* $Id: sunrpc.c,v 1.178 1996/07/09 23:35:32 janssen Exp $ */
+/* Last edited by Mike Spreitzer June 27, 1996 4:30 pm PDT */
 
 #ifdef MACOS
 #pragma segment ilu
@@ -34,8 +34,11 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 #include "method.h"
 #include "mooring.h"
 
-#define ILU_SUNRPC_PROGRAM_NUMBER	0x00061a78
-#define OLD_ILU_SUNRPC_PROGRAM_NUMBER	0x31000400
+#define ILU_SUNRPC_PROGRAM_NUMBER	0x00061a79
+#define OLD_ILU_SUNRPC_PROGRAM_NUMBER	0x00061a78
+#define OLDER_ILU_SUNRPC_PROGRAM_NUMBER	0x31000400
+
+#define ILU_UNSECURED_GENERIC_IDENTITY	0x000493E2	/* AUTH_UGEN, in Sun Registry */
 
 #define ODD(x)		(((x)&0x1)!=0)
 #define EVEN(x)		(((x)&0x1)==0)  
@@ -49,8 +52,6 @@ XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
 /*L1, L2, Main unconstrained*/
 
-static unsigned char zeros[4] = {0, 0, 0, 0};
-
 #define sunrpc_transport(call)	((call)->ca_prTrans)
 
 static          ilu_boolean
@@ -203,10 +204,10 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
       s->sui_pnumber = ILU_SUNRPC_PROGRAM_NUMBER;
       s->sui_version = _ilu_CRC32 ((ilu_bytes) (class->cl_unique_id), strlen(class->cl_unique_id));
     }
-  _ilu_Assert(_ilu_hash_AddToTable(RegistryHashTable,
+  _ilu_Assert((int) _ilu_hash_AddToTable(RegistryHashTable,
 				   s->sui_type_id, s),
 	      "SunRPC AddToTable RegistryHashTable");
-  _ilu_Assert(_ilu_hash_AddToTable(ProgramNumberHashTable, s, s),
+  _ilu_Assert((int) _ilu_hash_AddToTable(ProgramNumberHashTable, s, s),
 	      "SunRPC AddToTable ProgramNumberHashTable");
   return (s);
 }
@@ -226,8 +227,8 @@ static sunrpcinfo *_sunrpc_SunRPCInformationForClass (ilu_Class class)
 	(stderr, "%s \"%s:%s\", pnumber is 0x%lx, version is %lu.\n",
 	 "_sunrpc_SunRPCInformationForClass:  Class",
 	 class->cl_name, class->cl_unique_id,
-	 (s == NIL) ? 0 : s->sui_pnumber,
-	 (s == NIL) ? 0 : s->sui_version));
+	 (s == NIL) ? 0L : s->sui_pnumber,
+	 (s == NIL) ? 0L : s->sui_version));
 
   return (s);
 }
@@ -246,8 +247,13 @@ static sunrpcinfo *_sunrpc_ClassFromProgramNumber (ilu_cardinal pnumber,
 
   _sunrpc_EnsureRegistries();
 
-  if (pnumber == OLD_ILU_SUNRPC_PROGRAM_NUMBER)
-    dummy.sui_pnumber = ILU_SUNRPC_PROGRAM_NUMBER;
+  if (pnumber == OLD_ILU_SUNRPC_PROGRAM_NUMBER || pnumber == OLDER_ILU_SUNRPC_PROGRAM_NUMBER)
+    {
+      ILU_ERRPRINTF("The specified Sun RPC program number of 0x%x indicates that you are communicating\n"
+		    "with an old ILU client, pre-%s.  You should update that service to ILU %s.\n",
+		    pnumber, ilu_GetILUVersion(), ilu_GetILUVersion());
+      dummy.sui_pnumber = ILU_SUNRPC_PROGRAM_NUMBER;
+    }
   else
     dummy.sui_pnumber = pnumber;
   dummy.sui_version = version;
@@ -457,7 +463,7 @@ _sunrpc_InputShortCardinal(ilu_Call call, ilu_shortcardinal * i,
 
   _sunrpc_InputCardinal (call, &l, err);
   if (ILU_ERROK(*err))
-    *i = l & 0xFFFF;
+    *i = (ilu_shortcardinal) (l & 0xFFFF);
 }
 
 /*ARGSUSED*/
@@ -647,10 +653,7 @@ _sunrpc_OutputBytes(ilu_Call call, ilu_bytes s, ilu_cardinal len,
       return;
     if (ILU_ERROK(*err))
       (void) transport_write_bytes(sunrpc_transport(call), s,
-				   len, err);
-    if (ILU_ERROK(*err) && paddedlen > len)
-      (void) transport_write_bytes(sunrpc_transport(call), zeros,
-				   paddedlen - len, err);
+				   paddedlen, err);
   }
 }
 
@@ -751,7 +754,7 @@ _sunrpc_InputByte(ilu_Call call, ilu_byte * b,
 
   _sunrpc_InputCardinal (call, &l, err);
   if (ILU_ERROK(*err))
-    *b = l & 0xFF;
+    *b = (ilu_byte) (l & 0xFF);
 }
 
 /*ARGSUSED*/
@@ -762,6 +765,30 @@ _sunrpc_SizeOfByte(ilu_Call call, ilu_byte i, ILU_ERRS((IoErrs)) * err)
   return (4);
 }
 
+/* ==================== short char ==================== */
+
+static void
+_sunrpc_OutputShortChar(ilu_Call call, ilu_shortcharacter b,
+			ILU_ERRS((IoErrs)) * err)
+{
+  _sunrpc_OutputByte(call, (ilu_byte) b, err);
+}
+
+static void
+_sunrpc_InputShortChar(ilu_Call call, ilu_shortcharacter * b,
+		       ILU_ERRS((IoErrs)) * err)
+{
+  _sunrpc_InputByte(call, (ilu_byte *) b, err);
+}
+
+/* ARGSUSED */
+static          ilu_cardinal
+_sunrpc_SizeOfShortChar(ilu_Call call, ilu_shortcharacter i,
+			ILU_ERRS((IoErrs)) * err)
+{
+  return _sunrpc_SizeOfByte(call, (ilu_byte) i, err);
+}
+
 /* ==================== opaque ==================== */
 
 static void
@@ -770,10 +797,7 @@ _sunrpc_OutputOpaque(ilu_Call call, ilu_bytes o, ilu_cardinal len,
 {
   struct SunRPC  *d = SUNRPC(call_connection(call));
   ilu_cardinal    paddedlen = PADDED_SIZE(len);
-  (void) transport_write_bytes(sunrpc_transport(call), o, len, err);
-  if (ILU_ERROK(*err) && paddedlen > len)
-    (void) transport_write_bytes(sunrpc_transport(call), zeros,
-				 paddedlen - len, err);
+  (void) transport_write_bytes(sunrpc_transport(call), o, paddedlen, err);
 }
 
 static void
@@ -1053,7 +1077,7 @@ _sunrpc_ReadHeader(ilu_Call call, ilu_PacketType * type,
 	  : (packetType == ilu_PacketType_Reply) ? "reply"
 	  : "unknown type")));
 
-  *type = packetType;
+  *type = (ilu_PacketType) packetType;
   *sn = serialNumber;
   return (ans);
 }
@@ -1091,6 +1115,7 @@ _sunrpc_DiscardMessage(ilu_Call call, ILU_ERRS((internal)) * err)
       return FALSE;
     }
   } ILU_ERR_ENDSWITCH;
+  return ILU_CLER(*err);
 }
 
 static          ilu_boolean
@@ -1116,6 +1141,81 @@ static void
   transport_end_message(sunrpc_transport(call), ilu_FALSE, NIL, err);
 }
 
+/*======================================================================*/
+/*     Sun RPC Auth UNIX Identity type                                  */
+/*======================================================================*/
+
+static ilu_cardinal
+  _ilu_SunRPCAuthUnixIdentity_StringForm (ilu_IdentityInfo info,
+					  char *buffer,
+					  ilu_cardinal bufferlen,
+					  ilu_Error *err)
+{
+  ilu_SunRPCAuthUnixIdentityInfo i = info->ii_info;
+  ilu_cardinal needed;
+
+  needed = 15 + strlen(i->ii_hostname);
+
+  if (bufferlen < needed)
+    return (ILU_ERR_CONS1(bad_param, err, minor, ilu_bpm_small_buffer, needed));
+  else
+    {
+      ILU_CLER(*err);
+      return sprintf(buffer, "[%.5u,%.5u]@%s", i->ii_UID, i->ii_GID, i->ii_hostname);
+    }      
+}
+
+static ilu_refany
+  _ilu_SunRPCAuthUnixIdentity_DuplicateData (ilu_IdentityInfo info,
+					     ilu_Error *err)
+{
+  ilu_SunRPCAuthUnixIdentityInfo i;
+  int j;
+
+  ilu_cardinal needed = sizeof(*i) +
+    strlen(((ilu_SunRPCAuthUnixIdentityInfo)info->ii_info)->ii_hostname) + 1 +
+    ((ilu_SunRPCAuthUnixIdentityInfo)info->ii_info)->ii_ngids * sizeof(ilu_shortcardinal);
+  i = ilu_MallocE(needed, err);
+  if (ILU_ERRNOK(*err))
+    return NIL;
+  i->ii_UID = ((ilu_SunRPCAuthUnixIdentityInfo)(info->ii_info))->ii_UID;
+  i->ii_GID = ((ilu_SunRPCAuthUnixIdentityInfo)(info->ii_info))->ii_UID;
+  i->ii_ngids = ((ilu_SunRPCAuthUnixIdentityInfo)(info->ii_info))->ii_UID;
+  i->ii_gids = (ilu_shortcardinal *) (((char *)i) + sizeof(*i));
+  i->ii_hostname = ((char *)i) + sizeof(*i) + sizeof(ilu_shortcardinal)*i->ii_ngids;
+  strcpy (i->ii_hostname, ((ilu_SunRPCAuthUnixIdentityInfo)(info->ii_info))->ii_hostname);
+  for (j = 0;  j < i->ii_ngids;  j++)
+    i->ii_gids[j] = ((ilu_SunRPCAuthUnixIdentityInfo)(info->ii_info))->ii_gids[j];
+  return i;
+}
+
+static void
+  _ilu_SunRPCAuthUnixIdentity_FreeData (ilu_IdentityInfo info,
+					ilu_Error *err)
+{
+  ilu_SunRPCAuthUnixIdentityInfo i = (ilu_SunRPCAuthUnixIdentityInfo) info->ii_info;
+  ILU_CLER(*err);
+  if ((i->ii_gids == ((ilu_shortcardinal *) (((char *)i) + sizeof(*i)))) &&
+      (i->ii_hostname == ((char *)i) + sizeof(*i)) + (sizeof(ilu_shortcardinal)*(i->ii_ngids)))
+    {
+    }
+  else
+    {
+      ilu_free(i->ii_hostname);
+      ilu_free(i->ii_gids);
+    }
+  ilu_free(i);
+}
+
+struct _ilu_IdentityType_s ilu_SunRPCAuthUnixIdentity_s = {
+  "SunRPCAuthUnixIdentity",
+  _ilu_SunRPCAuthUnixIdentity_StringForm,
+  _ilu_SunRPCAuthUnixIdentity_DuplicateData,
+  _ilu_SunRPCAuthUnixIdentity_FreeData,
+  NULLFN, NULLFN };
+
+
+
 /*L1_sup < prmu*/
 static ilu_Passport 
 ReadCredentialsAndVerifier(ilu_Call call,
@@ -1139,28 +1239,33 @@ ReadCredentialsAndVerifier(ilu_Call call,
     {
       ilu_cardinal stamp;
       ilu_IdentityInfo i;
+      ilu_SunRPCAuthUnixIdentityInfo info;
 
       i = (ilu_IdentityInfo) ilu_MallocE(sizeof(*i), err);
       if (ILU_ERRNOK(*err)) return NIL;
+      info = (ilu_SunRPCAuthUnixIdentityInfo) ilu_MallocE(sizeof(*info), err);
+      if (ILU_ERRNOK(*err)) return NIL;
       i->ii_type = ilu_SunRPCAuthUnixIdentity;
       if (_sunrpc_InputCardinal (call, &credSize, err), ILU_ERRNOK(*err)) return NIL;
       if (_sunrpc_InputCardinal (call, &stamp, err), ILU_ERRNOK(*err)) return NIL;
-      i->ii_info.ii_SunRPCAuthUnixInfo.ii_hostname = NIL;
-      _sunrpc_InputString(call, &i->ii_info.ii_SunRPCAuthUnixInfo.ii_hostname, &stamp, 255, err);
+      info->ii_hostname = NIL;
+      _sunrpc_InputString(call, &info->ii_hostname, &stamp, 255, err);
       if (ILU_ERRNOK(*err)) return NIL;
-      _sunrpc_InputShortCardinal(call, &i->ii_info.ii_SunRPCAuthUnixInfo.ii_UID, err);
+      _sunrpc_InputShortCardinal(call, &info->ii_UID, err);
       if (ILU_ERRNOK(*err)) return NIL;
-      _sunrpc_InputShortCardinal(call, &i->ii_info.ii_SunRPCAuthUnixInfo.ii_GID, err);
+      _sunrpc_InputShortCardinal(call, &info->ii_GID, err);
       if (ILU_ERRNOK(*err)) return NIL;
-      _sunrpc_InputShortCardinal(call, &i->ii_info.ii_SunRPCAuthUnixInfo.ii_ngids, err);
+      _sunrpc_InputShortCardinal(call, &info->ii_ngids, err);
       if (ILU_ERRNOK(*err)) return NIL;
-      i->ii_info.ii_SunRPCAuthUnixInfo.ii_gids = (ilu_shortcardinal *) ilu_MallocE(i->ii_info.ii_SunRPCAuthUnixInfo.ii_ngids * sizeof(ilu_shortcardinal), err);
+      info->ii_gids = (ilu_shortcardinal *) ilu_MallocE(info->ii_ngids * sizeof(ilu_shortcardinal), err);
       if (ILU_ERRNOK(*err)) return NIL;
-      for (stamp = 0;  stamp < i->ii_info.ii_SunRPCAuthUnixInfo.ii_ngids;  stamp++)
+      for (stamp = 0;  stamp < info->ii_ngids;  stamp++)
 	{
-	  _sunrpc_InputShortCardinal(call, &i->ii_info.ii_SunRPCAuthUnixInfo.ii_gids[stamp], err);
+	  _sunrpc_InputShortCardinal(call, &info->ii_gids[stamp], err);
 	  if (ILU_ERRNOK(*err)) return NIL;
 	}
+      i->ii_info = (ilu_refany) info;
+      i->ii_owned_by_passport = ilu_TRUE;
       if (passport == NIL)
 	{
 	  passport = call->ca_caller = ilu_CreatePassport(i, err);
@@ -1174,10 +1279,54 @@ ReadCredentialsAndVerifier(ilu_Call call,
       if (ILU_ERRNOK(*err)) return passport;
       DEBUG(AUTHENTICATION_DEBUG,
 	    (stderr, "Sun RPC UNIX auth:  [%ld, %ld] on \"%s\", %ld groups\n",
-	     (long) i->ii_info.ii_SunRPCAuthUnixInfo.ii_UID,
-	     (long) i->ii_info.ii_SunRPCAuthUnixInfo.ii_GID,
-	     i->ii_info.ii_SunRPCAuthUnixInfo.ii_hostname,
-	     (long) i->ii_info.ii_SunRPCAuthUnixInfo.ii_ngids));
+	     (long) info->ii_UID,
+	     (long) info->ii_GID,
+	     info->ii_hostname,
+	     (long) info->ii_ngids));
+    }
+  else if (credType == ILU_UNSECURED_GENERIC_IDENTITY)
+    {
+      ilu_string idtypename = NIL;
+      ilu_bytes id_pickled = NIL;
+      ilu_cardinal id_pickled_len;
+      ilu_IdentityInfo info;
+      ilu_IdentityType id_type;
+
+      _sunrpc_InputCardinal (call, &credSize, err);
+      if (ILU_ERRNOK(*err)) return NIL;
+      _sunrpc_InputString (call, &idtypename, &credSize, 0xFFFF, err);
+      if (ILU_ERRNOK(*err)) return NIL;
+      _sunrpc_InputBytes (call, &id_pickled, &id_pickled_len, 0xFFFF, err);
+      if (ILU_ERRNOK(*err)) { ilu_free(idtypename); return NIL; }
+      _sunrpc_InputCardinal(call, &verifierType, err);
+      if (ILU_ERRNOK(*err)) { ilu_free(idtypename); ilu_free(id_pickled); return NIL; }
+      _sunrpc_InputBytes(call, &verifierBuf, &verifierSize, 0, err);
+      if (ILU_ERRNOK(*err)) { ilu_free(idtypename); ilu_free(id_pickled); return NIL; }
+      ilu_free(verifierBuf);
+
+      DEBUG(AUTHENTICATION_DEBUG,
+	    (stderr, "_sunrpc_InterpretRequest:  "
+	     "identity of type \"%s\" encountered.\n",
+	     idtypename));
+
+      if ((id_type = ilu_FindIdentityTypeByName (idtypename, err)) == NIL)
+	{ DEBUG(AUTHENTICATION_DEBUG,
+		(stderr, "_sunrpc_InterpretRequest:  "
+		 "identity type \"%s\" unknown in this address space.\n",
+		 idtypename));
+	  ilu_free(idtypename); ilu_free(id_pickled); return NIL; }
+      else if ((info = (*id_type->it_unpickle)(id_pickled, id_pickled_len, err)) == NIL)
+	{ ilu_free(id_pickled); return NIL; }
+      ilu_free(id_pickled);
+      info->ii_owned_by_passport = ilu_TRUE;
+      if (passport == NIL)
+	{
+	  passport = call->ca_caller = ilu_CreatePassport(info, err);
+	  if (ILU_ERRNOK(*err))
+	    { (*id_type->it_free_data)(info->ii_info, err); ilu_free(info); return NIL; }
+	}
+      else if (!ilu_AddIdentity (passport, info, err))
+	{ (*id_type->it_free_data)(info->ii_info, err); ilu_free(info); return NIL; }
     }
   else {
     _sunrpc_InputBytes(call, &credBuf, &credSize, 0, err);
@@ -1382,14 +1531,20 @@ static ilu_boolean GetSunRPCProgramNumberAndVersion (ilu_Class pclass,
 
 #include "os/posix.h"
 
+#define MAX_GIDS 100
 static int nameset = 0;
 static char hostname[255];
 static int hostnamelen, hostnamelenr, credlen;
 static int gid, uid, ngids;
-static gid_t gids[100];
+#ifdef HAS_SOLARIS1_GID_T_SIZE_BUG
+static unsigned int gids[MAX_GIDS];
+#else
+static gid_t gids[MAX_GIDS];
+#endif
 #ifdef _IS_BSD
 #include <netinet/in.h>
 #include <netdb.h>
+#include <arpa/inet.h>	/* for inet_ntoa */
 #endif /* def _IS_BSD */
 #endif /* _IS_POSIX */
 
@@ -1400,6 +1555,9 @@ static int credlenr = 4 * 2;
 static void FigureCredentialSize (ilu_Call call)
 {
 #ifdef _IS_POSIX
+  if (getenv("ILU_NO_SUNRPC_UNIX_AUTH") != NIL)
+    return;
+
   if (!nameset) {
 
     _ilu_AcquireMutex(ilu_prmu);
@@ -1422,7 +1580,7 @@ static void FigureCredentialSize (ilu_Call call)
       hostnamelenr = 4 * ((hostnamelen + 3) / 4);
       gid = getgid();
       uid = geteuid();
-      ngids = getgroups(sizeof(gids) / sizeof(gid), gids);
+      ngids = getgroups(MAX_GIDS, gids);
       _ilu_Assert(ngids >= 0, "getgroups() failed");
       credlen = 4 /* stamp */ + 4 + hostnamelenr	/* machinename */
 	+ 4 /* uid */ + 4 /* gid */ + 4 + 4 * ngids /* gids */ ;
@@ -1436,51 +1594,92 @@ static void FigureCredentialSize (ilu_Call call)
 static void
 WriteCredentials(ilu_Call call, ILU_ERRS((IoErrs)) * err)
 {
-#ifdef _IS_POSIX
-  Output_Cardinal(call, 1, err);	/* UNIX credentials follow */
-  if (ILU_ERRNOK(*err))
-    return;
-  Output_Cardinal(call, credlen, err);	/* credentials length */
-  if (ILU_ERRNOK(*err))
-    return;
-  Output_Cardinal(call, 1, err);	/* stamp */
-  if (ILU_ERRNOK(*err))
-    return;
-  _sunrpc_OutputString(call, hostname, hostnamelen, 255, err);
-  if (ILU_ERRNOK(*err))
-    return;
-  Output_Cardinal(call, uid, err);
-  if (ILU_ERRNOK(*err))
-    return;
-  Output_Cardinal(call, gid, err);
-  if (ILU_ERRNOK(*err))
-    return;
-  Output_Cardinal(call, ngids, err);
-  if (ILU_ERRNOK(*err))
-    return;
-  {
-    int             i;
-    for (i = 0; i < ngids; i++) {
-      Output_Cardinal(call, gids[i], err);
+  ilu_string identity_type;
+  ilu_IdentityType id_type;
+  ilu_IdentityInfo id_info;
+  ilu_cardinal len, size;
+  ilu_bytes buffer;
+
+  if (((identity_type = getenv("ILU_SUNRPC_PREFERRED_IDENTITY")) != NIL) &&
+      ((id_type = ilu_FindIdentityTypeByName(identity_type, err)) != NIL) &&
+      (id_type->it_pickle != NULLFN) &&
+      (call->ca_caller != NIL) &&
+      ((id_info = ilu_FindIdentity(call->ca_caller, id_type)) != NIL) &&
+      (len = (*id_type->it_pickle)(id_info, &buffer, err), ILU_ERROK(*err)))
+    {
+      Output_Cardinal(call, ILU_UNSECURED_GENERIC_IDENTITY, err);
+      if (ILU_ERRNOK(*err))
+	return;
+      size = strlen(id_type->it_name);
+      size = 4 + 4 + PADDED_SIZE(size) + PADDED_SIZE(len);
+      Output_Cardinal(call, size, err);
+      if (ILU_ERRNOK(*err))
+	return;
+      _sunrpc_OutputString (call, id_type->it_name, strlen(id_type->it_name), 0xFFFF, err);
+      if (ILU_ERRNOK(*err))
+	return;
+      _sunrpc_OutputBytes (call, buffer, len, 0xFFFF, err);
+      ilu_free(buffer);
+      if (ILU_ERRNOK(*err))
+	return;
+      Output_Cardinal(call, 0, err);	/* verifier:  AUTH_NULL */
+      if (ILU_ERRNOK(*err))
+	return;
+      Output_Cardinal(call, 0, err);	/* 0 bytes */
       if (ILU_ERRNOK(*err))
 	return;
     }
-  }
 
-  Output_Cardinal(call, 0, err);	/* verifier:  AUTH_NULL */
-  if (ILU_ERRNOK(*err))
-    return;
-  Output_Cardinal(call, 0, err);	/* 0 bytes */
-  if (ILU_ERRNOK(*err))
-    return;
+#ifdef _IS_POSIX
 
-#else
-
-  static unsigned char credAndVerf[16] = {0,};
-  (void) transport_write_bytes(sunrpc_transport(call), credAndVerf, 16, err);
+  else if (getenv("ILU_NO_SUNRPC_UNIX_AUTH") == NIL)
+    {
+      Output_Cardinal(call, 1, err);	/* UNIX credentials follow */
+      if (ILU_ERRNOK(*err))
+	return;
+      Output_Cardinal(call, credlen, err);	/* credentials length */
+      if (ILU_ERRNOK(*err))
+	return;
+      Output_Cardinal(call, 1, err);	/* stamp */
+      if (ILU_ERRNOK(*err))
+	return;
+      _sunrpc_OutputString(call, hostname, hostnamelen, 255, err);
+      if (ILU_ERRNOK(*err))
+	return;
+      Output_Cardinal(call, (ilu_cardinal) uid, err);
+      if (ILU_ERRNOK(*err))
+	return;
+      Output_Cardinal(call, (ilu_cardinal) gid, err);
+      if (ILU_ERRNOK(*err))
+	return;
+      Output_Cardinal(call, (ilu_cardinal) ngids, err);
+      if (ILU_ERRNOK(*err))
+	return;
+      {
+	int             i;
+	for (i = 0; i < ngids; i++) {
+	  Output_Cardinal(call, (ilu_cardinal) (gids[i]), err);
+	  if (ILU_ERRNOK(*err))
+	    return;
+	}
+      }
+      
+      Output_Cardinal(call, 0, err);	/* verifier:  AUTH_NULL */
+      if (ILU_ERRNOK(*err))
+	return;
+      Output_Cardinal(call, 0, err);	/* 0 bytes */
+      if (ILU_ERRNOK(*err))
+	return;
+    }
 
 #endif				/* def _IS_POSIX */
 
+  else
+    {
+      static unsigned char credAndVerf[16] = {0,};
+      (void) transport_write_bytes(sunrpc_transport(call), credAndVerf, 16, err);
+    }
+
 }
 
 /*Main Invariant holds; L2 >= {call's conn's callmu, iomu}*/
@@ -1731,6 +1930,7 @@ static ilu_Protocol _sunrpc_NewSunRPC (void)
 		     ilu_must_malloc(sizeof(struct _ilu_Protocol_s));
 
   new->pr_concurrent_requests = ilu_FALSE;
+  new->pr_sizing_required = ilu_FALSE;
 
   new->pr_init_call = _sunrpc_InitCall;
   new->pr_start_request = _sunrpc_StartRequest;
@@ -1742,6 +1942,7 @@ static ilu_Protocol _sunrpc_NewSunRPC (void)
   new->pr_begin_exception = _sunrpc_BeginException;
   new->pr_finish_exception = _sunrpc_FinishException;
   new->pr_finish_call = _sunrpc_FinishCall;
+  new->pr_prefinish_call = NULLFN;
 
   new->pr_read_header = _sunrpc_ReadHeader;
   new->pr_delay_interp = _sunrpc_DelayInterp;
@@ -1823,14 +2024,18 @@ static ilu_Protocol _sunrpc_NewSunRPC (void)
 	(ilu_cardinal (*)(ilu_Call,ilu_boolean,ilu_Error*))
 	_sunrpc_SizeOfCardinal;
 
-  new->pr_output_shortchar = _sunrpc_OutputByte;
-  new->pr_input_shortchar = _sunrpc_InputByte;
-  new->pr_size_of_shortchar = _sunrpc_SizeOfByte;
+  new->pr_output_shortchar = _sunrpc_OutputShortChar;
+  new->pr_input_shortchar = _sunrpc_InputShortChar;
+  new->pr_size_of_shortchar = _sunrpc_SizeOfShortChar;
 
   new->pr_output_string = _sunrpc_OutputString;
   new->pr_input_string = _sunrpc_InputString;
   new->pr_size_of_string = _sunrpc_SizeOfString;
 
+  new->pr_output_wstring = _ilu_OutputWString;
+  new->pr_input_wstring = _ilu_InputWString;
+  new->pr_size_of_wstring = _ilu_SizeOfWString;
+
   new->pr_output_bytes = _sunrpc_OutputBytes;
   new->pr_input_bytes = _sunrpc_InputBytes;
   new->pr_size_of_bytes = _sunrpc_SizeOfBytes;
@@ -1839,6 +2044,10 @@ static ilu_Protocol _sunrpc_NewSunRPC (void)
   new->pr_input_opaque = _sunrpc_InputOpaque;
   new->pr_size_of_opaque = _sunrpc_SizeOfOpaque;
 
+  new->pr_output_wstringvec = _ilu_OutputWStringVec;
+  new->pr_input_wstringvec = _ilu_InputWStringVec;
+  new->pr_size_of_wstringvec = _ilu_SizeOfWStringVec;
+
   new->pr_output_object_id = _ilu_OutputObjectID;
   new->pr_input_object_id = _ilu_InputObjectID;
   new->pr_size_of_object_id = _ilu_SizeOfObjectID;
