@@ -14,8 +14,8 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: sunrpcrm.c,v 1.24 1996/02/19 20:12:55 spreitze Exp $ */
-/* Last edited by Mike Spreitzer February 19, 1996 11:53 am PST */
+/* $Id: sunrpcrm.c,v 1.28 1996/06/06 19:18:23 spreitze Exp $ */
+/* Last edited by Mike Spreitzer June 6, 1996 10:37 am PDT */
 
 #include "iluntrnl.h"
 
@@ -205,7 +205,7 @@ _sunrpcrm_BeginMessage(ilu_Transport self,
       if (p->inLength == 0) {
 	_ilu_Assert(p->inChunkRem == 0 && self->tr_inLimit == p->inLength,
 		    "sunrpcrm.c:BeginMessage");
-	p->busyIn = FALSE;
+	p->busyIn = FALSE; 
 	return ilu_rhrc_nothing;
       }
     }
@@ -297,7 +297,7 @@ _sunrpcrm_WriteBytes(ilu_Transport self, ilu_bytes b,
   if (!p->busyOut)
     return ILU_ERR_CONS1(internal, err, minor, ilu_im_bytesWithoutMsg,
 			 FALSE);
-  direct = bufferSize >= DIRECT_THRESHOLD;
+  direct = (bufferSize >= DIRECT_THRESHOLD);
   if (direct) {
     ilu_cardinal    l1 = self->tr_outNext - p->outStart;
     if (l1 > 0) {
@@ -336,8 +336,9 @@ _sunrpcrm_WriteBytes(ilu_Transport self, ilu_bytes b,
     self->tr_outNext += l1;
     b += l1;
     bufferSize -= l1;
-    if (bufferSize > 0) {
-      /* Gotta write buffer to make room for more. */
+    if (bufferSize > 0 || (self->tr_outLimit - self->tr_outNext) < 16) {
+      /* Gotta write buffer to make room for more, or to make sure we
+	 always have room for 16 more bytes in the buffer. */
       /* Possible improvement: avoid writing 0-length chunk here. */
       FORMAT_HEADER(self->tr_outBuff + p->outStart - 4,
 		    self->tr_outNext - p->outStart, FALSE);
@@ -353,6 +354,8 @@ _sunrpcrm_WriteBytes(ilu_Transport self, ilu_bytes b,
       self->tr_outNext = 4 + bufferSize;
     }
   }
+  _ilu_Assert((self->tr_outLimit - self->tr_outNext)>=16,
+	      "_sunrpcrm_WriteBytes failed to provide at least 16 bytes in output buffer");
   return ILU_CLER(*err);
 }
 
@@ -655,13 +658,14 @@ _sunrpcrm_CreateMooring(ilu_TransportCreator self,
 			ilu_TransportInfo * tinfo_out,
 			ilu_boolean buffer,
 			ilu_integer *dfd,
+			ilu_Passport pp,	/* unused in this transport */
 			ILU_ERRS((no_memory)) * err)
 {
   CreatorParms    cp = (CreatorParms) self->tcr_data;
   ilu_Mooring     lower, ans;
   ilu_TransportInfo      subtinfo = NIL;
   lower = ((*cp->lower->tcr_createMooring)
-	   (cp->lower, tinfo_out ? &subtinfo : NIL, FALSE, dfd, err));
+	   (cp->lower, tinfo_out ? &subtinfo : NIL, FALSE, dfd, pp, err));
   if (ILU_ERRNOK(*err))
     return NIL;
   ans = (ilu_Mooring) ilu_malloc(sizeof(*ans));
