@@ -17,7 +17,7 @@
  * IF XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH
  * DAMAGES.
  */
-/* $Id: type.c,v 1.66 1996/03/08 19:53:13 spreitze Exp $ */
+/* $Id: type.c,v 1.68 1996/07/02 03:48:36 janssen Exp $ */
 /* Last edited by Mike Spreitzer March 8, 1996 9:08 am PST */
 
 #define _POSIX_SOURCE
@@ -170,13 +170,13 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
 					     _ilu_hash_StringCompare);
 
   if (_ilu_hash_FindInTable(ClassNameTable, class_name(class)) == NIL)
-    _ilu_Assert(_ilu_hash_AddToTable(ClassNameTable, class_name(class),
+    _ilu_Assert((int) _ilu_hash_AddToTable(ClassNameTable, class_name(class),
 				     class),
 		"RegisterClass AddToTable ClassNameTable");
   new = NIL == _ilu_hash_FindInTable(ClassIDTable,
 				      class_unique_id(class));
   if (new)
-    _ilu_Assert(_ilu_hash_AddToTable(ClassIDTable,
+    _ilu_Assert((int) _ilu_hash_AddToTable(ClassIDTable,
 				   class_unique_id(class), class),
 		"RegisterClass AddToTable ClassIDTable");
 
@@ -205,7 +205,7 @@ by a pre-1.6.4-p8 stubber.  Please re-stub, re-compile, and re-link.\n",
 			       class_superclass_id(class, index));
 	if (v == NIL) {
 	  v = _ilu_vector_new(1);
-	  _ilu_Assert(_ilu_hash_AddToTable(UnlinkedClasses,
+	  _ilu_Assert((int) _ilu_hash_AddToTable(UnlinkedClasses,
 				class_superclass_id(class, index),
 					   v),
 		      "RegisterClass AddToTable UnlinkedClasses");
@@ -342,7 +342,7 @@ ilu_DefineException(char *i, char *e,
       errtab = _ilu_hash_MakeNewTable(CLASS_HASHTABLESIZE,
 				      _ilu_hash_HashString,
 				      _ilu_hash_StringCompare);
-      _ilu_Assert(_ilu_hash_AddToTable(ifcErrors, i, errtab),
+      _ilu_Assert((int) _ilu_hash_AddToTable(ifcErrors, i, errtab),
 		  "ifcErrors");
     }
   }
@@ -352,7 +352,7 @@ ilu_DefineException(char *i, char *e,
       ans = _ilu_Strdup(e);
     else
       ans = _ilu_Strcat5("ilu:", i, ".", e, "");
-    _ilu_Assert(_ilu_hash_AddToTable(errtab, e, ans), "errtab");
+    _ilu_Assert((int) _ilu_hash_AddToTable(errtab, e, ans), "errtab");
   }
   ILU_CLER(*err);
   return ans;
@@ -759,7 +759,7 @@ ClassNamesToID_DAG(ilu_string * names)
   if (n == NIL) {
     ilu_string      id = _ilu_Strdup(*names);
     n = (ID_DAG) ilu_malloc(sizeof(ID_DAG_s));
-    _ilu_Assert(_ilu_hash_AddToTable(UnknownTypeIDs, id, n),
+    _ilu_Assert((int) _ilu_hash_AddToTable(UnknownTypeIDs, id, n),
 		"ClassNamesToID_DAG AddToTable UnknownTypeIDs");
     n->id = id;
     n->it = (ilu_Class) _ilu_hash_FindInTable(ClassIDTable, id);
@@ -904,9 +904,10 @@ StringToClass(ilu_string s)
   ans = t->mska;
   _ilu_ReleaseMutex(ilu_otmu);
   if (ans == NIL) {
-    ILU_ERRPRINTF(
-	    "(StringToClass):  %s ancestors of type <%s> are known!  Unable to resolve type.\n",
-	    t->saak ? "Multiple" : "No", s);
+    DEBUG(OBJECT_DEBUG,
+	  (stderr, 
+	   "(StringToClass):  %s ancestors of type <%s> are known!",
+	   t->saak ? "Multiple" : "No", s));
   } else {
     DEBUG(OBJECT_DEBUG,
 	  (stderr, "(StringToClass):  Found class %s.\n",
@@ -1012,6 +1013,15 @@ _ilu_FindClassViaRPC(ilu_Object o)
 
   if (types != NIL) {
     c = StringToClass(types);
+#ifdef ENABLE_DEBUGGING
+    if (c == NIL) {
+      ilu_DebugPrintf("ILU Error:  Unable to determine the object type of object <%s/%s>.\n",
+		      server_id(object_server(o)), object_ih(o));
+      ilu_DebugPrintf("            The specified type codes <%s> are unknown in this address space.\n",
+		      types);
+      ilu_DebugPrintf("            This may indicate either an ILU version mismatch, or an ISL interface version mismatch.\n");
+    };
+#endif
     FREETOKEN(types);
   }
   DEBUG(OBJECT_DEBUG,
