@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */
-/* $Id: udp.c,v 1.52 1996/03/13 04:57:07 janssen Exp $ */
+/* $Id: udp.c,v 1.55 1996/05/04 04:54:12 janssen Exp $ */
 /* Last edited by Mike Spreitzer February 19, 1996 11:31 pm PST */
 
 #include "iluntrnl.h"
@@ -158,7 +158,7 @@ static int add_fcntl_flag (int fd, int dflags)
 #define set_listener_non_blocking(fd)	add_fcntl_flag((fd), O_NONBLOCK)
 #define set_doer_non_blocking(fd)	add_fcntl_flag((fd), O_NONBLOCK)
 
-#elif defined(_IS_BSD)
+#elif (defined(_IS_BSD) && defined(FIONREAD))
 
 static int set_doer_non_blocking (int fd)
 {
@@ -167,6 +167,14 @@ static int set_doer_non_blocking (int fd)
 }
 #define set_listener_non_blocking	set_doer_non_blocking
 
+#elif (defined(_IS_BSD) && defined(HAS_SOLARIS1_NONBLOCKING_BUG))
+static int set_doer_non_blocking (int fd)
+{
+  int one = 1;
+  return (OS_SOCKIOCTL(fd, 0x8004667e, &one));
+}
+#define set_listener_non_blocking	set_doer_non_blocking
+
 #else	/* not POSIX or BSD or Windows */
 
 #error "Don't know how to do non-blocking I/O for anything but POSIX, Windows, and BSD just now"
@@ -329,13 +337,14 @@ static Sender NewSender(ilu_integer *dfd, ILU_ERRS((IoErrs)) * err)
 	(stderr, "udp:  Created new outgoing socket, FD = %d\n",
 	 fd));
   ans->fd = fd;
-  _ilu_Assert(_ilu_hash_AddToTable(senders, &ans->fd, ans),
+  _ilu_Assert((int) _ilu_hash_AddToTable(senders, &ans->fd, ans),
 	      "UDP NewSender");
   return ans;
 }
 
 /*L1 >= {trmu}*/
-ilu_boolean UseSenderP(ilu_refany entry_data, ilu_refany rock)
+static ilu_boolean
+  UseSenderP(ilu_refany entry_data, ilu_refany rock)
 {
   Sender          sender = (Sender) entry_data;
   INetSockAddr   *paddr = (INetSockAddr *) rock;
@@ -652,7 +661,7 @@ SendWholeMessage(ilu_Transport tr,
 	(stderr,
 	 "udp(%p):  (re-)sending %lu bytes from %p to fd %d.\n",
 	 tr, msgh->msg_len, msgh->msg_base, parms->fd));
-  sent = sendto(parms->fd, (char *) buffer, count, 0,
+  sent = sendto(parms->fd, (char *) buffer, (int) count, 0,
 		(struct sockaddr *) & parms->addr,
 		sizeof(parms->addr));
   if (sent < 0 || count != (ilu_cardinal) sent) {
@@ -815,7 +824,7 @@ CreateTransport(ilu_TransportCreator tcr,
     else
       tParms->change = NIL;
     ILU_MUST_BE_SUCCESS(*err);
-    _ilu_Assert(_ilu_hash_AddToTable(tParms->sender->perpeers,
+    _ilu_Assert((int) _ilu_hash_AddToTable(tParms->sender->perpeers,
 				     &tParms->addr, tParms),
 		"UDP CreateTransport hash_AddToTable");
   }
@@ -1068,11 +1077,11 @@ static struct _ilu_Mooring_s mooringProto = {
 /*L1_sup < trmu; L2 unconstrained*/
 static          ilu_Mooring
 CreateMooring(ilu_TransportCreator tcr, ilu_TransportInfo * tinfo_out,
-	      ilu_boolean buffer, ilu_integer *dfd,
+	      ilu_boolean buffer, ilu_integer *dfd, ilu_Passport pp /* unused */,
 	      ILU_ERRS((IoErrs)) * err)
 {
   CreatorParms    cParms = (CreatorParms) tcr->tcr_data;
-  INetSockAddr    sin = cParms->addr;
+  INetSockAddr    mysin = cParms->addr;
   MooringParms    mParms = NIL;
   int             skt = -1;
   SOCKET_SIZE_TYPE namelen;
@@ -1085,12 +1094,12 @@ CreateMooring(ilu_TransportCreator tcr, ilu_TransportInfo * tinfo_out,
   *dfd = 1;
   while (1) {
     int             err1;
-    if (!OS_SOCKERR(bind(skt, (struct sockaddr *) & sin, sizeof(sin))))
+    if (!OS_SOCKERR(bind(skt, (struct sockaddr *) & mysin, sizeof(mysin))))
       break;
     err1 = sockerrno;
     DEBUG((EXPORT_DEBUG | UDP_DEBUG),
 	  (stderr, "udp(%p): bind to port %lx:%u failed:  %s.\n",
-	   tcr, ntohl(sin.sin_addr.s_addr),
+	   tcr, ntohl(mysin.sin_addr.s_addr),
 	   ntohs(cParms->addr.sin_port), strerror(err1)));
     if (err1 == SOCKERRID(ADDRNOTAVAIL) && !loopedback) {
       /*
@@ -1099,7 +1108,7 @@ CreateMooring(ilu_TransportCreator tcr, ilu_TransportInfo * tinfo_out,
        * bind to INADDR_ANY.  So we bind to that, and report
        * "127.0.0.1" (the loopback address) in the tinfo_out.
        */
-      sin.sin_addr.s_addr = INADDR_ANY;
+      mysin.sin_addr.s_addr = INADDR_ANY;
       loopedback = TRUE;
       goto tryagain;
     }
@@ -1109,9 +1118,9 @@ CreateMooring(ilu_TransportCreator tcr, ilu_TransportInfo * tinfo_out,
 tryagain:
     0;
   }
-  if (sin.sin_port == 0) {
-    namelen = sizeof(sin);
-    if (OS_SOCKERR(getsockname(skt, (struct sockaddr *) & sin,
+  if (mysin.sin_port == 0) {
+    namelen = sizeof(mysin);
+    if (OS_SOCKERR(getsockname(skt, (struct sockaddr *) & mysin,
 			       &namelen))) {
       DEBUG((EXPORT_DEBUG | UDP_DEBUG),
 	    (stderr, "udp: getsockname failed:  %s.\n",
@@ -1131,7 +1140,7 @@ tryagain:
     else
       hostname = "127.0.0.1";
 
-    tinfo = FormHandle(hostname, ntohs(sin.sin_port), FALSE, err);
+    tinfo = FormHandle(hostname, ntohs(mysin.sin_port), FALSE, err);
     if (ILU_ERRNOK(*err))
       goto faild;
     *tinfo_out = _ilu_ConcatTinfo(tinfo, (ilu_TransportInfo) & t, err);
@@ -1151,10 +1160,10 @@ tryagain:
   }
   DEBUG((EXPORT_DEBUG | UDP_DEBUG),
 	(stderr, "udp(%p): new mooring on FD %d, addr %s, port %u.\n",
-	 ans, skt, inet_ntoa(sin.sin_addr), ntohs(sin.sin_port)));
+	 ans, skt, inet_ntoa(mysin.sin_addr), ntohs(mysin.sin_port)));
   *ans = mooringProto;
   ans->mo_data = mParms;
-  mParms->addr = sin;
+  mParms->addr = mysin;
   mParms->fd = skt;
   mParms->tih = NIL;
   return (ans);
