@@ -14,9 +14,13 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: iluftobject.h,v 1.4 1994/09/13 19:38:24 severson Exp $
+$Id: iluftobject.h,v 1.6 1996/07/09 23:05:27 larner Exp $
 */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* ILU FineTime Object implementation */
 
 typedef struct
@@ -31,3 +35,7 @@ extern PyTypeObject	Iluft_Type;
 
 extern PyObject *	iluft_FromFineTime(ilu_FineTime ft);
 extern ilu_FineTime	iluft_AsFineTime(PyObject *self);
+
+#ifdef __cplusplus
+}
+#endif
