@@ -14,9 +14,13 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: ilulpobject.h,v 1.1 1996/02/28 03:19:43 janssen Exp $
+$Id: ilulpobject.h,v 1.4 1996/07/09 23:09:27 larner Exp $
 */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* ILU Loop Toggle implementation */
 
 typedef struct
@@ -29,4 +33,8 @@ extern PyTypeObject	Ilulp_Type;
 
 #define ilulp_Check(op)	((op)->ob_type == &Ilulp_Type)
 
-extern PyObject *	ilulp_Create();
+extern PyObject *	ilulp_Create(void);
+
+#ifdef __cplusplus
+}
+#endif
