@@ -14,9 +14,13 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: ilulrobject.h,v 1.2 1995/10/06 02:36:46 janssen Exp $
+$Id: ilulrobject.h,v 1.4 1996/07/09 23:06:09 larner Exp $
 */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* ILU Long Real Object implementation */
 
 typedef struct
@@ -34,3 +38,7 @@ extern PyTypeObject	Ilulr_Type;
 
 extern PyObject *	ilulr_FromLongReal(ilu_longreal lr);
 extern PyObject *	ilulr_FromDouble(double dval);
+
+#ifdef __cplusplus
+}
+#endif
