@@ -14,10 +14,13 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: ivobject.h,v 1.4 1994/11/11 01:08:06 severson Exp $
+$Id: ivobject.h,v 1.6 1996/07/09 23:10:58 larner Exp $
 */
 
 /* Instance Variables Object implementation */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 typedef struct
 {
@@ -34,3 +37,7 @@ extern PyTypeObject	Iv_Type;
 #define iv_Check(op)	((op)->ob_type == &Iv_Type)
 
 extern PyObject *	iv_New(void);
+
+#ifdef __cplusplus
+}
+#endif
