@@ -14,9 +14,13 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: thcobject.h,v 1.4 1995/10/27 01:32:19 janssen Exp $
+$Id: thcobject.h,v 1.6 1996/07/09 23:11:33 larner Exp $
 */
 
+#ifdef __cplusplus
+extern "C"
+{
+#endif
 /* Timer Handler Closure Object implementation */
 
 typedef void (*TimerHandler)(void * rock);
@@ -35,3 +39,6 @@ extern PyTypeObject	Thc_Type;
 
 extern PyObject *	thc_New(ilu_FineTime time, TimerHandler handler, ilu_private rock);
 
+#ifdef __cplusplus
+}
+#endif
