@@ -1,20 +1,20 @@
 /*
- ** Copyright (c) 1991-1994 Xerox Corporation.  All Rights Reserved.
+ ** Copyright (c) 1991-1996 Xerox Corporation.  All Rights Reserved.
  **
  ** Unlimited use, reproduction, and distribution of this software is
  ** permitted.  Any copy of this software must include both the above
  ** copyright notice of Xerox Corporation and this paragraph.  Any
  ** distribution of this software must comply with all applicable United
- ** States export control laws.  This software is made available AS IS, ** and XEROX CORPORATION DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, ** INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ ** States export control laws.  This software is made available AS IS,
+ ** and XEROX CORPORATION DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
+ ** INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF MERCHANTABILITY
  ** AND FITNESS FOR A PARTICULAR PURPOSE, AND NOTWITHSTANDING ANY OTHER
  ** PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM
  ** THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
  ** CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
  ** XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
  */
-/* $Id: client.c,v 1.71 1996/03/09 04:44:35 janssen Exp $ */
-/* Last edited by Mike Spreitzer December 15, 1995 9:48 am PST */
-
+/* $Id: client.c,v 1.77 1996/06/21 01:12:56 janssen Exp $ */
 
 #include "cstubber.h" 
 
@@ -51,20 +51,26 @@ static void declareCallerReturnValue(Type type, Context context)
   }
 }
 
-static void listArgument (Argument arg, Context context)
+static void listArgument (refany elt, refany rock)
 {
-  fprintf (context->file, ", %s %s",     c_parameter_type(arg->type, arg->direction),     c_argument_name(arg));
+  Argument        arg = (Argument) elt;
+  Context         context = (Context) rock;
+  fprintf(context->file, ", %s %s",
+	  c_parameter_type(arg->type, arg->direction),
+	  c_argument_name(arg));
 }
 
 void generateProcHeader (Procedure m, Context c, boolean external)
 {
-  fprintf (c->file, "%s%s %s%s_%s (%s _handle",
-	   external ? "" : "static ",
-	   (type_basic_type(m->returnType) == void_Type) ? "void" : c_return_type (m->returnType),
-	   external ? "" : "_", c_type_name (c->class),
-	   c_simple_name (m->name), c_type_name (c->class));
-  list_enumerate (m->arguments, (void  (*) ()) listArgument, c);
-  fprintf (c->file, ", ILU_C_ENVIRONMENT *_status)\n{\n");
+  fprintf(c->file, "%s%s %s%s_%s (%s _handle",
+	  external ? "" : "static ",
+	  ((type_basic_type(m->returnType) == void_Type)
+	   ? "void" : c_return_type(m->returnType)),
+	  external ? "" : "_",
+	  c_type_name(c->class),
+	  c_simple_name(m->name), c_type_name(c->class));
+  list_enumerate(m->arguments, listArgument, c);
+  fprintf(c->file, ", ILU_C_ENVIRONMENT *_status)\n{\n");
 }
 
 int methodNdxOf (Procedure p, Type t)
@@ -89,33 +95,20 @@ int methodNdxOf (Procedure p, Type t)
 
 static void sizeArgument (Argument arg, Context context)
 {
-  char b[1000];
-  Type ut = ur_type(arg->type);
+  char            b[1000];
+  Type            ut = ur_type(arg->type);
+  TypeKind        t = type_basic_type(ut);
 
-  if  (arg->direction == Out)
+  if (arg->direction == Out)
     return;
-  else if (arg->direction == InOut) /* all args are passed by ref in this case */
-    {
-      enum PrimitiveTypes t;
-
-      t = type_basic_type(ut);
-      if (TypeIsString(ut) OR
-	  TypeIsWString(ut) OR
-	  (t != sequence_Type AND
-	   t != union_Type AND
-	   t != array_Type AND
-	   t != record_Type))
-	{
-	  *b = '*';
-	  strcpy (b+1, c_argument_name(arg));
-	}
-      else
-	strcpy (b, c_argument_name(arg));
-    }
-  else
-    strcpy (b, c_argument_name(arg));
-  fprintf (context->file, "\n     +");
-  SizeValue (context, ut, b);
+  sprintf(b, "%s%s",
+	  (t != array_Type &&
+	   (arg->direction == InOut || t == union_Type ||
+	    t == record_Type || NonStringSequence(ut)))
+	  ? "*" : "",
+	  c_argument_name(arg));
+  fprintf(context->file, "\n     +");
+  SizeValue(context, ut, b);
 }
 
 static void outputArgument (Argument arg, Context context)
@@ -148,18 +141,6 @@ static void outputArgument (Argument arg, Context context)
   MarshallValue (context, ut, b, 2);
 }
 
-static void inputArgument (Argument arg, Context context)
-{
-  char b[1000];
-
-  if  (arg->direction == In)
-    return;
-
-  strcpy (b, c_argument_name(arg));
-
-  UnmarshallValue (context, ur_type(arg->type), arg->def, b, 6, True, FALSE);
-}
-
 static void TypeIsObj (Type t, boolean *objarg)
 {
   if (type_basic_type(t) == object_Type)
@@ -173,44 +154,49 @@ static void scanForObjectArg (Argument a, boolean *objarg)
 
 static void listArg (Context c, char *op, Type t, char *name)
 {
-  fprintf (c->file, ",\n                       %s", name);
-  if (! (ur_type(t)->builtIn OR
-	 (type_ur_kind(t) == sequence_Type AND
-	  ((type_ur_kind(type_description(t)->structuredDes.sequence.type) == shortcharacter_Type) OR
-	   (type_ur_kind(type_description(t)->structuredDes.sequence.type) == shortcharacter_Type))) OR
-	 (type_ur_kind(t) == object_Type) OR
-	 (type_ur_kind(t) == enumeration_Type)))
-    fprintf (c->file, ", _%s__%s", c_type_name(ur_type(t)), op);
-  if (type_ur_kind(t) == object_Type)
-    fprintf (c->file, ", _%s__ILUType", c_type_name(ur_type(t)));
-  if (type_ur_kind(t) == sequence_Type AND
-      (type_ur_kind(type_description(t)->structuredDes.sequence.type) == shortcharacter_Type OR
-       type_ur_kind(type_description(t)->structuredDes.sequence.type) == shortcharacter_Type))
-    fprintf (c->file, ", 0x%lx", (unsigned long) type_description(t)->structuredDes.sequence.limit);
+  TypeKind        tk = type_ur_kind(t);
+  fprintf(c->file, ",\n                       %s", name);
+  if (!(ur_type(t)->builtIn OR
+	TypeIsEitherString(t) OR
+	(type_ur_kind(t) == object_Type) OR
+	(type_ur_kind(t) == enumeration_Type)))
+    fprintf(c->file, ", _%s__%s", c_type_name(ur_type(t)), op);
+  if (tk == object_Type)
+    fprintf(c->file, ", _%s__ILUType", c_type_name(ur_type(t)));
+  if (TypeIsEitherString(t))
+    fprintf(c->file, ", (ilu_cardinal) 0x%lx",
+	    (unsigned long) (type_description(t)->
+			     structuredDes.sequence.limit));
 }
 
-static void listSizeOfArgs (Argument a, Context context)
+static void listSizeOfArgs (refany elt, refany rock)
 {
+  Argument        a = (Argument) elt;
+  Context         context = (Context) rock;
   if (a->direction == Out)
     return;
   listArg (context, "SizeOf", a->type, c_argument_name(a));
 }
 
-static void listOutputArgs (Argument a, Context context)
+static void listOutputArgs (refany elt, refany rock)
 {
+  Argument        a = (Argument) elt;
+  Context         context = (Context) rock;
   if (a->direction == Out)
     return;
   listArg (context, "Output", a->type, c_argument_name(a));
 }
 
-static void listInputArgs (Argument a, Context context)
+static void listInputArgs (refany elt, refany rock)
 {
+  Argument        a = (Argument) elt;
+  Context         context = (Context) rock;
   if (a->direction == In)
     return;
   listArg (context, "Input", a->type, c_argument_name(a));
 }
 
-void codeType (Type t, char **p, ArgDirection dir)
+void codeType (Type t, char **p, Role role)
 {
   char a;
 
@@ -232,39 +218,53 @@ void codeType (Type t, char **p, ArgDirection dir)
   case object_Type:     a = 'p'; break;
   case sequence_Type:
     {
-      if (type_ur_kind(type_description(t)->structuredDes.sequence.type) == shortcharacter_Type)
+      TypeKind        ek = type_ur_kind(type_description(t)->
+				      structuredDes.sequence.type);
+      if (ek == shortcharacter_Type)
 	a = 'q';
-      else if (type_ur_kind(type_description(t)->structuredDes.sequence.type) == character_Type)
+      else if (ek == character_Type)
 	a = 'r';
+      else if (role == role_Out || role == role_Return)
+	a = 'y';
       else
 	a = 'z';
     }
     break;
   case array_Type:
+    a = (role == role_Out && VariableLength(t) ||
+	 role == role_Return) ? 'y' : 'z';
+    break;
   case record_Type:
   case union_Type:
-  case optional_Type:     a = 'z'; break;
+    a = ((role == role_Out || role == role_Return) &&
+	 VariableLength(t)) ? 'y' : 'z';
+    break;
+  case optional_Type:	a = 'z'; break;
   case void_Type:     a = '*'; break;
   default:     fatal ("Error: Bad parameter type %s.\n", c_type_name (t));
   }
 
-  if (dir == InOut)
+  if (role == role_InOut)
     a = toupper(a);
 
   **p = a;
   (*p)++;
 }
 
-static void codeInputArg (Argument a, char **p)
+static void codeInputArg (refany elt, refany rock)
 {
+  Argument        a = (Argument) elt;
+  char          **p = (char **) rock;
   if (a->direction != Out)
-    codeType (a->type, p, a->direction);
+    codeType (a->type, p, adRole[a->direction]);
 }
 
-static void codeOutputArg (Argument a, char **p)
+static void codeOutputArg (refany elt, refany rock)
 {
+  Argument        a = (Argument) elt;
+  char          **p = (char **) rock;
   if (a->direction != In)
-    codeType (a->type, p, a->direction);
+    codeType (a->type, p, adRole[a->direction]);
 }
 
 static char *genParmDescription (Procedure m)
@@ -274,175 +274,190 @@ static char *genParmDescription (Procedure m)
 
   p = (char *) malloc(len);
   q = p;
-  list_enumerate (m->arguments, (EnumProc) codeInputArg, &q);
+  list_enumerate (m->arguments, codeInputArg, &q);
   *q++ = ':';
   if (m->returnType != NULL && type_ur_kind(m->returnType) != void_Type)
-    codeType (m->returnType, &q, Out);
-  list_enumerate (m->arguments, (EnumProc) codeOutputArg, &q);
+    codeType (m->returnType, &q, role_Return);
+  list_enumerate (m->arguments, codeOutputArg, &q);
   *q = 0;
   return p;
 }
 
 static void addExceptionInputFn (refany elt, refany rock)
 {
-  Exception e = (Exception) elt;
-  Context context = (Context) rock;
+  Exception       e = (Exception) elt;
+  Context         context = (Context) rock;
   Exception       ure = e->import ? e->import : e;
   Type            ut = ur_type(ure->type);
   TypeKind        tk = type_kind(ut);
+  FILE           *f = context->file;
 
-  fprintf (context->file, "%s\n    { ", context->counter == 0 ? "" : ",");
+  fprintf(context->file, "%s\n    { ", context->counter == 0 ? "" : ",");
   context->counter += 1;
 
   if (tk == void_Type)
-    fprintf (context->file, "0, ");
+    fprintf(f, "0, ");
   else
-    fprintf (context->file, "sizeof(%s), ", c_type_name(ut));
+    fprintf(f, "sizeof(%s), ", c_type_name(ut));
 
   if (tk == object_Type)
-    fprintf (context->file, "_%s__ILUType, ", c_type_name(ut));
+    fprintf(f, "_%s__ILUType, ", c_type_name(ut));
   else
-    fprintf (context->file, "ILU_NIL, ");
+    fprintf(f, "ILU_NIL, ");
 
-  switch (tk)
-    {
-    case integer_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputInteger, (void (*) (void)) 0");
-      break;
+  switch (tk) {
+  case integer_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputInteger, 0");
+    break;
+  case cardinal_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputCardinal, 0");
+    break;
 
-    case cardinal_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputCardinal, (void (*) (void)) 0");
-      break;
+  case shortinteger_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputShortInteger, 0");
+    break;
+  case shortcardinal_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputShortCardinal, 0");
+    break;
 
-    case shortinteger_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputShortInteger, (void (*) (void)) 0");
-    case shortcardinal_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputShortCardinal, (void (*) (void)) 0");
-      break;
+  case longinteger_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputLongInteger, 0");
+    break;
+  case longcardinal_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputLongCardinal, 0");
+    break;
 
-    case longinteger_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputLongInteger, (void (*) (void)) 0");
-      break;
+  case character_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputCharacter, 0");
+    break;
+  case shortcharacter_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputShortCharacter, 0");
+    break;
 
-    case longcardinal_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputLongCardinal, (void (*) (void)) 0");
-      break;
+  case real_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputReal, 0");
+    break;
+  case shortreal_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputShortReal, 0");
+    break;
+  case longreal_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputLongReal, 0");
+    break;
 
-    case character_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputCharacter, (void (*) (void)) 0");
-      break;
+  case byte_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputByte, 0");
+    break;
+  case boolean_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ILU_C_InputBoolean, 0");
+    break;
 
-    case shortcharacter_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputShortCharacter, (void (*) (void)) 0");
-      break;
+  case enumeration_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) ilu_InputEnum, 0");
+    break;
 
-    case real_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputReal, (void (*) (void)) 0");
-      break;
+  case object_Type:
+    fprintf(f, "0, 0, 0, 0");
+    break;
 
-    case shortreal_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputShortReal, (void (*) (void)) 0");
-      break;
+  case array_Type:
+  case record_Type:
+  case union_Type:
+  case sequence_Type:
+  case optional_Type:
+    fprintf(f, "0, 0, (ILU_C_InputFn) _%s__Input, ", c_type_name(ut));
+    if (HasFreeRoutine(ut))
+      fprintf(f, "(ILU_C_FreeFn) %s__Free", c_type_name(ut));
+    else
+      fprintf(f, "0");
+    break;
 
-    case longreal_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputLongReal, (void (*) (void)) 0");
-      break;
+  case void_Type:
+    fprintf(f, "0, 0, 0, 0");
+    break;
 
-    case byte_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputByte, (void (*) (void)) 0");
-      break;
-
-    case boolean_Type:       fprintf (context->file, "(void (*) (void)) ILU_C_InputBoolean, (void (*) (void)) 0");
-      break;
-
-    case enumeration_Type:       fprintf (context->file, "(void (*) (void)) ilu_InputEnum, (void (*) (void)) 0");
-      break;
-
-    case object_Type:       fprintf (context->file, "(void (*) (void)) (void (*) (void)) 0, (void (*) (void)) 0");
-      break;
-
-    case array_Type:     case record_Type:     case union_Type:     case sequence_Type:     case optional_Type:       fprintf (context->file, "(void (*) (void)) _%s__Input, ", c_type_name(ut));
-      if (HasFreeRoutine(ut))
-	fprintf (context->file, "(void (*) (void)) %s__Free", c_type_name(ut));
-      else
-	fprintf (context->file, "(void (*) (void)) 0");
-      break;
-
-    case void_Type:       fprintf (context->file, "0, 0");
-      break;
-
-    default:     fatal ("Error: Bad parameter type %s.\n", c_type_name (ure->type));
-    }
-  fprintf (context->file, " }");
+  default:
+    fatal("Error: Bad parameter type %s.\n", c_type_name(ure->type));
+  }
+  fprintf(f, " }");
 }
 
 static void declareExceptionValInputFns (list exceptions, Context context)
 {
   if (exceptions == NULL || list_size(exceptions) < 1)
     return;
-
-  fprintf (context->file, "  static struct _ILU_C_ExceptionDescription_s _evec[] = {");
+  fprintf(context->file,
+      "  static struct _ILU_C_ExceptionDescription_s _evec[] = {");
   context->counter = 0;
-  list_enumerate (exceptions, addExceptionInputFn, context);
-  fprintf (context->file, "};\n\n");
+  list_enumerate(exceptions, addExceptionInputFn, context);
+  fprintf(context->file, "};\n\n");
 }
 
 static void generateMethodCode (Procedure m, Context context)
 {
-  char *p;
-  struct ilu_argument_s retasarg;
-  enum PrimitiveTypes t = type_basic_type(ur_type(m->returnType));
+  char           *p;
+  TypeKind        t = type_basic_type(ur_type(m->returnType));
 
-  if  (methodInList (c_simple_name (m->name)))
-    {
-      MethodRecordID++;
-      return;
-    }
-  addMethodToList (c_simple_name(m->name));
+  if (methodInList(c_simple_name(m->name))) {
+    MethodRecordID++;
+    return;
+  }
+  addMethodToList(c_simple_name(m->name));
 
   generateProcHeader(m, context, FALSE);
 
-  /* declare var to hold return value, if any */
-  declareCallerReturnValue (m->returnType, context);
-  /* declare var to hold possible exception value unmarshalling routines, if any */
-  declareExceptionValInputFns (m->exceptions, context);
+  /* Declare var to hold return value, if any. */
+  declareCallerReturnValue(m->returnType, context);
+  /*
+   * Declare var to hold possible exception value unmarshalling
+   * routines, if any.
+   */
+  declareExceptionValInputFns(m->exceptions, context);
 
   /*
-   ** if method is functional, see if it is cached,    ** and if so, return the cached value
+   * If method is functional, see if it is cached,     and if so,
+   * return the cached value.
    */
-
-  if  (IsCacheableMethod (m)) {
-
-    /*
-     ** eventually...
-     */
+  if (IsCacheableMethod(m)) {
+    /* eventually... */
   }
-
-  /* if return type is an array, we need to allocate a new copy of it,
-     as GenericCall will be expecting the space to be pre-allocated.
-     Arrays are the only type for which this is necessary. */
-
-  if (t == array_Type)
-    {
-      fprintf (context->file, "  if ((_retvalue = (%s) ilu_malloc(sizeof(%s))) == ILU_NIL)\n",
-	       c_return_type(m->returnType), c_type_name(m->returnType));
-      fprintf (context->file, "    { ILU_C_RAISE_SYSTEM(_status,NO_MEMORY,sizeof(%s),NO); return _retvalue; };\n",
-	       c_type_name(m->returnType));
-    }
+  /*
+   * If return type is an array, we need to allocate a new copy of
+   * it, as GenericCall will be expecting the space to be
+   * pre-allocated.  Arrays are the only type for which this is
+   * necessary.
+   */
 
   /* generate call to _ILU_C_GenericCall */
 
-  fprintf (context->file, "  _ILU_C_GenericCall (_%s__ILUType, &_%s__ILUType->cl_methods[%d],\n",     c_type_name(m->object), c_type_name(m->object), methodNdxOf(m, m->object));
+  fprintf(context->file,
+	  "  _ILU_C_GenericCall (_%s__ILUType, &_%s__ILUType->cl_methods[%d],\n",
+	  c_type_name(m->object), c_type_name(m->object),
+	  methodNdxOf(m, m->object));
   p = genParmDescription(m);
-  fprintf (context->file, "                       %s, _handle, _status, \"%s\"",
-	   list_size(m->exceptions) < 1 ? "ILU_NIL" : "_evec", p);
+  fprintf(context->file,
+	  "                       %s, _handle, _status, \"%s\"",
+	  list_size(m->exceptions) < 1 ? "ILU_NIL" : "_evec", p);
   free(p);
-  list_enumerate (m->arguments, (void (*)(void *, void *)) listSizeOfArgs, context);
-  list_enumerate (m->arguments, (void (*)(void *, void *)) listOutputArgs, context);
+  list_enumerate(m->arguments, listSizeOfArgs, context);
+  list_enumerate(m->arguments, listOutputArgs, context);
+  if (t != void_Type) {
+    listArg(context, "Input", m->returnType, "&_retvalue");
+  }
+  list_enumerate(m->arguments, listInputArgs, context);
+  fprintf(context->file, "\n                      );\n");
+
+  /*
+   * If we returned an error, release the storage allocated for an
+   * array.
+   */
+
+  if (t == array_Type) {
+    fprintf(context->file, "  if (!ILU_C_SUCCESSFUL(_status))\n");
+    fprintf(context->file, "    ilu_free(_retvalue);\n");
+  }
   if (t != void_Type)
-    {
-      listArg (context, "Input", m->returnType,
-	       (type_ur_kind(m->returnType) != array_Type) ? "&_retvalue" : "_retvalue");
-    }
-  list_enumerate (m->arguments, (void (*)(void *, void *)) listInputArgs, context);
-  fprintf (context->file, "\n                      );\n");
-
-  /* if we returned an error, release the storage allocated for an array */
-
-  if (t == array_Type)
-    fprintf (context->file, "  if (!ILU_C_SUCCESSFUL(_status))\n    ilu_free(_retvalue);\n");
-
-  if (t != void_Type)
-    fprintf (context->file, "  return _retvalue;\n");
-  fprintf (context->file, "}\n\n");
+    fprintf(context->file, "  return _retvalue;\n");
+  fprintf(context->file, "}\n\n");
 }
 
 static void mkPrototype (Procedure m, Context c)
@@ -450,10 +465,9 @@ static void mkPrototype (Procedure m, Context c)
   enum PrimitiveTypes t = type_basic_type (m->returnType);
 
   fprintf (c->file,
-	   "%s _%s_%s (%s",
-	   (!m->returnType) ? "void" : ((t == array_Type) ? c_return_type (m->returnType) : c_type_name (m->returnType)),
-	   c_type_name (m->object), c_simple_name (m->name), c_type_name (m->object));
-  list_enumerate (m->arguments, (iluparser_EnumProc) listArgumentTypes, c);
+	   "%s _%s_%s (%s", c_return_type(m->returnType),
+	   c_type_name(m->object), c_simple_name(m->name), c_type_name(m->object));
+  list_enumerate (m->arguments, listArgumentTypes, c);
   fprintf (c->file, ", ILU_C_ENVIRONMENT *);\n");
 }
 
@@ -549,7 +563,7 @@ static void generateMethods (    Type class,     Context context)
   if  (!class)
     return;
 
-  list_enumerate(class_object(class)->methods,  (void  (*) ()) generateMethodCode, context);
+  list_enumerate(class_object(class)->methods,  (iluparser_EnumProc) generateMethodCode, context);
 }
 
 static void generateClassCode(refany elt, refany rock)
