@@ -1,5 +1,5 @@
 /*
- ** Copyright (c) 1991-1994 Xerox Corporation.  All Rights Reserved.
+ ** Copyright (c) 1991-1996 Xerox Corporation.  All Rights Reserved.
  **
  ** Unlimited use, reproduction, and distribution of this software is
  ** permitted.  Any copy of this software must include both the above
@@ -14,159 +14,176 @@
  ** CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
  ** XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
  */
-/* $Id: server.c,v 1.66 1996/03/12 04:03:33 janssen Exp $ */
-/* Last edited by Mike Spreitzer February 13, 1996 12:02 pm PST */
+/* $Id: server.c,v 1.74 1996/06/19 03:08:18 janssen Exp $ */
+/* Last edited by Mike Spreitzer April 3, 1996 9:01 am PST */
 
 #include "cstubber.h"
 
 static boolean  FirstClass = TRUE;
 
-static void declareReturnValue (Type type, Context context)
+static void listArgument(refany elt, refany rock)
 {
-  if (type != NULL AND
-      type_basic_type(type) != void_Type)
-    {
-      fprintf (context->file, "  %s _retvalue;\n", c_return_type(type));
-    }
+  Argument        a = (Argument) elt;
+  Context         context = (Context) rock;
+  TypeKind        t = type_basic_type(ur_type(a->type));
+  int             indv, indp;
+  indv = Indirectness(a->type, adRole[a->direction], TRUE, 0);
+  indp = Indirectness(a->type, adRole[a->direction], FALSE, 0);
+  fprintf(context->file, ", %s%s",
+	  ((indp > indv) ? "&" : ""),
+	  c_argument_name(a));
 }
 
-static void listArgument (refany elt, refany rock)
+void 
+FreeValue(Type type, string val, Role role, Context context,
+	  int indent)
 {
-  Argument a = (Argument) elt;
-  Context context = (Context) rock;
-  enum PrimitiveTypes t = type_basic_type (ur_type(a->type));
-
-  fprintf (context->file, ", %s%s",
-	   ((a->direction == In AND
-	     (t == union_Type OR t == record_Type OR
-	      (t == sequence_Type AND NOT TypeIsString(ur_type(a->type)) AND NOT TypeIsWString(ur_type(a->type))))) OR
-	    ((a->direction == Out OR a->direction == InOut) AND
-	     t != array_Type)) ? "&" : "",
-	   c_argument_name(a));
-}
-
-void FreeValue (Type type, char *name, Context context, int indent)
-{
-  TypeKind        t = type_ur_kind(type);
   Type            ut = ur_type(type);
+  TypeKind        t = type_ur_kind(type);
+  int             indv = Indirectness(ut, role, TRUE, 1);
+  int             indp = Indirectness(ut, role_Exn, FALSE, 1);
 
-  if (HasFreeRoutine(ut))
-    fprintf(context->file, "%*.*s%s__Free (%s%s);\n", indent, indent, "",
-	    c_type_name(ut),
-	    ((t == array_Type) ? "" : "&"),
-	    name);
+  if (HasFreeRoutine(ut)) {
+    if (Sliced(ut, role)) {
+      if (indv != indp) {
+	fprintf(stderr, "Bug at %s:%d: %d != %d, for %s, %d.\n",
+		__FILE__, __LINE__, indv, indp,
+		c_type_name(ut), role);
+	fprintf(context->file,
+		"%*.*s_ilu_Assert(0, \"FreeValue(%s, %d)\");\n",
+		indent, indent, "",
+		c_type_name(ut), role);
+      }
+      fprintf(context->file, "%*.*s%s__Free ((%s)%s);\n",
+	      indent, indent, "", c_type_name(ut),
+	      c_role_type(ut, role_Exn, FALSE), val);
+    } else
+      fprintf(context->file, "%*.*s%s__Free (%s%s);\n",
+	      indent, indent, "",
+	      c_type_name(ut),
+	      ((indp > indv) ? "&" : ""),
+	      val);
+  }
+  if (indv)
+    fprintf(context->file, "  CORBA_free(%s);\n", val);
 }
 
-static void freeArgument (refany elt, refany rock)
+static void freeArgument(refany elt, refany rock)
 {
-  Argument arg = (Argument) elt;
-  Context context = (Context) rock;
-  FreeValue (arg->type, c_argument_name(arg), context, 2);
-}     
+  Argument        arg = (Argument) elt;
+  Context         context = (Context) rock;
+  FreeValue(arg->type, c_argument_name(arg), adRole[arg->direction],
+	    context, 2);
+}
 
 static void freeReturnValue (Type type, Context context)
 {
-  FreeValue (type, "_retvalue", context, 2);
+  FreeValue (type, "_retvalue", role_Return, context, 2);
 }
 
-static void sizeArgument (refany elt, refany rock)
+static void sizeArgument(char *name, Type ut, Role role, Context context)
 {
-  Argument a = (Argument) elt;
-  Context context = (Context) rock;
-  char b[ 1000 ];
-  Type ut = ur_type(a->type);
-  enum PrimitiveTypes t = type_basic_type (ur_type(a->type));
-
-  if  (a->direction == In)
-    return;
-  if (t == union_Type OR
-      t == record_Type OR
-      (t == sequence_Type AND NOT TypeIsString(ut) AND NOT TypeIsWString(ut)))
-    {
-      b[0] = '&';
-      strcpy (b+1, c_argument_name(a));
-    }
-  else
-    strcpy(b, c_argument_name(a));
-  fprintf (context->file, "      _size += ");
-  SizeValue (context, ut, b);
-  fprintf (context->file, ";\n");
-  fprintf (context->file, "      if (ILU_ERRNOK(*_err)) goto errexit;\n");
+  char            buf[1000];
+  TypeKind        t = type_ur_kind(ut);
+  int             ind;
+  ind = Indirectness(ut, role, TRUE, 0);
+  sprintf(buf, "%s%s", (ind ? "*" : ""), name);
+  fprintf(context->file, "        _size += ");
+  SizeValue(context, ut, buf);
+  fprintf(context->file, ";\n");
+  fprintf(context->file, "      if (ILU_ERRNOK(*_err)) goto errexit;\n");
 }
 
-static void outputArgument (refany elt, refany rock)
+static void sizeArg(refany elt, refany rock)
 {
-  Argument a = (Argument) elt;
-  Context context = (Context) rock;
-  char b[ 1000 ];
-  Type ut = ur_type(a->type);
-  enum PrimitiveTypes t = type_basic_type (ur_type(a->type));
-
-  if  (a->direction == In)
+  Argument        arg = (Argument) elt;
+  Context         context = (Context) rock;
+  if (arg->direction == In)
     return;
-  if (t == union_Type OR
-      t == record_Type OR
-      (t == sequence_Type AND NOT TypeIsString(ut) AND NOT TypeIsWString(ut)))
-    {
-      b[0] = '&';
-      strcpy (b+1, c_argument_name(a));
-    }
-  else
-    strcpy(b, c_argument_name(a));
-  MarshallValue (context, ut, b, 6);
-  fprintf (context->file, "      if (ILU_ERRNOK(*_err)) goto errexit;\n");
+  sizeArgument(c_argument_name(arg), arg->type, adRole[arg->direction],
+	       context);
+}
+
+static void outputArgument (char *name, Type ut, Role role, Context context)
+{
+  char            buf[1000];
+  int             indp, indv;
+  indp = Indirectness(ut, role_In, FALSE, 0);
+  indv = Indirectness(ut, role, TRUE, 0);
+  sprintf(buf, "%s%s",
+	  ((indp < indv) ? "*" : (indp > indv) ? "&" : ""),
+	  name);
+  MarshallValue(context, ut, buf, 6);
+}
+
+static void outputArg(refany elt, refany rock)
+{
+  Argument        arg = (Argument) elt;
+  if (arg->direction != In)
+    outputArgument(c_argument_name(arg), arg->type,
+		   adRole[arg->direction], (Context) rock);
 }
 
 static void inputArgument (refany elt, refany rock)
 {
-  Argument arg = (Argument) elt;
-  Context context = (Context) rock;
-  char buf[1000];
-
+  Argument        arg = (Argument) elt;
+  Context         context = (Context) rock;
+  char            buf[1000];
+  int             indp, indv;
   if (arg->direction == Out)
     return;
-  sprintf (buf, "%s%s", TypeIsPointer(ur_type(arg->type)) ? "" : "&",
-	   c_argument_name(arg));
-  UnmarshallValue (context, ur_type(arg->type), arg->def, buf, 2, TRUE, FALSE);
-  fprintf (context->file, "  if (ILU_ERRNOK(*_err)) goto errexit;\n");
+  indv = Indirectness(arg->type, adRole[arg->direction], TRUE, 0);
+  indp = Indirectness(arg->type, role_InOut, FALSE, 0);
+  sprintf(buf, "%s%s", (indp > indv) ? "&" : "", c_argument_name(arg));
+  UnmarshallValue(context, ur_type(arg->type), arg->def, buf, 2);
 }
 
-static void declareArgument (refany elt, refany rock)
+static void declareArg (refany elt, refany rock)
 {
-  Argument arg = (Argument) elt;
-  TypeKind t = type_ur_kind(arg->type);
-  Context context = (Context) rock;
-
-  fprintf(context->file, "  %s %s;\n", c_type_name(arg->type), c_argument_name(arg));
+  Argument        arg = (Argument) elt;
+  Context         context = (Context) rock;
+  Type            type = arg->type;
+  string          name = c_argument_name(arg);
+  fprintf(context->file, "  %s %s = {0};\n",
+	  c_role_type(type, adRole[arg->direction], TRUE), name);
 }
 
-static void initType (char *name, Type type, Context context, ArgDirection dir)
+static void initType(char *name, Type type, Context context, Role role)
 {
-  TypeKind t = type_ur_kind(type);
-
-  if (t == object_Type || TypeIsString(type) ||
-      t == shortinteger_Type || t == integer_Type ||
-      t == shortcardinal_Type || t == cardinal_Type ||
-      t == shortreal_Type || t == real_Type || t == boolean_Type ||
-      t == optional_Type || t == enumeration_Type || t == byte_Type)
-    fprintf (context->file, "  %s = (%s) 0;\n", name, c_type_name(type));
-  else if ((dir == In) && (t == union_Type || t == record_Type || (t == sequence_Type && !TypeIsString(type))))
-    fprintf (context->file, "  memset (&%s, 0, sizeof(%s));\n", name, c_type_name(type));
-  else if ((dir == In) && t == array_Type)
-    fprintf (context->file, "  memset (%s, 0, sizeof(%s));\n", name, c_type_name(type));
+  TypeKind        t = type_ur_kind(type);
+  int             ind = Indirectness(type, role, TRUE, 1);
 
+  if (ind > 0) {
+    fprintf(context->file, "  %s = (%s) 0;\n", name,
+	    c_role_type(type, role, TRUE));
+  } else if (t == array_Type) {
+    fprintf(context->file, "  memset ((void *) %s, 0, sizeof(%s));\n",
+	    name, c_type_name(type));
+  } else if (t == record_Type || t == union_Type ||
+	     NonStringSequence(type)) {
+    fprintf(context->file, "  memset ((void *) &%s, 0, sizeof(%s));\n",
+	    name, c_type_name(type));
+  } else {
+    fprintf(context->file, "  %s = (%s) 0;\n",
+	    name, c_type_name(type));
+  }
 }
 
-static void initArgument (refany elt, refany rock)
+static void initArgument(refany elt, refany rock)
 {
-  Argument arg = (Argument) elt;
-
-  initType (c_argument_name(arg), arg->type, (Context) rock, arg->direction == Out);
+  Argument        arg = (Argument) elt;
+  if (arg == NULL || arg->type == NULL ||
+      (type_kind(arg->type) == void_Type))
+    return;
+  initType(c_argument_name(arg), arg->type, (Context) rock,
+	   adRole[arg->direction]);
 }
 
 static void initReturnValue (Type t, Context context)
 {
-  initType ("_retvalue", t, context, TRUE);
+  if (t == NIL || type_kind(t) == void_Type)
+    return;
+  initType ("_retvalue", t, context, role_Return);
 }
 
 static void addExceptionOutputFn (refany elt, refany rock)
@@ -178,77 +195,78 @@ static void addExceptionOutputFn (refany elt, refany rock)
   TypeKind        tk = type_kind(ut);
   char            a;
   char           *p = &a;
+  FILE           *f = context->file;
 
-  codeType (ut, &p, In);
-  fprintf (context->file, "%s\n    { %d, ", context->counter == 0 ? "" : ",", (unsigned int) a);
+  codeType (ut, &p, role_In);
+  fprintf(context->file, "%s\n    { (unsigned int) '%c', ",
+	  context->counter == 0 ? "" : ",", a);
   context->counter += 1;
 
   if (tk == object_Type)
-    fprintf (context->file, "_%s__ILUType, ", c_type_name(ut));
+    fprintf (f, "_%s__ILUType, (ILU_C_SizeFn) ", c_type_name(ut));
   else
-    fprintf (context->file, "(ilu_Class) 0, ");
+    fprintf (f, "(ilu_Class) 0, (ILU_C_SizeFn) ");
 
   switch (tk)
     {
     case integer_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfInteger, (void (*) (void)) ilu_OutputInteger");
+      fprintf (f, "ilu_SizeOfInteger, (ILU_C_OutputFn) ilu_OutputInteger");
       break;
       
     case cardinal_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfCardinal, (void (*) (void)) ilu_OutputCardinal");
+      fprintf (f, "ilu_SizeOfCardinal, (ILU_C_OutputFn) ilu_OutputCardinal");
       break;
       
     case shortinteger_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfShortInteger, (void (*) (void)) ilu_OutputShortInteger");
+      fprintf (f, "ilu_SizeOfShortInteger, (ILU_C_OutputFn) ilu_OutputShortInteger");
       break;
       
     case shortcardinal_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfShortCardinal, (void (*) (void)) ilu_OutputShortCardinal");
+      fprintf (f, "ilu_SizeOfShortCardinal, (ILU_C_OutputFn) ilu_OutputShortCardinal");
       break;
       
     case longinteger_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfLongInteger, (void (*) (void)) ilu_OutputLongInteger");
+      fprintf (f, "ilu_SizeOfLongInteger, (ILU_C_OutputFn) ilu_OutputLongInteger");
       break;
       
     case longcardinal_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfLongCardinal, (void (*) (void)) ilu_OutputLongCardinal");
+      fprintf (f, "ilu_SizeOfLongCardinal, (ILU_C_OutputFn) ilu_OutputLongCardinal");
       break;
 
     case character_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfCharacter, (void (*) (void)) ilu_OutputCharacter");
+      fprintf (f, "ilu_SizeOfCharacter, (ILU_C_OutputFn) ilu_OutputCharacter");
       break;
 
     case shortcharacter_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfShortCharacter, (void (*) (void)) ilu_OutputShortCharacter");
+      fprintf (f, "ilu_SizeOfShortCharacter, (ILU_C_OutputFn) ilu_OutputShortCharacter");
       break;
 
     case real_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfReal, (void (*) (void)) ilu_OutputReal");
+      fprintf (f, "ilu_SizeOfReal, (ILU_C_OutputFn) ilu_OutputReal");
       break;
 
     case shortreal_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfShortReal, (void (*) (void)) ilu_OutputShortReal");
+      fprintf (f, "ilu_SizeOfShortReal, (ILU_C_OutputFn) ilu_OutputShortReal");
       break;
 
     case longreal_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfLongReal, (void (*) (void)) ilu_OutputLongReal");
+      fprintf (f, "ilu_SizeOfLongReal, (ILU_C_OutputFn) ilu_OutputLongReal");
       break;
 
     case byte_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfByte, (void (*) (void)) ilu_OutputByte");
+      fprintf (f, "ilu_SizeOfByte, (ILU_C_OutputFn) ilu_OutputByte");
       break;
 
     case boolean_Type:
-      fprintf (context->file, "(void (*) (void)) ILU_C_SizeOfBoolean, (void (*) (void)) ILU_C_OutputBoolean");
+      fprintf (f, "ILU_C_SizeOfBoolean, (ILU_C_OutputFn) ILU_C_OutputBoolean");
       break;
 
     case enumeration_Type:
-      fprintf (context->file, "(void (*) (void)) ilu_SizeOfEnum, (void (*) (void)) ilu_OutputEnum");
+      fprintf (f, "ilu_SizeOfEnum, (ILU_C_OutputFn) ilu_OutputEnum");
       break;
 
     case object_Type:
-      fprintf(context->file, "(void (*) (void)) 0, (void (*) (void)) 0",
-	      c_type_name(ut));
+      fprintf(f, "0, 0");
       break;
 
     case array_Type:
@@ -256,19 +274,19 @@ static void addExceptionOutputFn (refany elt, refany rock)
     case union_Type:
     case sequence_Type:
     case optional_Type:
-      fprintf(context->file,
-	      "(void (*) (void)) _%s__SizeOf, (void (*) (void)) _%s__Output",
+      fprintf(f,
+	      "(ILU_C_SizeFn) _%s__SizeOf, (ILU_C_OutputFn) _%s__Output",
 	      c_type_name(ut), c_type_name(ut));
       break;
 
     case void_Type:
-      fprintf (context->file, "(void (*) (void)) 0, (void (*) (void)) 0");
+      fprintf (f, "0, 0");
       break;
 
   default:
     fatal ("Error: Bad parameter type %s.\n", c_type_name (ure->type));
   }
-  fprintf (context->file, " }");
+  fprintf (f, ", 0, 0 }");
 }
 
 static void declareExceptionValOutputFns (list exceptions, Context context)
@@ -282,14 +300,12 @@ static void declareExceptionValOutputFns (list exceptions, Context context)
   fprintf (context->file, "};\n\n");
 }
 
-static void serverMethods (Procedure  m, Context  context)
+static void serverMethods(refany elt, refany rock)
 {
-  fprintf(context->file, "    server_%s_%s,\n", c_type_name(context->class), c_simple_name(m->name));
-}
-
-static void serverProto (Procedure  m, Context  context)
-{
-  fprintf(context->file, "extern void server_%s_%s ();\n", c_type_name(context->class), c_simple_name(m->name));
+  Procedure       m = (Procedure) elt;
+  Context         context = (Context) rock;
+  fprintf(context->file, "    (void(*)(void)) server_%s_%s,\n",
+	  c_type_name(context->class), c_simple_name(m->name));
 }
 
 static void generateCalleeStub (Procedure  m, Context  context)
@@ -306,13 +322,17 @@ static void generateCalleeStub (Procedure  m, Context  context)
 	  c_type_name(context->class), c_simple_name(m->name));
   fprintf(context->file, "  %s _h;\n", c_type_name(context->class));
   fprintf(context->file, "  ILU_C_ENVIRONMENT _status;\n");
-  list_enumerate(m->arguments, declareArgument, context);
-  declareReturnValue(m->returnType, context);
+  list_enumerate(m->arguments, declareArg, context);
+  if (type_ur_kind(m->returnType) != void_Type)
+    fprintf(context->file, "  %s _retvalue = {0};\n",
+	    c_role_type(m->returnType, role_Return, TRUE));
   declareExceptionValOutputFns (m->exceptions, context);
   fprintf(context->file, "\n");
 
+#if 0
   list_enumerate(m->arguments, initArgument, context);
   initReturnValue(m->returnType, context);
+#endif
   fprintf(context->file, "  ILU_CLER(*_err);\n");
   fprintf(context->file, "  _h = (%s) ",
 	  c_type_name(context->class));
@@ -340,36 +360,26 @@ static void generateCalleeStub (Procedure  m, Context  context)
     fprintf(context->file, "  }\n");
     fprintf(context->file, "  if (!_ILU_C_NoReply(_call, _err)) goto errexit;\n");
   } else {
-    char            buf[1000];
-    sprintf(buf, "%s_retvalue",
-	    (t == record_Type
-	     || NonStringSequence(ur_type(m->returnType))
-	     || t == union_Type)
-	    ? "&" : "");
     fprintf(context->file, "    /* check for errors */\n");
     fprintf(context->file, "    if (_status.returnCode == ILU_NIL) {\n");
-    fprintf(context->file, "      ilu_cardinal _size = ");
-    fprintf(context->file, "ilu_BeginSizingReply(_call, %s, _err);\n",
+    fprintf(context->file, "      ilu_cardinal _size = 0;\n");
+    fprintf(context->file, "      if (ilu_CallNeedsSizing(_call)) {\n");
+    fprintf(context->file, "        _size = ilu_BeginSizingReply(_call, %s, _err);\n",
 	(list_size(m->exceptions) > 0) ? "ilu_TRUE" : "ilu_FALSE");
-    fprintf(context->file, "      if (ILU_ERRNOK(*_err))\n");
-    fprintf(context->file, "        goto errexit;\n");
+    fprintf(context->file, "        if (ILU_ERRNOK(*_err))\n");
+    fprintf(context->file, "          goto errexit;\n");
     if (t != void_Type) {
-      fprintf(context->file, "      _size += ");
-      SizeValue(context, ur_type(m->returnType), buf);
-      fprintf(context->file, ";\n");
-      fprintf(context->file, "      if (ILU_ERRNOK(*_err))\n");
-      fprintf(context->file, "        goto errexit;\n");
+      sizeArgument ("_retvalue", ur_type(m->returnType), role_Return, context);
     }
-    list_enumerate(m->arguments, sizeArgument, context);
+    list_enumerate(m->arguments, sizeArg, context);
+    fprintf(context->file, "      };\n");
     fprintf(context->file,
 	    "      if (!_ILU_C_BeginReply (_call, %s, _size, _err)) goto errexit;\n",
 	(list_size(m->exceptions) > 0) ? "ilu_TRUE" : "ilu_FALSE");
     if (type_basic_type(ur_type(m->returnType)) != void_Type)
-      {
-	MarshallValue(context, m->returnType, buf, 6);
-	fprintf (context->file, "      if (ILU_ERRNOK(*_err)) goto errexit;\n");
-      }
-    list_enumerate(m->arguments, outputArgument, context);
+      outputArgument("_retvalue", ur_type(m->returnType), role_Return,
+		     context);
+    list_enumerate(m->arguments, outputArg, context);
     fprintf(context->file,
 	    "      if (! _ILU_C_FinishReply (_call, _err)) goto errexit;\n");
     fprintf(context->file, "    }\n");
@@ -386,7 +396,7 @@ static void generateCalleeStub (Procedure  m, Context  context)
     fprintf(context->file, "    }\n");
     fprintf(context->file, "  }\n\n");
   }
-  fprintf (context->file, " errexit:\n");
+  fprintf (context->file, " marshalError:\n errexit:\n");
   fprintf(context->file, "  _ILU_C_FinishServingCall(_call, _err);\n");
   if (t != void_Type)
     freeReturnValue(m->returnType, context);
@@ -403,19 +413,7 @@ static void classTable (Type class, Context context)
        (od = class_object (class)) == NULL)
     return;
   if  (list_size (od->methods) > 0)
-    list_enumerate (od->methods, (void (*)(refany, refany)) serverMethods, context);
-}
-
-static void generateProto (Type  class, Context  context)
-{
-  Class  od;
-
-  if  (class == NULL || 
-       type_basic_type (class) != object_Type || 
-       (od = class_object (class)) == NULL)
-    return;
-  if  (list_size (od->methods) > 0)
-    list_enumerate (od->methods, (void (*)(refany, refany)) serverProto, context);
+    list_enumerate (od->methods, serverMethods, context);
 }
 
 
@@ -431,7 +429,6 @@ static void generateServerMiMethodPtrs(refany elt, refany rock)
   if (methodInList(c_type_name(ut)))
     return;
   addMethodToList(c_type_name(ut));
-  generateProto(ut, context);
   fprintf(context->file,
   "static struct _%s__MethodBlock_s _%s_%s__TrueMethodBlock = {\n",
 	  c_type_name(ut), c_type_name(context->class),
@@ -605,8 +602,6 @@ void generateServerCode (Interface parse, FILE *file)
   pc_interfacename = interface_header_name(c_interface_name(parse));
 
   fprintf(file, "#include <stdio.h>\n\n");
-  fprintf(file, "#define _%s_TRUE_STUB_CODE_\n\n",
-	  c_interface_name(parse));
   fprintf(file, "#include \"%s.h\"\n\n", pc_interfacename);
 
   list_enumerate(parse->classes, generateServerClassTable, &context);
