@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1991, 1992, 1993 Xerox Corporation.  All Rights Reserved.  
+Copyright (c) 1991-1996 Xerox Corporation.  All Rights Reserved.  
 
 Unlimited use, reproduction, and distribution of this software is
 permitted.  Any copy of this software must include both the above
@@ -13,18 +13,16 @@ PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM
 THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
+
+$Id: common.c,v 1.45 1996/06/21 16:15:16 spreitze Exp $
 */
-/* $Id: common.c,v 1.41 1996/02/14 01:47:17 janssen Exp $ */
-/* Last edited by Mike Spreitzer December 5, 1995 4:19 pm PST */
+/* Last edited by Mike Spreitzer June 21, 1996 9:14 am PDT */
 
 #ifdef MACOS
 #pragma segment common
 #endif
 
-#ifdef WIN32
-// dll - added to get declaration for exit(int)
 #include <stdlib.h>
-#endif
 
 #include "cplusplus.h"
 
@@ -124,8 +122,7 @@ boolean HasFreeRoutine (Type type)
     {
       return (TRUE);
     }
-  else if (tk == optional_Type AND
-	   (HasFreeRoutine(type_description(type)->structuredDes.optional)))
+  else if (tk == optional_Type)
     {
       return (TRUE);
     }
@@ -210,7 +207,7 @@ void SizeType (Type type, string name, Context context)
 	     (type_ur_kind(d->structuredDes.sequence.type)
 	      == shortcharacter_Type))
     MarshS(context,
-	   "ilu::SizeOfString(_call, ((char *)%s), strlen((char *) %s), %lu)",
+	   "ilu::SizeOfString(_call, %s, strlen(%s), %lu)",
 	   name, name, d->structuredDes.sequence.limit);
   else if (t == array_Type
 	   && type_ur_kind(d->structuredDes.array.type) == byte_Type
@@ -338,7 +335,7 @@ void EncodeValue (Type type, string name, Context context)
   else if (t == character_Type)
     SimpleOut(context, "Character", "(ilu_Character)", name);
   else if (t == shortcharacter_Type)
-    SimpleOut(context, "Byte", "(unsigned char)", name);
+    SimpleOut(context, "Byte", "(ilu_ShortCharacter)", name);
   else if (t == real_Type)
     SimpleOut(context, "Real", "(double)", name);
   else if (t == shortreal_Type)
@@ -439,7 +436,7 @@ void UnpackValue (Context context, Type type, LineNumber line, string buffer, bo
   else if (t == character_Type)
     fprintf (context->file, "\tilu::InputCharacter (_call, %s%s);\n", ref ? "" : "&", buffer);
   else if (t == shortcharacter_Type)
-    fprintf (context->file, "\tilu::InputByte (_call, %s%s);\n", ref ? "" : "&", buffer);
+    fprintf (context->file, "\tilu::InputShortCharacter (_call, %s%s);\n", ref ? "" : "&", buffer);
   else if (t == real_Type)
     fprintf (context->file, "\tilu::InputReal (_call, %s%s);\n", ref ? "" : "&", buffer);
   else if (t == shortreal_Type)
