@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1991, 1992, 1993 Xerox Corporation.  All Rights Reserved.  
+Copyright (c) 1991-1996 Xerox Corporation.  All Rights Reserved.  
 
 Unlimited use, reproduction, and distribution of this software is
 permitted.  Any copy of this software must include both the above
@@ -13,16 +13,11 @@ PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM
 THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
-*/
-/* Last edited by Mike Spreitzer December 7, 1995 11:15 pm PST */
 
-/*
-$Id: headers.c,v 1.49 1996/02/14 19:35:42 janssen Exp $
+$Id: headers.c,v 1.50 1996/05/29 19:00:59 janssen Exp $
 */
-#ifdef WIN32
-// dll - added to get declaration for exit(int)
+
 #include <stdlib.h>
-#endif
 
 #include "cplusplus.h"
 
@@ -151,8 +146,6 @@ static void GenerateRecordField (Argument a, Context context)
 
 static void GenerateRecordDeclaration (Type t, Context context)
 {
-  TypeDescription type_description();
-
   fprintf (context->file, "struct _%s_record {\n", cplusplus_type_name(t));
   list_enumerate (type_description(t)->structuredDes.record, (iluparser_EnumProc) GenerateRecordField, context);
   fprintf (context->file, "};\n");
@@ -360,8 +353,6 @@ static void GenerateSequenceTypedef (Type t, Context context)
 
 static void GenerateRecordTypedef (Type t, Context context)
 {
-  TypeDescription type_description();
-
   fprintf (context->file, "typedef struct _%s_record %s;\n", cplusplus_type_name(t), cplusplus_type_name(t));
 }
 
