@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: be_gen_isl.cc,v 2.6 1994/11/04 20:17:31 severson Exp $";
+static const char rcsid[] = "$Id: be_gen_isl.cc,v 2.8 1996/04/30 06:33:07 janssen Exp $";
 
 /*
  *======================================================================
@@ -32,6 +32,7 @@ static const char attributeGenericPrefix[]	= "ilu--prefix-idlAttribute-";
 static const char attributeGetPrefix[]		= "-get-";
 static const char attributeSetPrefix[]		= "-set-";
 static const char qualifiedNameSeparator[]	= "-";
+static const char repIDSeparator[]		= "/";
 
 static void
 bePrefixMessage()
@@ -1345,6 +1346,21 @@ be_interface::pregen_types(ostream &)
 	// empty
 }
 
+static void dumpStringList (ostream &o, UTL_StrList *p)
+{
+  UTL_StrlistActiveIterator *i;
+  if (p != NULL)
+    {
+      i = new UTL_StrlistActiveIterator(p);
+      while (!i->is_done())
+	{
+	  o << "  <" << i->item()->get_string() << ">\n";
+	  i->next();
+	}
+      delete(i);
+    }
+}
+
 void
 be_interface::gen_isl(ostream &o)
 {
@@ -1371,6 +1387,7 @@ be_interface::gen_isl(ostream &o)
 		}
 		o << " END";
 	}
+	this->gen_type_id(o);
 
 	genIslForScope(this, o, "\n\tMETHODS\n\t\t", methodSeparator,
 		"\n\tEND", NTF_attr_and_op);
@@ -1383,6 +1400,119 @@ be_interface::gen_type_name(ostream &o)
 	printQualifiedDeclName(o, this);
 }
 
+static char *
+qualifiedRepIDName(AST_Decl *d)
+{
+	UTL_IdListActiveIterator iterator(d->name());
+	char buffer[1024];
+	char *dst = buffer;
+
+	if (iterator.item()->get_string()[0] == 0)
+		iterator.next();
+	else
+		beInternalError("qualifiedRepIDName: first item not null");
+
+	if (beGlobals.opts.useImports && !isInCurrentIslInterface(d))
+	{
+		if (beGlobals.imports)
+			beGlobals.imports->add_import(d);
+		if (beGlobals.opts.islInterfacePerTopIdlModule)
+		{
+			dst = stringCopy(dst, iterator.item()->get_string());
+			iterator.next();
+		}
+		else
+		{
+			char *importedName = filePrefixOfDecl(d);
+
+			dst = stringCopy(dst, importedName);
+			delete importedName;
+		}
+		*dst++ = '/';
+	}
+	else if (beGlobals.opts.islInterfacePerTopIdlModule)
+		iterator.next();
+
+	for (int count = 0; !iterator.is_done(); count += 1, iterator.next())
+	{
+		if (count > 0)
+			dst = stringCopy(dst, repIDSeparator);
+		dst = stringCopy(dst, iterator.item()->get_string());
+	}
+	*dst = 0;
+	return translatedName(newString(buffer));
+}
+
+char *
+be_interface::repository_id ()
+{
+  if (this->_repository_id == NULL)
+    {
+      UTL_StrList *l = this->pragmas();
+      if (l == NULL)
+	{
+	  char buf[1024];
+	  sprintf (buf, "IDL:%s:1.0", qualifiedRepIDName(this));
+	  this->_repository_id = newString(buf);
+	}
+      else
+	{
+	  /* look at pragmas to see if there's anything we need
+	    from there (id itself, or prefix, or version) */
+	  char *prefix = NULL;
+	  char *id = NULL;
+	  char *version = NULL;
+	  char *s;
+	  char *this_type = this->local_name()->get_string();
+	  UTL_StrlistActiveIterator *i = new UTL_StrlistActiveIterator(l);
+	  while (!i->is_done())
+	    {
+	      s = i->item()->get_string();
+	      if (strncmp(s, "#pragma prefix ", strlen("#pragma prefix ")) == 0)
+		{
+		  static char buf[1024];
+		  if (sscanf(s + strlen("#pragma prefix "), "\"%[^\n\"]\"", buf) == 1)
+		    prefix = buf;
+		}
+	      else if (strncmp(s, "#pragma ID ", strlen("#pragma ID ")) == 0)
+		{
+		  static char buf[1024];
+		  static char type[1024];
+		  if ((sscanf(s + strlen("#pragma ID "), "%s \"%[^\n\"]\"", type, buf) == 2)
+		      && (strcmp(type, this_type) == 0))
+		    id = buf;
+		}
+	      else if (strncmp(s, "#pragma version ", strlen("#pragma version ")) == 0)
+		{
+		  static char buf[16];
+		  if (sscanf(s + strlen("#pragma version "), "%s", buf) == 1)
+		    version = buf;
+		}
+	      i->next();
+	    }
+	  if (id != NULL)
+	    this->_repository_id = newString(id);
+	  else
+	    {
+	      char buf[1024];
+	      sprintf (buf, "IDL:%s%s%s:%s",
+		       (prefix == NULL) ? "" : prefix,
+		       (prefix == NULL) ? "" : "/",
+		       qualifiedRepIDName(this),
+		       (version == NULL) ? "1.0" : version);
+	      this->_repository_id = newString(buf);
+	    }
+	}
+    }
+  return this->_repository_id;
+}
+
+void
+be_interface::gen_type_id (ostream &o)
+{
+  o << " TYPEID \"" << this->repository_id() << "\"";
+}
+
 idl_bool
 be_interface::gen_isl_typedef(ostream &, AST_Decl *)
 {
