@@ -14,7 +14,7 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: pprint.c,v 1.2 1995/10/26 06:03:23 janssen Exp $
+$Id: pprint.c,v 1.3 1996/06/21 01:48:23 janssen Exp $
 */
 
 #include <stdio.h>
@@ -504,24 +504,80 @@ static void PrintConstant (Constant c)
     }
   else
     {
-      if (c->value->type == integer_Type)
-	sprintf (buf, "= %s%lu;", (c->value->val.i.sign < 0) ? "-" : "", c->value->val.i.value);
-      else if (c->value->type == real_Type && c->value->val.r.fraction == NULL)
-	sprintf (buf, "= %s%se%lu;", (c->value->val.i.sign < 0) ? "-" : "", c->value->val.r.value, c->value->val.r.exponent);
-      else if (c->value->type == real_Type && c->value->val.r.fraction != NULL)
-	sprintf (buf, "= %s%s.%se%lu;", (c->value->val.i.sign < 0) ? "-" : "", c->value->val.r.value,
-		c->value->val.r.fraction, c->value->val.r.exponent);
-      else if (c->value->type == shortcharacter_Type)
+      TypeKind ctk, vtk;
+      char *ctypename = name_base_name(c->name);
+
+      ctk = type_ur_kind(c->type);
+      vtk = c->value->type;
+
+      switch (ctk)
 	{
-	  string s = pprint_string(c->value->val.s, TRUE);
-	  sprintf (buf, "= \"%s\";", s);
-	  free(s);
+	case shortinteger_Type:
+	case integer_Type:
+	case longinteger_Type:
+	  if (vtk != integer_Type)
+	    fprintf (stderr, "\n*** Constants of type \"%s\" must have an associated integer value.\n", ctypename);
+	  else
+	    sprintf (buf, "= %s%lu", (c->value->val.i.sign < 0) ? "-" : "", c->value->val.i.value);
+	  break;
+
+	case byte_Type:
+	case shortcardinal_Type:
+	case cardinal_Type:
+	case longcardinal_Type:
+	  if (vtk != integer_Type)
+	    fprintf (stderr, "\n*** Constants of type \"%s\" must have an associated integer value.\n", ctypename);
+	  else if (c->value->val.i.sign < 0)
+	    fprintf (stderr, "\n*** Constants of type \"%s\" may not be negative.\n", ctypename);
+	  else
+	    printf ("= %lu", c->value->val.i.value);
+	  break;
+
+	case real_Type:
+	case shortreal_Type:
+	case longreal_Type:
+	  if (vtk != real_Type)
+	    fprintf (stderr, "\n*** Constants of type \"%s\" must have an associated real value.\n", ctypename);
+	  else if (c->value->val.r.fraction == NULL)
+	    printf ("= %s%se%lu", (c->value->val.i.sign < 0) ? "-" : "", c->value->val.r.value, c->value->val.r.exponent);
+	  else
+	    printf ("= %s%s.%se%lu", (c->value->val.i.sign < 0) ? "-" : "", c->value->val.r.value,
+		     c->value->val.r.fraction, c->value->val.r.exponent);
+	  break;
+
+	case boolean_Type:
+	  if (vtk != boolean_Type)
+	    fprintf (stderr, "\n*** Constants of type \"%s\" must have an associated boolean value.\n", ctypename);
+	  else
+	    printf ("= %s", c->value->val.b ? "True" : "False");
+	  break;
+
+	case sequence_Type:
+	  if (vtk != shortcharacter_Type)
+	    fprintf (stderr, "\n*** Constants of type \"%s\" must have an associated string value.\n", ctypename);
+	  else
+	    {
+	      string s = pprint_string(c->value->val.s, TRUE);
+	      sprintf (buf, "= \"%s\";", s);
+	      free(s);
+	    }
+	  break;
+
+	case enumeration_Type:
+	  if (vtk != shortcharacter_Type)
+	    fprintf (stderr, "*** Constants of type \"%s\" must have an associated string value.\n", ctypename);
+	  else
+	    {
+	      string s = pprint_string(c->value->val.s, TRUE);
+	      sprintf (buf, "= %s;", s);
+	      free(s);
+	    }
+	  break;
+
+	default:
+	  fprintf (stderr, "\n*** Constants of type \"%s\" not allowed.\n", ctypename);
+	  break;
 	}
-      else if (type_basic_type(c->type) == boolean_Type && (c->value->type == boolean_Type))
-	sprintf (buf, "= %s;", c->value->val.b ? "TRUE" : "FALSE");
-      else {
-	fprintf (stderr, "\n*** Bad constant \"%s\" encountered.", name_base_name(c->name));
-      }
     }
   len += strlen(buf);
   pad = (len < CommentCol) ? (CommentCol - len) : 0;
