@@ -14,9 +14,10 @@ THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING IN
 CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF
 XEROX CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 
-$Id: genstub.c,v 1.41 1996/03/05 03:50:15 janssen Exp $
+$Id: genstub.c,v 1.48 1996/07/03 19:23:11 janssen Exp $
 */
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include "iluptype.h"
@@ -113,7 +114,7 @@ stConstantValue(ConstantValue v, Type t)
 		    Type t2 = type_description(t)->structuredDes.uniond.discriminator_type;
 		    if (t2->importInterfaceName != NULL)
 		      printf ("%s.", getInterfaceName(ur_type(t2)->interface));
-		    printf ("%s__%s", getTypeName(ur_type(t2)), p);
+		    printf ("%s.%s", getTypeName(ur_type(t2)), p);
 		    free(p);
 		  }
 		break;
@@ -375,6 +376,26 @@ stDefineRecordTypeOutSize(Type t, const char *prefix)
 		printf("    return %s\n", nameVarSize);
 }
 
+static void
+listFieldParam (Argument f, char *prefix)
+{
+  printf ("%s%s", prefix, getArgumentName(f));
+}
+
+static void
+listPickleFieldParam (Argument f, char *prefix)
+{
+  printf ("self.%s, ", getArgumentName(f));
+}
+
+static void
+initRecordField (Argument f, void *junk)
+{
+  listFieldParam (f, "        self.");
+  listFieldParam (f, " = _");
+  printf (";\n");
+}
+
 static void
 stDefineRecordTypeClass(Type t)
 {
@@ -384,38 +405,24 @@ stDefineRecordTypeClass(Type t)
   newline();
   printf ("class %s (iluRt.IluRecord):\n", getTypeName(t));
   printf ("    __ilu_type_name__ = '%s.%s'\n",
-	  getInterfaceName(t->interface), getTypeName(t));
-}
-
-static void
-listFieldParam (Argument f, char *prefix)
-{
-  printf ("%s%s", prefix, getArgumentName(f));
-}
-
-static void
-initRecordField (Argument f, void *junk)
-{
-  listFieldParam (f, "        self.");
-  printf (" = None\n");
-}
-
-static void
-stDefineRecordTypeClassInit(Type t)
-{
-  TypeDescription	d	= t->description;
-  list			fields	= d->structuredDes.record;
-
-  newline();
-  printf ("    def __init__(self, *args, **dict):\n");
+	  interface_name(t->interface), type_name(t));
+  printf ("    def __init__(self");
+  list_enumerate(fields, (EnumProc) listFieldParam, ", _");
+  printf ("):\n");
   list_enumerate(fields, (EnumProc) initRecordField, 0);
+
+  printf ("\n");
+  printf ("    def __getinitargs__(self):\n");
+  printf ("      return (");
+  list_enumerate(fields, (EnumProc) listPickleFieldParam, 0);
+  printf (")");
+  printf ("\n");
 }
 
 static void
 stRecordType(Type t)
 {
   stDefineRecordTypeClass(t);
-  stDefineRecordTypeClassInit(t);
   stDefineRecordTypeInput(t);
   stDefineRecordTypeOutSize(t, prefixFuncOutput);
   stDefineRecordTypeOutSize(t, prefixFuncSizeOf);
@@ -983,7 +990,7 @@ stMethodGetReply(Procedure m)
 	{
 		int	count	= 0;
 
-		printf("\t    return ");
+		printf("\t    %s = ", nameVarResult);
 		if (m->returnType)
 		{
 			++count;
@@ -1004,6 +1011,8 @@ stMethod(Procedure m)
 	printf("    def %s(%s", getProcedureName(m), nameVarSelf);
 	printArgList(m->arguments, 1);
 	printf("):\n");
+	if (methodResultCount(m) > 0)
+	  printf("\t%s = None\n", nameVarResult);
 
 #if (ILUPYTHON_MAJOR_VERSION >= 1 && ILUPYTHON_MINOR_VERSION >= 2)
 
@@ -1032,10 +1041,13 @@ stMethod(Procedure m)
 	printf("\ttry:\n");
 	stMethodSendRequest(m);
 	if (m->asynch == 0)
-		stMethodGetReply(m);
-	if (m->asynch == 0)
-	  printf("\t    %s.ReplyRead(%s)\n", nameModuleIlu, nameVarCall);
+	  {
+	    stMethodGetReply(m);
+	    printf("\t    %s.ReplyRead(%s)\n", nameModuleIlu, nameVarCall);
+	  }
 	printf("\tfinally:\n\t    %s.FinishCall(%s)\n", nameModuleIlu, nameVarCall);
+	if (methodResultCount(m) > 0)
+	  printf("\treturn %s\n", nameVarResult);
 }
 
 static void
@@ -1117,14 +1129,16 @@ stCatchException(Exception e, int *pCount)
 {
 	Type	et	= e->import ? e->import->type : e->type;
 
+	printf("    %sif %s == %s:\n", ++*pCount > 1 ? "el" : "",
+	       nameVarExceptName, getExceptionName(e));
+	printf("\t%s = %s", nameVarExceptName, getExceptionName(e));
+	newline();
 	if (et)
-	{
-		printf("    %sif %s is %s:\n", ++*pCount > 1 ? "el" : "",
-			nameVarExceptName, getExceptionName(e));
-		printf("\t%s = ", nameVarExceptValue);
-		ioTypeInput(et);
-		newline();
-	}
+	  {
+	    printf("\t%s = ", nameVarExceptValue);
+	    ioTypeInput(et);
+	    newline();
+	  }
 }
 
 static void
@@ -1135,11 +1149,10 @@ stFuncCatchExceptions(list exceptions)
 	newline();
 	printf("def %s(%s, %s, %s):\n", nameFuncCatchExcept,
 		nameVarCall, nameVarClass, nameVarExceptCode);
+	printf("    %s = None\n", nameVarExceptValue);
 	printf("    %s = %s.ExceptionName(%s, %s, %s)\n", nameVarExceptName,
 		nameModuleIlu, nameVarCall, nameVarClass, nameVarExceptCode);
 	list_enumerate(exceptions, (EnumProc) stCatchException, &nExceptions);
-	printf("    %s%s = None\n", nExceptions > 0 ? "else:\n\t" : "",
-		nameVarExceptValue);
 	printf("    raise %s, %s\n", nameVarExceptName, nameVarExceptValue);
 }
 
@@ -1154,6 +1167,7 @@ generateStub(Interface ifc)
 	currentIfc = ifc;
 	ifcName = interface_name(ifc);
 	printBanner("Stubs", ifc);
+	printImportTable();
 	list_enumerate(ifc->imports,    (EnumProc) stImported,  NULL);
 	list_enumerate(ifc->constants,  (EnumProc) stConstant,  &nConstants);
 	list_enumerate(ifc->exceptions, (EnumProc) stException, &nExceptions);
