@@ -26,6 +26,10 @@ Dave Maxey - 1991
 
 */
 
+/* disable expected warning messages about inconsistent dll linkage */
+#if (defined(_MSC_VER) && defined (WIN32))
+#pragma warning( disable : 4273 )
+#endif
 
 /* PROTOTYPES in alphabetic order */
 
